package dk.sosi.sts.integrations.blurring;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.nio.charset.UnsupportedCharsetException;

public class BlurringServiceCaller {
    private static final Logger logger = Logger.getLogger(BlurringServiceCaller.class);

    private final CloseableHttpClient httpClient = HttpClients.createDefault();

    private String lookupEndpoint;
    private String saltEndpoint;

    // Used by Spring
    public void setLookupEndpoint(String endpoint) { this.lookupEndpoint = endpoint; }
    public void setSaltEndpoint(String endpoint) { this.saltEndpoint = endpoint; }

    public String callGetBlurredOrganisations(String request) {
        return httpWrite(lookupEndpoint, SOAPOperation.GetBlurredOrganisations.getSoapOperation(), request);
    }

    public String callListActiveOrgBlurrings(String request) {
        return httpWrite(lookupEndpoint, SOAPOperation.ListAllActiveOrgBlurrings.getSoapOperation(), request);
    }

    public String callSaltService(String request) {
        return httpWrite(saltEndpoint,
                         "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/#GetCurrentSalt",
                         request);
    }

    private String httpWrite(String endpoint, String soapAction, String xml) {
        logger.debug("Calling " + endpoint + ", SOAPAction=" + soapAction);
        HttpPost request = httpRequest(endpoint, soapAction, xml);

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseString;
            try {
                responseString = EntityUtils.toString(response.getEntity(), "UTF-8");
            } catch (IOException e) {
                logger.error("Failed to parse response from blurring service. HTTP response code was " + statusCode, e);
                throw new RuntimeException(BlurringService.errorMessage);
            }
            if (statusCode != 200) {
                logger.error("Blurring service at '" + endpoint + "' returned HTTP response code " +
                        statusCode + ". Response was '" + responseString + "'");
                throw new RuntimeException(BlurringService.errorMessage);
            }
            return responseString;
        } catch (IOException e) {
            logger.error("Call to burring service failed.", e);
            throw new RuntimeException(BlurringService.errorMessage);
        }
    }

    private HttpPost httpRequest(String endpoint, String soapAction, String xml) {
        HttpPost request = new HttpPost(endpoint);
        request.addHeader("SOAPAction", "\"" + soapAction + "\"");

        StringEntity stringEntity;
        try { stringEntity = new StringEntity(xml, "UTF-8"); }
        catch (UnsupportedCharsetException e) {
            logger.error("Unsupported encoding used", e);
            throw new RuntimeException(BlurringService.errorMessage);
        }
        stringEntity.setContentType("text/xml");
        request.setEntity(stringEntity);
        return request;
    }
}