package dk.sosi.sts.integrations.blurring;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.CareProvider;
import dk.sosi.seal.model.SystemIDCard;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;

import java.time.Instant;
import java.util.Date;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.function.Function;

public class IdCardFactory {
    private final SOSIFactory sosiFactory;
    private String itSystemName, careProviderId, careProviderOrg;

    private final ConcurrentLinkedQueue<SystemIDCard> pool = new ConcurrentLinkedQueue<>();

    // injected by spring
    public void setItSystemName(String itSystemName) { this.itSystemName = itSystemName; }
    public void setCareProviderId(String careProviderId) { this.careProviderId = careProviderId; }
    public void setCareProviderOrg(String careProviderOrg) { this.careProviderOrg = careProviderOrg; }

    public IdCardFactory(SOSIFactory sosiFactory) {
        this.sosiFactory = sosiFactory;
    }

    public <R> R applyIdCard(Function<SystemIDCard, R> consumer) {
        SystemIDCard idCard = pool.poll();
        if (renewalNeeded(idCard)) {
            CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, careProviderId, careProviderOrg);
            AuthenticationLevel authenticationLevel = AuthenticationLevel.VOCES_TRUSTED_SYSTEM;
            idCard = sosiFactory.createNewSystemIDCard(itSystemName, careProvider, authenticationLevel, null, null, null, null);
        }
        R result = consumer.apply(idCard);
        pool.add(idCard);
        return result;
    }

    private boolean renewalNeeded(SystemIDCard idCard) {
        return idCard == null || Date.from(Instant.now().plusSeconds(300)).after(idCard.getExpiryDate());
    }
}