/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.server.facade.jwt;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import java.nio.charset.Charset;
import java.time.Duration;
import java.time.Instant;
import java.util.Base64;
import java.util.Date;

public class JWTTokenBuilder {
    private final String cpr;
    private final String subjectDN;
    private final FakeIdp idp;

    // should contain sosi-sts, and a text configured to match the intended audience
    private String scope = "sosi-sts clear";

    private String issuer = "http://sts-tester"; // issuer should be known (configured) inside STS

    private Date expiration = new Date(Instant.now().plus(Duration.ofHours(2)).toEpochMilli());

    public JWTTokenBuilder(FakeIdp idp, String subjectDN, String cpr) {
        this.idp = idp;
        this.subjectDN = subjectDN;
        this.cpr = cpr;
    }

    public String build() {
        final JwtBuilder builder = Jwts.builder();
        builder.signWith(SignatureAlgorithm.RS256, idp.getPrivateKey());
        builder.setHeaderParam("kid", idp.getKeyIdentifier());
        final Claims claims = Jwts.claims();
        claims.setIssuer(issuer);
        if (subjectDN != null) {
            claims.put("certsubdn", Base64.getMimeEncoder().encodeToString(subjectDN.getBytes(Charset.forName("UTF-8"))));
        }
        claims.put("scope", scope);
        if (cpr != null) {
            claims.put("cpr", cpr);
        }
        claims.setExpiration(expiration);
        builder.setClaims(claims);
        return builder.compact();
    }
}
