/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.fuldmagt;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

public class Standalone {
    public static final String NAMESPACE = "http://schemas.datacontract.org/2004/07/DK.OES.KFOBS.Delegation.Frontend.DelegationWebService";

    public static void main(String[] args) throws MalformedURLException {
        String keystore = System.getProperty("javax.net.ssl.keyStore");
        if (!new File(keystore).exists()) {
            throw new RuntimeException("Keystore not found " + keystore);
        }
        //System.setProperty("javax.net.ssl.keyStore", keystore);
        //System.setProperty("javax.net.ssl.keyStorePassword", "Test1234");

        Standalone standalone = new Standalone();
        String endpoint = System.getProperty("endpoint");//, "https://delegationquery.fuldmagt.test-nemlog-in.dk/QueryWebService.svc");
        if (endpoint == null) {
            throw new RuntimeException("endpoint missing");
        }
        standalone.endpoint = new URL(endpoint);

        standalone.init();
        String ssn = System.getProperty("pid", "PID:9208-2002-2-514358910503");
        standalone.listProcurationPrivileges(ssn);
    }

    private IQueryWebService port;
    private URL endpoint;
    private String entityId = "https://saml.sts.nspop.dk";

    /* invoked by spring */
    public void init() {
        final URL wsdlResource = getClass().getClassLoader().getResource("dqws.wsdl");
        final IQueryWebService port = new QueryWebService(wsdlResource).getBasicHttpBindingIQueryWebService();
        BindingProvider bindingProvider = (BindingProvider) port;
        bindingProvider.getRequestContext().put(BindingProvider.SOAPACTION_USE_PROPERTY, Boolean.TRUE);
        bindingProvider.getRequestContext().put(BindingProvider.SOAPACTION_URI_PROPERTY, "https://DelegationQuery.Nemlog-in.dk/IQueryWebService/GetDelegations");
        bindingProvider.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpoint.toExternalForm());
        this.port = port;
    }

    public void listProcurationPrivileges(String ssn) {
        RepresentativeId representativeId = new RepresentativeId();
        QName representativeQname = new QName(NAMESPACE, "PID");
        representativeId.setPID(new JAXBElement<>(representativeQname, String.class, ssn.substring("PID:".length())));
        final DelegationResponse response = getDelegations(representativeId);
        final List<Delegation> delegations = response.getDelegations().getValue().getDelegation();
        System.out.println("Procurations to PID: " + ssn);
        for (Delegation delegation : delegations) {
            System.out.println("...Procurations found from " + delegation.getCitizenCpr().getValue() + " " + delegation.getPrivileges().getValue().getString().size());
        }
    }

    private DelegationResponse getDelegations(RepresentativeId representativeId) {
        try {
            return port.getDelegations(entityId, representativeId);
        } catch (IQueryWebServiceGetDelegationsQueryServiceFaultFaultFaultMessage e) {
            final QueryServiceFault faultInfo = e.getFaultInfo();
            final String errorDetails = getErrorDetails(faultInfo);
            throw new RuntimeException("Error from procurationservice: " + errorDetails, e);
        }
    }

    private String getErrorDetails(QueryServiceFault faultInfo) {
        if (faultInfo == null) {
            return "(no faultinfo)";
        }
        String errorCode = faultInfo.getErrorCode() != null ? faultInfo.getErrorCode().value() : "(null)";
        String errorMessage = faultInfo.getMessage() != null ? faultInfo.getMessage().getValue() : "(null)";
        return "code=" + errorCode + " message=" + errorMessage;
    }

}
