/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ibo;

import org.apache.log4j.Logger;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.sql.DataSource;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

public class DBIboConfigChecker {

	private Logger log = Logger.getLogger(DBIboConfigChecker.class);

	private JdbcTemplate jdbcTemplate;

	private String selectAudiences = "select audience from iboConfig";
	private String updateAudiences = "update iboConfig set audience = ? where audience = ?";

	/**
	 * Updates the database so that audiences are normalized.
	 */
	public void normalizeAudiences() {
		try {
			final List<String> audiences = jdbcTemplate.queryForList(selectAudiences,
																	 new Object[]{},
																	 String.class);
			log.info("Normalizing IBO audience configuration.");
			jdbcTemplate.batchUpdate(updateAudiences,
									 new BatchPreparedStatementSetter() {
										 @Override
										 public void setValues(PreparedStatement ps, int i) throws SQLException {
											 final String audience = audiences.get(i);
											 final String normalized = IboConventions.normalizeAudience(audience);
											 ps.setString(1, normalized);
											 ps.setString(2, audience);
										 }

										 @Override
										 public int getBatchSize() {
											 return audiences.size();
										 }
									 });
		} catch (Throwable t) {
			log.warn("Could not normalize the IBO audience configuration in the database due to ", t);
		}

	}

	/**
	 * Used by to wire the object -- mandatory.
	 */
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Used by to wire the object -- optional.
	 */
	public void setSelectAudiences(final String selectAudiences) {
		this.selectAudiences = selectAudiences;
	}

	/**
	 * Used by to wire the object -- optional.
	 */
	public void setUpdateAudiences(final String updateAudiences) {
		this.updateAudiences = updateAudiences;
	}
}

