/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ibo;

import junit.framework.TestCase;

public class IboConventionsTest extends TestCase {

	public void testNull() {
		assertNull(IboConventions.normalizeAudience(null));
	}

	public void testInvalidURI() {
		assertNull(IboConventions.normalizeAudience("not legal uri"));
	}

	private void check(String audience, String expected) {
		final String result = IboConventions.normalizeAudience(audience);
		assertNotNull(result);

		assertEquals(expected, result);
	}

	public void testValidWithPath() {
		check("http://sundhed.dk/", "http://sundhed.dk");
		check("http://SUNDHED.DK/", "http://sundhed.dk");
		check("HTTP://sundhed.dk/", "http://sundhed.dk");
		check("http://sundhed.dk/this/is/login", "http://sundhed.dk/this/is/login");
		check("http://sundhed.dk/this/IS/login", "http://sundhed.dk/this/IS/login");
	}

	public void testValidWithoutPath() {
		check("http://sundhed.dk", "http://sundhed.dk");
	}

	public void testQuery() {
		check("http://sundhed.dk?login=true", "http://sundhed.dk?login=true");
		check("http://sundhed.dk?login=true#nowhere", "http://sundhed.dk?login=true#nowhere");
	}

	public void testPort() {
		check("http://sundhed.dk:8080?login=true", "http://sundhed.dk:8080?login=true");
	}

	public void testMisc() {
		check("//sundhed.dk.nu/hello", "//sundhed.dk.nu/hello");
		check("/hello", "/hello");
		check("arosii:/hello", "arosii:/hello");
		check("sundhed.dk", "/sundhed.dk"); // URI parsing gives us a path element although an authority is intuitive
	}

	public void testDefaultBstAudienceAlreadyNormalized() {
		check("https://fmk", "https://fmk");
	}

	public void testAppendingSlashFromDotNetStillGetsSameNormalizedAudience() {
		check("https://fmk/", "https://fmk");
	}


}
