package dk.nsp.idsas.ws._20240124;

import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import org.apache.cxf.annotations.SchemaValidation;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPConstants;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPFault;
import javax.xml.ws.BindingType;
import javax.xml.ws.soap.SOAPFaultException;

import java.util.List;
import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.OrganisationIdClassificationType.*;
import static java.util.stream.Collectors.*;
import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name = "IdsasLookupService2024", urlPatterns = {"/20240124/lookup"})
@WebService(serviceName = "IdsasLookupService2024", wsdlLocation = "wsdl/20240124/idsas-lookup.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2024/01/24/",
        portName = "IdsasLookupServicePort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
public class IdsasLookupServiceWS implements IdsasLookupServicePortType {

    @Override
    public BlurredOrganisationsResponse getBlurredOrganisations(GetBlurredOrganisationsRequest u) {
        if (!u.getPatientId().getId().equals("2406139002")) { throw mockSoapFault(); }
        return blurredOrganisationsResponse("33257872");
    }

    @Override
    public ListOrgBlurringsForCVRResponse listOrgBlurringsForCVR(ListOrgBlurringsForCVRRequest listOrgBlurringsForCVRRequest) {
        return null; // Not needed by the STS
    }

    @Override
    public ListAllActiveOrgBlurringsResponse listAllActiveOrgBlurrings(ListAllActiveOrgBlurringsRequest listAllActiveOrgBlurringsRequest) {
        List<OrganisationIdType> orgs = Stream.of(organisationIdType("650901000016000", SOR),
                                                  organisationIdType("7042010", SHAK)).collect(toList());

        ListAllActiveOrgBlurringsResponse response = new ListAllActiveOrgBlurringsResponse();
        response.getOrg().addAll(orgs);
        return response;
    }

    private static OrganisationIdType organisationIdType(String id, OrganisationIdClassificationType classification) {
        OrganisationIdType e = new OrganisationIdType(); e.setId(id); e.setClassification(classification);
        return e;
    }

    static BlurredOrganisationsResponse blurredOrganisationsResponse(String id) {
        OrganisationIdType e = new OrganisationIdType(); e.setId(id); e.setClassification(CVR);
        BlurredOrganisationsResponse r = new BlurredOrganisationsResponse(); r.getOrganisationId().add(e);
        return r;
    }

    static SOAPFaultException mockSoapFault() {
        try {
            SOAPFault fault = SOAPFactory.newInstance(SOAPConstants.SOAP_1_1_PROTOCOL)
                    .createFault("Mock error", new QName(SOAPConstants.URI_NS_SOAP_1_1_ENVELOPE, "Client"));
            fault.addDetail()
                    .addDetailEntry(new QName("http://sundhedsdatastyrelsen.dk/identitetssloering/", "FaultCode"))
                    .addTextNode("Mock error");
            return new SOAPFaultException(fault);
        } catch (Exception e) { throw new RuntimeException("Error generating SOAPFault", e); }
    }
}