/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.its;

import dk.sosi.sts.model.Configuration;
import dk.sosi.sts.model.AudienceConfiguration;
import dk.sosi.sts.model.AudienceConfigurationBuilder;
import dk.sosi.sts.service.ItsConfigService;
import dk.sosi.sts.util.DbUtil;

import javax.sql.DataSource;

import org.apache.log4j.Logger;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

public class DBItsConfigService implements ItsConfigService {

    private static final String DEFAULT_SELECT = "SELECT attribute, attribute_value FROM audienceConfiguration WHERE audience = ?";
	private final Logger log = Logger.getLogger(DBItsConfigService.class);
	
    private final DataSource dataSource;
    private final String select;
	private final long fuzzyTime;

	public DBItsConfigService(DataSource dataSource, long fuzzyTime) {
		this(dataSource, 
				fuzzyTime,
				DEFAULT_SELECT);
    }

	public DBItsConfigService(DataSource dataSource, long fuzzyTime, String select) {
        this.dataSource = dataSource;
		this.select = select;
		this.fuzzyTime = fuzzyTime;
    }

    public AudienceConfiguration getAudienceConfiguration(String audience) {
    	AudienceConfigurationBuilder<AudienceConfiguration> audienceConfigurationBuilder = new AudienceConfigurationBuilder<AudienceConfiguration>(MappedItsConf.class);
    	return getAudienceConfiguration(audience, audienceConfigurationBuilder);
    }
        
    @Override
    public <E extends AudienceConfiguration> E getAudienceConfiguration(String audience, AudienceConfigurationBuilder<E> builder) {
    	Connection conn = null;
        PreparedStatement pstmt = null;
        Properties result = new Properties();
        try {
            conn = dataSource.getConnection();
			pstmt = conn.prepareStatement(select);
            pstmt.setString(1, audience);

            ResultSet rs = pstmt.executeQuery();
            while (rs.next()) {
                final String attribute = rs.getString("attribute");
                final String value = rs.getString("attribute_value");
                result.put(attribute, value);
            }
            rs.close();
        } catch (SQLException e) {
            throw new RuntimeException("Error checking audienceConfiguration", e);
        } finally {
            DbUtil.close(pstmt, log);
            DbUtil.close(conn, log);
        }

        if (result.isEmpty())
            return null;
        else {
        	return builder
        			.setAudience(audience)
        			.setConfiguration(new Configuration(result))
        			.build();
        }
    }

	public long getFuzzyTime() {
		return fuzzyTime;
	}
}
