/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.its;

import java.security.cert.X509Certificate;

import dk.sosi.sts.model.Configuration;
import dk.sosi.sts.model.SboAudienceConfiguration;
import dk.sosi.sts.util.CertificateUtil;

public class SboAudienceConfigurationImpl extends MappedItsConf implements SboAudienceConfiguration {
    public SboAudienceConfigurationImpl(final String audience, final Configuration conf) {
        super(audience, conf);
    }

	/**
	 * recipientURL
	 * @return
	 */
	public String getRecipientURL() {
		return conf.get("recipientURL");
	}
	
	/**
	 * includeBST
	 */
	public boolean includeBootStrapToken() {
		return conf.getBoolean("includeBST");
	}
	
	/**
	 * publicKey
	 */
	public String getPublicKey() {
		return conf.get("publicKey");
	}
	
	/**
	 * publicKey parsed into X509Certificate
	 */
	public X509Certificate getTargetCertificate() {
		//return iboConfigRef.getTargetCertificate();
		String key = getPublicKey();
		if (key == null) {
			throw new RuntimeException("No publicKey found!");
		}
		return CertificateUtil.parseCertificateConfiguration(key);
	}
}
