/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import java.util.HashMap;
import java.util.Map;

import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.ocescpr.cvrrid.util.ValidateHelper;
import dk.sosi.sts.service.OcesCvrRidService;

public class MockOcesCvrRidService implements OcesCvrRidService {

	protected final Map<SubjectSerialNumber, CprNr> map;
	
	public MockOcesCvrRidService() {
		this(new HashMap<SubjectSerialNumber, CprNr>());
	}
	
	public MockOcesCvrRidService(Map<SubjectSerialNumber, CprNr> map) {
		this.map = map;
	}

	public boolean isRelated(OcesInfo ocesInfo, CprNr cpr) throws StsException {
    final SubjectSerialNumber ssn = ocesInfo.getSubjectSerialNumber();
    ValidateHelper.validate(ssn, cpr);
    return cpr.equals(map.get(ssn));
  }

	public CprNr findRelatedCpr(OcesInfo ocesInfo) throws StsException {
		ValidateHelper.validate(ocesInfo.getSubjectSerialNumber());
		return map.get(ocesInfo.getSubjectSerialNumber());
	}

  public long lastSuccess() {
    return 0;
  }

  public long lastFailure() {
    return 0;
  }

	@Override
	public int getFailedCallsPercentage() { return 0; }

	@Override
	public int getHoursBackInTimeToCalculateFailedCallsPercentage() { return 0; }

	public void add(String ssn, String cpr) {
		add(new SubjectSerialNumber(ssn), new CprNr(cpr));
	}

	public void add(SubjectSerialNumber ssn, CprNr cpr) {
		map.put(ssn, cpr);
	}
}
