package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.service.OcesCvrRidService;
import org.apache.log4j.Logger;

/**
 * This class can be used as a fallback a failing OcesCvrRidService. It will
 * mask errors in the following manner:
 * - findRelatedCpr will return the configured cprNr when StsExceptions are
 *   thrown from the delegate.
 * - isRelated will return true if an StsException was from the delegate
 *
 */
public class WsOcesCvrRidFallbackService implements OcesCvrRidService {

	private static Logger log = Logger.getLogger(WsOcesCvrRidFallbackService.class);

	private OcesCvrRidService delegate;
	private CprNr cprNr;

	@Override
	public CprNr findRelatedCpr(final OcesInfo ocesInfo) throws StsException {
		try {
			return delegate.findRelatedCpr(ocesInfo);
		} catch (StsException e) {
			log.warn("fallback for findRelatedCpr, returning " + cprNr, e);
			return cprNr;
		}
	}

	@Override
	public long lastSuccess() {
		return delegate.lastSuccess();
	}

	@Override
	public long lastFailure() {
		return delegate.lastFailure();
	}

	@Override
	public int getFailedCallsPercentage() { return delegate.getFailedCallsPercentage(); }

	@Override
	public int getHoursBackInTimeToCalculateFailedCallsPercentage() {
		return delegate.getHoursBackInTimeToCalculateFailedCallsPercentage();
	}

	@Override
	public boolean isRelated(final OcesInfo ocesInfo, final CprNr cpr) throws StsException {
		try {
			return delegate.isRelated(ocesInfo, cpr);
		} catch (StsException e) {
			log.warn("fallback for isRelated, returning true", e);
			return true;
		}
	}

	private void fallback(final StsException e) {
	}

	public OcesCvrRidService getDelegate() {
		return delegate;
	}

	public void setDelegate(OcesCvrRidService delegate) {
		this.delegate = delegate;
	}

	public String getCprNr() {
		return cprNr.getValue();
	}

	public void setCprNr(String cprNr) {
		this.cprNr = new CprNr(cprNr);
	}
}
