/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.cvrrid.service.impl.ssl;

import org.apache.axis.AxisEngine;
import org.apache.axis.AxisProperties;
import org.apache.axis.ConfigurationException;
import org.apache.axis.Handler;
import org.apache.axis.SimpleTargetedChain;
import org.apache.axis.configuration.SimpleProvider;
import org.apache.axis.transport.http.HTTPTransport;
import org.apache.log4j.Logger;

import javax.xml.namespace.QName;
import java.util.Hashtable;

/**
 * --------------------------------------------------------------------------------
 * This file was part of the project found at:
 *
 *    http://code.google.com/p/axis-ssl/
 *
 * and was changed to be part of STS. The modification were:
 *
 * - Eliminated the need for class-shadowing.
 * - updated the code to comply with java 1.6
 * --------------------------------------------------------------------------------
 *
 * @author Richard Unger
 * @auther ht@arosii.dk (STS)
 */
public class SSLClientAxisEngineConfig extends SimpleProvider {

	protected static Logger log = Logger.getLogger(SSLClientAxisEngineConfig.class.getName());

	private String proxyHost;
	private String proxyPort;
	private String proxyUser;
	private String proxyPassword;

	private String protocol;
	private String algorithm;

	private String keyStore;
	private String keyStoreType;
	private String keyStorePassword;

	private String trustStore;
	private String trustStorePassword;
	private String trustStoreType;

	private Hashtable<String, Object> options;

	/**
	 *
	 */
	public SSLClientAxisEngineConfig() {
		super();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.apache.axis.configuration.SimpleProvider#configureEngine(org.apache
	 * .axis.AxisEngine)
	 */
	@Override
  public void configureEngine(AxisEngine engine)
			throws ConfigurationException {
		super.configureEngine(engine);
		initializeAxisProperties();
		initializeGlobalOptions(engine);
		initializeTransport();
	}

	private void initializeAxisProperties() {
		log.debug("SSLClientAxisEngineConfig::AxisProperties::init");
		AxisProperties
				.setProperty("axis.socketSecureFactory",
						"dk.sosi.sts.ocescpr.cvrrid.service.impl.ssl.CertificateSecureSocketFactory");
		AxisProperties.setProperty("https.proxyHost", proxyHost);
		AxisProperties.setProperty("https.proxyPort", proxyPort);
		AxisProperties.setProperty("https.proxyUser", proxyUser);
		AxisProperties.setProperty("https.proxyPassword", proxyPassword);
	}

	private void initializeGlobalOptions(AxisEngine engine)
			throws ConfigurationException {
		setGlobalOptions(getOptions());
		engine.refreshGlobalOptions();

		if (log.isDebugEnabled()) {
			log.debug("Global options: "+getGlobalOptions());
		}
	}

	private void initializeTransport() throws ConfigurationException {
		QName http = new QName(null, HTTPTransport.DEFAULT_TRANSPORT_NAME);
		Handler transport = getTransport(http);

		if (transport == null) {
			log.debug("SSLClientAxisEngineConfig::Transport::init");

			Handler pivot = new StsHTTPSender();
			if (protocol != null) {
				pivot.setOption("protocol", protocol);
			}
			if (algorithm != null) {
				pivot.setOption("algorithm", algorithm);
			}

			if (keyStore != null) {
				pivot.setOption("clientauth", "true");
				pivot.setOption("keyStore", keyStore);
				if (keyStoreType != null)
					pivot.setOption("keyStoreType", keyStoreType);
				if (keyStorePassword != null) {
					pivot.setOption("keyStorePassword", keyStorePassword);
				}
			}
			if (trustStore != null) {
				pivot.setOption("trustStore", trustStore);
				if (trustStoreType != null)
					pivot.setOption("trustStoreType", trustStoreType);
				if (trustStorePassword != null)
					pivot.setOption("trustStorePassword", trustStorePassword);
			}

			transport = new SimpleTargetedChain(pivot);
			deployTransport(http, transport);
		}
	}

  /**
	 * @param protocol
	 *            the protocol to set
	 */
	public void setProtocol(String protocol) {
		this.protocol = protocol;
	}

	/**
	 * @param algorithm
	 *            the algorithm to set
	 */
	public void setAlgorithm(String algorithm) {
		this.algorithm = algorithm;
	}

	/**
	 * @param keyStore
	 *            the keyStore to set
	 */
	public void setKeyStore(String keyStore) {
		this.keyStore = keyStore;
	}

	/**
	 * @param keyStorePassword
	 *            the keyStorePassword to set
	 */
	public void setKeyStorePassword(String keyStorePassword) {
		this.keyStorePassword = keyStorePassword;
	}

	/**
	 * @param keyStoreType
	 *            the keyStoreType to set
	 */
	public void setKeyStoreType(String keyStoreType) {
		this.keyStoreType = keyStoreType;
	}

	/**
	 * @param trustStore
	 *            the trustStore to set
	 */
	public void setTrustStore(String trustStore) {
		this.trustStore = trustStore;
	}

	/**
	 * @param trustStorePassword
	 *            the trustStorePassword to set
	 */
	public void setTrustStorePassword(String trustStorePassword) {
		this.trustStorePassword = trustStorePassword;
	}

	/**
	 * @param trustStoreType
	 *            the trustStoreType to set
	 */
	public void setTrustStoreType(String trustStoreType) {
		this.trustStoreType = trustStoreType;
	}

	/**
	 * @param proxyHost
	 *            the proxyHost to set
	 */
	public void setProxyHost(String proxyHost) {
		this.proxyHost = proxyHost;
	}

	/**
	 * @param proxyPort
	 *            the proxyPort to set
	 */
	public void setProxyPort(String proxyPort) {
		this.proxyPort = proxyPort;
	}

	/**
	 * @param proxyUser
	 *            the proxyUser to set
	 */
	public void setProxyUser(String proxyUser) {
		this.proxyUser = proxyUser;
	}

	/**
	 * @param proxyPassword
	 *            the proxyPassword to set
	 */
	public void setProxyPassword(String proxyPassword) {
		this.proxyPassword = proxyPassword;
	}

	/**
	 * @param disablePrettyXML
	 *            the disablePrettyXML to set
	 */
	public void setDisablePrettyXML(Boolean disablePrettyXML) {
		getOptions().put(AxisEngine.PROP_DISABLE_PRETTY_XML, disablePrettyXML);
	}

	/**
	 * @param enableNamespacePrefixOptimization
	 *            the enableNamespacePrefixOptimization to set
	 */
	public void setEnableNamespacePrefixOptimization(
			Boolean enableNamespacePrefixOptimization) {
		getOptions().put(AxisEngine.PROP_ENABLE_NAMESPACE_PREFIX_OPTIMIZATION,
				enableNamespacePrefixOptimization);
	}

	/**
	 * @param sendMultiRefs
	 *            the sendMultiRefs to set
	 */
	public void setSendMultiRefs(Boolean sendMultiRefs) {
		getOptions().put(AxisEngine.PROP_DOMULTIREFS, sendMultiRefs);
	}

	/**
	 * @param key
	 * @param value
	 */
	public void setOption(String key, Object value) {
		getOptions().put(key, value);
	}

	/**
	 * @return
	 */
	public Hashtable<String, Object> getOptions() {
		if (options == null) {
			options = new Hashtable<String, Object>();
		}
		return options;
	}
}
