/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.cvrrid.service.impl.ssl;

/*
 * Copyright 2002-2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * This file modified from its original form by Richard Unger, Telekom Austria
 *
 * Modifications are intended to make the SocketFactoryFactory consider that for
 * SSL, different client certificates might be used at runtime.
 * Currently, the SocketFactory of a given type is initialized only once. The parameters
 * for SSL are set during initialization. Thus without this modification, there
 * is no way to change the SSL parameters at runtime.
 *
 * The modification considers the SSL parameters requested, and creates a new factory if the
 * params are different.
 *
 * --------------------------------------------------------------------------------
 * This file was part of the project found at:
 *
 *    http://code.google.com/p/axis-ssl/
 *
 * and was changed to be part of STS. The modification were:
 *
 * - Eliminated the need for class-shadowing.
 * - updated the code to comply with java 1.6
 * --------------------------------------------------------------------------------
 *
 * @author ht@arosii.dk (STS)
 */
import java.util.Hashtable;

import org.apache.axis.AxisProperties;
import org.apache.axis.components.net.SecureSocketFactory;
import org.apache.axis.components.net.SocketFactory;
import org.apache.log4j.Logger;

public class SocketFactoryFactory {
	/** Field log **/
	protected static Logger log = Logger.getLogger(SSLClientAxisEngineConfig.class.getName());

	/** socket factory **/
	private static Hashtable<String, SocketFactory> factories = new Hashtable<String, SocketFactory>();

	private static final Class<?> classes[] = new Class[] { Hashtable.class };

	static {
		AxisProperties.setClassOverrideProperty(SocketFactory.class,
				"axis.socketFactory");

		AxisProperties.setClassDefault(SocketFactory.class,
				"org.apache.axis.components.net.DefaultSocketFactory");

		AxisProperties.setClassOverrideProperty(SecureSocketFactory.class,
				"axis.socketSecureFactory");

		AxisProperties.setClassDefault(SecureSocketFactory.class,
				"org.apache.axis.components.net.JSSESocketFactory");
	}

	static final String PROTOCOL_HTTP = "http";
	static final String PROTOCOL_HTTPS = "https";

	/**
	 * Returns a copy of the environment's default socket factory.
	 *
	 * @param protocol
	 *            Today this only supports "http" & "https".
	 * @param attributes
	 *
	 * @return
	 */
	public static synchronized SocketFactory getFactory(String protocol,
			Hashtable<String, String> attributes) {

		final String attribute;

		log.debug("Getting socketfactory for protocol: " + protocol);

		if (PROTOCOL_HTTPS.equalsIgnoreCase(protocol)) {
			attribute = PROTOCOL_HTTPS + "||" + attributes.get("keyStore")
					+ "||" + attributes.get("trustStore");
		} else if (PROTOCOL_HTTP.equalsIgnoreCase(protocol)) {
			attribute = PROTOCOL_HTTP;
		} else {
			throw new IllegalArgumentException("Unknown Protocol type: "
					+ protocol);
		}

		SocketFactory theFactory = factories.get(attribute);
		if (theFactory == null) {
			Object objects[] = new Object[] { attributes };

			log.debug("New socket factory created for factory: " + attribute);

			if (PROTOCOL_HTTPS.equalsIgnoreCase(protocol)) {
				theFactory = (SecureSocketFactory) AxisProperties.newInstance(
						SecureSocketFactory.class, classes, objects);
			} else if (PROTOCOL_HTTP.equalsIgnoreCase(protocol)) {
				theFactory = (SocketFactory) AxisProperties.newInstance(
						SocketFactory.class, classes, objects);
			} else {
				throw new IllegalArgumentException("Unknown Protocol type: "
						+ protocol);
			}

			factories.put(attribute, theFactory);
		}

		return theFactory;
	}
}
