/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.pid;

import dk.sosi.sts.StsException;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.ocescpr.cvrrid.service.impl.CprCache;
import dk.sosi.sts.ocescpr.cvrrid.util.ValidateHelper;
import dk.sosi.sts.service.OcesPidService;
import dk.sosi.sts.timing.Timer;
import org.apache.log4j.Logger;

public class DBCacheOcesPidService implements OcesPidService {

	protected final OcesPidService delegate;
	protected final CprCache cache;
	private static Logger log = Logger.getLogger(DBCacheOcesPidService.class);

	public DBCacheOcesPidService(OcesPidService delegate, CprCache cache) {
		this.delegate = delegate;
		this.cache = cache;
	}


	public boolean isRelated(SubjectSerialNumber subjectSerialNumber, CprNr cpr) throws StsException {
		log.info("Executing method [isRelated] with cvrRid,cpr:" + subjectSerialNumber + "," + cpr);

		ValidateHelper.validate(subjectSerialNumber, cpr);

		if (cache.exists(subjectSerialNumber, cpr)) {
			return true;
		} else {
			final boolean result;
			Timer.start(this.getClass(), "delegate:isRelated");
			try {
				result = delegate.isRelated(subjectSerialNumber, cpr);
			} finally {
				Timer.stop(this.getClass(), "delegate:isRelated");
			}
			if (result) {
				cache.map(subjectSerialNumber, cpr);
			}
			return result;
		}
	}

	@Override
	public long lastSuccess() {
		return delegate.lastSuccess();
	}

	@Override
	public long lastFailure() {
		return delegate.lastFailure();
	}

	@Override
	public int getFailedCallsPercentage() { return delegate.getFailedCallsPercentage(); }

	@Override
	public int getHoursBackInTimeToCalculateFailedCallsPercentage() {
		return delegate.getHoursBackInTimeToCalculateFailedCallsPercentage();
	}
}
