/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.pid;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.ocescpr.cvrrid.util.ValidateHelper;
import dk.sosi.sts.ocescpr.uuid.uuid2cpr.Uuid2CprProxyClient;
import dk.sosi.sts.service.OcesPidService;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicLong;

public class WsOcesPidService implements OcesPidService {
	private static Logger log = Logger.getLogger(WsOcesPidService.class);

	private Uuid2CprProxyClient uuid2CprProxyClient;
	private AtomicLong lastSuccess = new AtomicLong(-1);
	private AtomicLong lastFailure = new AtomicLong(-1);

	private final ServiceInfoCollector serviceInfoCollector;

	public WsOcesPidService(Uuid2CprProxyClient uuid2CprProxyClient, ServiceInfoCollector serviceInfoCollector) {
		this.uuid2CprProxyClient = uuid2CprProxyClient;
        this.serviceInfoCollector = serviceInfoCollector;
    }

	@Override
	public boolean isRelated(SubjectSerialNumber subjectSerialNumber, CprNr cprNr) throws StsException {
		try {
			ValidateHelper.validate(subjectSerialNumber, cprNr);
			boolean related = checkRelated(subjectSerialNumber, cprNr);
			serviceInfoCollector.registerSuccess();
			return related;
		} catch (Exception e) { serviceInfoCollector.registerFailure(); throw e; }
	}

	public boolean checkRelated(SubjectSerialNumber ssn, CprNr cprNr) throws StsException {
		try {
			Optional<String> cpr = uuid2CprProxyClient.lookupCprFromPid(ssn.getValue());
			lastSuccess.set(System.currentTimeMillis());
			boolean b = cpr.map(v -> v.equalsIgnoreCase(cprNr.getValue())).orElse(false);
			serviceInfoCollector.registerSuccess();
			return b;
		} catch (IOException e) {
			log.error(e);
			lastFailure.set(System.currentTimeMillis());
			serviceInfoCollector.registerFailure();
			throw new StsException(e);
		}
	}

	@Override
	public long lastSuccess() {
		return lastSuccess.get();
	}

	@Override
	public long lastFailure() {
		return lastFailure.get();
	}

	@Override
	public int getFailedCallsPercentage() { return serviceInfoCollector.getFailedCallsPercentage(); }

	@Override
	public int getHoursBackInTimeToCalculateFailedCallsPercentage() { return serviceInfoCollector.getHoursBackInTime(); }
}
