/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.uuid.service.impl;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.ocescpr.uuid.util.ValidateHelper;
import dk.sosi.sts.ocescpr.uuid.uuid2cpr.Uuid2CprProxyClient;
import dk.sosi.sts.service.OcesUUID2CPRService;
import org.apache.log4j.Logger;

import java.util.concurrent.atomic.AtomicLong;

public class WsOcesUuidService implements OcesUUID2CPRService {

	private static Logger log = Logger.getLogger(WsOcesUuidService.class);

	private AtomicLong lastSuccess = new AtomicLong(-1);
	private AtomicLong lastFailure = new AtomicLong(-1);

    private final Uuid2CprProxyClient uuid2CprProxyClient;

	private final ServiceInfoCollector serviceInfoCollector;

	public WsOcesUuidService(Uuid2CprProxyClient client, ServiceInfoCollector serviceInfoCollector) {
		this.uuid2CprProxyClient = client;
        this.serviceInfoCollector = serviceInfoCollector;
    }

	public boolean isRelated(final OcesInfo ocesInfo, final CprNr cpr) throws StsException {
		try {
			ValidateHelper.validate(ocesInfo.getSubjectSerialNumber(), cpr);
			CprNr cprLookup = findRelatedCpr(ocesInfo);
			boolean result = cpr.equals(cprLookup);
			serviceInfoCollector.registerSuccess();
			return result;
		} catch (Exception e) { serviceInfoCollector.registerFailure(); throw e; }
	}

	public CprNr findRelatedCpr(OcesInfo ocesInfo) throws StsException {
		try {
			ValidateHelper.validate(ocesInfo.getSubjectSerialNumber());
            CprNr cprNr = uuid2CprProxyClient.lookupCpr(ocesInfo.getSubjectSerialNumber().getValue())
                    .map(CprNr::new).orElse(null);
			lastSuccess.set(System.currentTimeMillis());
			serviceInfoCollector.registerSuccess();
			return cprNr;
		} catch (Exception e) {
			lastFailure.set(System.currentTimeMillis());
			serviceInfoCollector.registerFailure();
			throw new StsException(String.format("Error looking up cpr for ssn %s",
												 ocesInfo.getSubjectSerialNumber().getValue()), e);
		}
	}

	public long lastSuccess() {
		return lastSuccess.get();
	}

	public long lastFailure() {
		return lastFailure.get();
	}

	@Override
	public int getFailedCallsPercentage() { return serviceInfoCollector.getFailedCallsPercentage();	}

	@Override
	public int getHoursBackInTimeToCalculateFailedCallsPercentage() { return serviceInfoCollector.getHoursBackInTime(); }

}
