package dk.sosi.sts.ocescpr.uuidrid;

import dk.sosi.sts.StsException;
import dk.sosi.sts.service.Uuid2RidService;
import dk.sosi.sts.timing.Timer;
import org.apache.log4j.Logger;

import java.util.Optional;

/**
 * Implementation of Uuid2RidService that uses another Uuid2RidService to map UUIDs to RID
 * and then caches the mappings for future lookups.
 */
public class DbCacheUuid2RidService implements Uuid2RidService {
    private final Logger log = Logger.getLogger(this.getClass());
    private final Uuid2RidService delegate;
    private final DbUuidRidCache cache;

    public DbCacheUuid2RidService(Uuid2RidService delegate, DbUuidRidCache cache) {
        this.delegate = delegate;
        this.cache = cache;
    }

    @Override
    public Optional<String> findRelatedRid(String uuid) throws StsException {
        log.info("Executing method [findRelatedRid] with UUID:" + uuid);

        Optional<String> rid = Optional.ofNullable(cache.findRidByUuid(uuid));
        if (rid.isPresent()) {
            return rid;
        }

        Timer.start(this.getClass(), "delegate:findRelatedRid");
        try {
            rid = delegate.findRelatedRid(uuid);
        } finally {
            Timer.stop(this.getClass(), "delegate:findRelatedRid");
        }
        if (rid.isPresent()) {
            cache.insert(uuid, rid.get());
        }
        return rid;
    }
}
