package dk.sosi.sts.ocescpr.uuidrid;

import dk.sosi.sts.StsException;
import org.apache.log4j.Logger;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.sql.DataSource;
import java.util.Collections;
import java.util.List;

/**
 * This class is used for storing and retrieving UUID-RID relations.
 */
public class DbUuidRidCache {
    private static final String SQL_CACHE_FIND = "SELECT rid FROM uuidridcache WHERE uuid=?";
    private static final String SQL_CACHE_INSERT = "INSERT INTO uuidridcache (uuid, rid) VALUES (?, ?)";

    private final Logger log = Logger.getLogger(this.getClass());
    private final JdbcTemplate jdbcTemplate;

    public DbUuidRidCache(DataSource ds) {
        jdbcTemplate = new JdbcTemplate(ds);
    }

    public String findRidByUuid(String uuid) throws StsException {
        try {
            Object[] args = Collections.singletonList(uuid).toArray();
            List<String> result = jdbcTemplate.queryForList(SQL_CACHE_FIND, args, String.class);
            return (result.isEmpty()) ? null : result.get(0);
        } catch (Exception e) {
            throw new StsException(e);
        }
    }

    public void insert(String uuid, String rid) throws StsException {
        try {
            jdbcTemplate.update(SQL_CACHE_INSERT, uuid, rid);
        } catch (DuplicateKeyException e) {
            log.info("Cache entry already exists for uuid: " + uuid);
        } catch (Exception e) {
            throw new StsException(e);
        }

    }
}
