package dk.sosi.sts.ocescpr.uuidrid;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.ocescpr.uuid.uuid2cpr.Uuid2CprProxyClient;
import dk.sosi.sts.service.ExternalService;
import dk.sosi.sts.service.Uuid2RidService;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicLong;

/**
 * Implementation of Uuid2RidService that uses the Uuid2CprProxyClient to map UUIDs to RID
 */
public class WsUuid2RidService implements Uuid2RidService, ExternalService {
    private final Logger log = Logger.getLogger(this.getClass());
    private final Uuid2CprProxyClient uuid2CprProxyClient;
    private final AtomicLong lastSuccess = new AtomicLong(-1);
    private final AtomicLong lastFailure = new AtomicLong(-1);
    private final ServiceInfoCollector serviceInfoCollector;

    public WsUuid2RidService(Uuid2CprProxyClient uuid2CprProxyClient, ServiceInfoCollector serviceInfoCollector) {
        this.uuid2CprProxyClient = uuid2CprProxyClient;
        this.serviceInfoCollector = serviceInfoCollector;
    }

    @Override
    public Optional<String> findRelatedRid(String uuid) throws StsException {
        try {
            Optional<String> rid = uuid2CprProxyClient.lookupRidFromUuid(uuid);
            lastSuccess.set(System.currentTimeMillis());
            serviceInfoCollector.registerSuccess();
            return rid;
        } catch (IOException e) {
            log.error(e);
            lastFailure.set(System.currentTimeMillis());
            serviceInfoCollector.registerFailure();
            throw new StsException(e);
        }
    }

    @Override
    public long lastSuccess() {
        return lastSuccess.get();
    }

    @Override
    public long lastFailure() {
        return lastFailure.get();
    }

    @Override
    public int getFailedCallsPercentage() { return serviceInfoCollector.getFailedCallsPercentage(); }

    @Override
    public int getHoursBackInTimeToCalculateFailedCallsPercentage() { return serviceInfoCollector.getHoursBackInTime(); }
}
