/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import dk.sosi.sts.ServiceInfoCollector;
import junit.framework.TestCase;
import localhost.HandleSundhedsportalWSPort;

import java.net.URL;
import java.rmi.RemoteException;

public class CvrRidWSStubIT extends TestCase {

    public static final String SERVICE_ENDPOINT_OCES1 = "https://test.sundhedsportalws.certifikat.dk/sundhedsportalws/HandleSundhedsportalWS";
	public static final String SERVICE_ENDPOINT_IG = "https://ws-erhverv.ig.certifikat.dk/rid_serviceprovider_server/services/HandleSundhedsportalWSPort";
    public static final String SERVICE_ENDPOINT_PP = "https://ws-erhverv.pp.certifikat.dk/rid_serviceprovider_server/services/HandleSundhedsportalWSPort";
    public static final String SERVICE_ENDPOINT_HTTPS = SERVICE_ENDPOINT_PP;
		
	private static HandleSundhedsportalWSPort service;

	protected void setUp() throws Exception {
		final WsOcesCvrRidService wsOcesCvrRidService = new WsOcesCvrRidService(new ServiceInfoCollector(0));
		wsOcesCvrRidService.setTrustStore("../sts-server/src/main/config/testtdc.keystore");
		wsOcesCvrRidService.setTrustStorePassword("gnyffe");
		wsOcesCvrRidService.setKeyStore("../sts-server/src/main/config/nsp-test-rid.jks");
		wsOcesCvrRidService.setKeyStorePassword("Test1234");
		wsOcesCvrRidService.setEndpoint(new URL(SERVICE_ENDPOINT_HTTPS));
		wsOcesCvrRidService.init();

		if (service == null) {
			service = wsOcesCvrRidService.getCvrRidService();
		}
	}

	protected void tearDown() throws Exception {
	}

	public void testValidCvrRid() throws RemoteException {
		String reply = service.getCPR("CVR:19343634-RID:1165813849809");
		assertEquals("1111111118", reply);
	}
	
	public void testInvalidCvrRid() throws RemoteException {
		String reply = service.getCPR("CVR:19343634-RID:1165813849804");
		assertEquals(null, reply);
	}

}
