/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import java.util.HashMap;

import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.certificate.OcesInfoParser;
import junit.framework.TestCase;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import org.junit.Ignore;

@Ignore("SDS-6660")
public class MockOcesCvrRidServiceTest extends TestCase {
	
	private static final SubjectSerialNumber SSN1 = new SubjectSerialNumber("CVR:11111111-RID:1111111111111");
  private static OcesInfo OCESINFO1 = OcesInfoParser.parseSubjectSerialNumber(SSN1);
	private static final CprNr CPR1 = new CprNr("1111111111");

	private static final SubjectSerialNumber SSN2 = new SubjectSerialNumber("CVR:22222222-RID:2222222222222");
  private static OcesInfo OCESINFO2 = OcesInfoParser.parseSubjectSerialNumber(SSN2);
	private static final CprNr CPR2 = new CprNr("2222222222");
	
	public void testNonEmptyConstructor() throws Exception {
		HashMap<SubjectSerialNumber, CprNr> map = new HashMap<SubjectSerialNumber, CprNr>();
		map.put(SSN1, CPR1);
		MockOcesCvrRidService service = new MockOcesCvrRidService(map);

		assertTrue(service.isRelated(OCESINFO1, CPR1));
		assertEquals(CPR1, service.findRelatedCpr(OCESINFO1));
	}

	public void testAddSubjectSerialNumberCprNr() throws Exception {
		MockOcesCvrRidService service = new MockOcesCvrRidService();
		service.add(SSN1, CPR1);
		
		assertTrue(service.isRelated(OCESINFO1, CPR1));
	}

	public void testAddStringString() throws Exception {
		MockOcesCvrRidService service = new MockOcesCvrRidService();
		service.add(SSN1.getValue(), CPR1.getValue());
		
		assertTrue(service.isRelated(OCESINFO1, CPR1));
	}

	public void testIsRelated() throws Exception {
		MockOcesCvrRidService service = new MockOcesCvrRidService();
		service.add(SSN1, CPR1);
		service.add(SSN2, CPR2);
		
		assertTrue(service.isRelated(OCESINFO1, CPR1));
		assertFalse(service.isRelated(OCESINFO1, CPR2));
		
		assertTrue(service.isRelated(OCESINFO2, CPR2));
		assertFalse(service.isRelated(OCESINFO2, CPR1));
	}

	public void testFindRelatedCpr() throws Exception {
		MockOcesCvrRidService service = new MockOcesCvrRidService();
		service.add(SSN1, CPR1);
		service.add(SSN2, CPR2);
		
		assertEquals(CPR1, service.findRelatedCpr(OCESINFO1));
		assertEquals(CPR2, service.findRelatedCpr(OCESINFO2));
	}

}
