package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.certificate.OcesInfoParser;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import junit.framework.TestCase;
import org.junit.Ignore;

import java.net.URL;

/**
 *
 */
@Ignore("SDS-6660")
public class WsOcesCvrRidFallbackTest extends TestCase {

	private static SubjectSerialNumber EXISTING_CVRRID = new SubjectSerialNumber("CVR:19343634-RID:1165813849809");
	private static OcesInfo EXISTING_CVRRID_OCESINFO = OcesInfoParser.parseSubjectSerialNumber(EXISTING_CVRRID);
	private static CprNr EXISTING_CPR = new CprNr( "1111111118");

	private static final int timeout = 400;

	public void testTimeout() throws Exception {
		WsOcesCvrRidService service = new WsOcesCvrRidService(new ServiceInfoCollector(0));
		service.setEndpoint(new URL("http://nowhere.co"));
		service.setTimeout(timeout);
		service.init();


		try {
			service.findRelatedCpr(EXISTING_CVRRID_OCESINFO);
			fail("Exception should be thrown");
		} catch (StsException t) {

		}
	}

	public void testFallback() throws Exception {
		WsOcesCvrRidService service = new WsOcesCvrRidService(new ServiceInfoCollector(0));
		service.setEndpoint(new URL("http://nowhere.co"));
		service.setTimeout(timeout);
		service.init();

		WsOcesCvrRidFallbackService fb = new WsOcesCvrRidFallbackService();

		fb.setDelegate(service);
		final String fallbackCpr = "1111112001";
		fb.setCprNr(fallbackCpr);

		final CprNr relatedCpr = fb.findRelatedCpr(EXISTING_CVRRID_OCESINFO);
		assertEquals(fallbackCpr, relatedCpr.getValue());
	}

	public void testNoFallback() throws Exception {
		MockOcesCvrRidService service = new MockOcesCvrRidService();
		service.add(EXISTING_CVRRID, EXISTING_CPR);

		WsOcesCvrRidFallbackService fb = new WsOcesCvrRidFallbackService();

		fb.setDelegate(service);
		final String fallbackCpr = "2111112001";
		fb.setCprNr(fallbackCpr);

		final CprNr cpr = fb.findRelatedCpr(EXISTING_CVRRID_OCESINFO);
		assertEquals(EXISTING_CPR, cpr);
	}

	public void testFallbackWith405() throws Exception {
		WsOcesCvrRidService service = new WsOcesCvrRidService(new ServiceInfoCollector(0));
		service.setEndpoint(new URL("http://www.google.com"));
		service.setTimeout(timeout);
		service.init();

		WsOcesCvrRidFallbackService fb = new WsOcesCvrRidFallbackService();

		fb.setDelegate(service);
		final String fallbackCpr = "2111112001";
		fb.setCprNr(fallbackCpr);

		final CprNr cpr = fb.findRelatedCpr(EXISTING_CVRRID_OCESINFO);
		assertEquals(fallbackCpr, cpr.getValue());
	}
}
