/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.certificate.OcesInfoParser;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import junit.framework.TestCase;

import java.net.MalformedURLException;
import java.net.URL;

public class WsOcesCvrRidServiceIT extends TestCase {

	private static SubjectSerialNumber EXISTING_CVRRID = new SubjectSerialNumber("CVR:19343634-RID:1165813849809");
  private static OcesInfo EXISTING_CVRRID_OCESINFO = OcesInfoParser.parseSubjectSerialNumber(EXISTING_CVRRID);
	private static CprNr EXISTING_CPR = new CprNr( "1111111118");

	private static SubjectSerialNumber MISSING_CVRRID = new SubjectSerialNumber("CVR:19343634-UID:1165814083652");
  private static OcesInfo MISSING_CVRRID_OCESINFO = OcesInfoParser.parseSubjectSerialNumber(MISSING_CVRRID);

	private WsOcesCvrRidService service;
	
	@Override
	protected void setUp() throws Exception {
		this.service = initService();
	}

	public static WsOcesCvrRidService initService() throws MalformedURLException {
		WsOcesCvrRidService service  = new WsOcesCvrRidService(new ServiceInfoCollector(0));
		service.setEndpoint(new URL(CvrRidWSStubIT.SERVICE_ENDPOINT_HTTPS));
		service.setTrustStore("../sts-server/src/main/config/testtdc.keystore");
		service.setTrustStorePassword("gnyffe");
		// trusted by server:
		// the same as ValidVoces ... expires march 2020
		service.setKeyStore("../sts-server/src/main/config/nsp-test-rid.jks");
		service.setKeyStorePassword("Test1234");
		service.init();
		return service;
	}

	public void testNullEndpoint() {
		try {
			new WsOcesCvrRidService(new ServiceInfoCollector(0)).init();
			fail("exception expected");
		} catch (Exception e) { /* expected */ }
	}

	public void testIsRelated_RelationExists() throws StsException {
		assertTrue(service.isRelated(EXISTING_CVRRID_OCESINFO, EXISTING_CPR));
	}

	public void testFindRelatedCpr_RelationExists() throws StsException {
		CprNr cpr = service.findRelatedCpr(EXISTING_CVRRID_OCESINFO);
		assertEquals(EXISTING_CPR, cpr);
	}

	public void testIsRelated_MissingRelation() throws StsException {
        CprNr missingCpr = new CprNr( "1111111119");
        assertFalse(service.isRelated(MISSING_CVRRID_OCESINFO, missingCpr));
	}

	public void testFindRelatedCpr_MissingRelation() throws StsException {
		CprNr cpr = service.findRelatedCpr(MISSING_CVRRID_OCESINFO);
		assertNull(cpr);
	}

}
