package dk.sosi.sts.ocescpr.cvrrid.service.impl;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.ocescpr.pid.WsOcesPidService;
import dk.sosi.sts.ocescpr.uuid.uuid2cpr.Uuid2CprProxyClient;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.IOException;
import java.util.Optional;

public class WsOcesPidServiceTest {

    public WsOcesPidService subject;
    private Uuid2CprProxyClient mockUuid2CprProxyClient;

    @Before
    public void setup() {
        mockUuid2CprProxyClient = Mockito.mock(Uuid2CprProxyClient.class);
        subject = new WsOcesPidService(mockUuid2CprProxyClient, new ServiceInfoCollector(0));
    }

    @Test
    public void testEmptyAnswerUUid2CprProxyResultsInNotRelated() throws IOException, StsException {

        // If
        Mockito.when(mockUuid2CprProxyClient.lookupCprFromPid(Mockito.anyString())).thenReturn(Optional.empty());

        // When
        boolean answer = subject.isRelated(new SubjectSerialNumber("TestSSN"), new CprNr("9999999999"));

        // Then
        Assert.assertFalse(answer);
    }


    @Test
    public void testMatchingAnswerUUid2CprProxyResultsRelated() throws IOException, StsException {

        // If
        String correctCpr = "1111111111";
        Mockito.when(mockUuid2CprProxyClient.lookupCprFromPid(Mockito.anyString())).thenReturn(Optional.of(correctCpr));

        // When
        boolean answer = subject.isRelated(new SubjectSerialNumber("TestSSN"), new CprNr(correctCpr));

        // Then
        Assert.assertTrue(answer);
    }

    @Test
    public void testNotMatchingAnswerUUid2CprProxyResultsNotRelated() throws IOException, StsException {

        // If
        String correctCpr = "1111111111";
        Mockito.when(mockUuid2CprProxyClient.lookupCprFromPid(Mockito.anyString())).thenReturn(Optional.of(correctCpr));

        // When
        boolean answer = subject.isRelated(new SubjectSerialNumber("TestSSN"), new CprNr("9999999999"));

        // Then
        Assert.assertFalse(answer);
    }

    @Test(expected = StsException.class)
    public void testExceptionFromUUid2CprProxyResultsExceptionWhenFindingRelatedCpr() throws IOException, StsException {

        // If
        Mockito.when(mockUuid2CprProxyClient.lookupCprFromPid(Mockito.anyString())).thenThrow(new IOException("TestMessage"));

        // When
        try {
            subject.isRelated(new SubjectSerialNumber("TestSSN"), new CprNr("9999999999"));
        } catch (StsException e) {
            // Then
            Assert.assertEquals("java.io.IOException: TestMessage", e.getMessage());
            throw e;
        }
    }
}
