/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.ocescpr.utilities;

import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.HashSet;

import junit.framework.TestCase;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;

public class HashHelperTest extends TestCase {

	private SubjectSerialNumber cvrRid1 = new SubjectSerialNumber("CVR:19343634-RID:1165813849809");
	private CprNr cpr1 = new CprNr( "1111111118");

	private SubjectSerialNumber cvrRid2 = new SubjectSerialNumber("CVR:19343634-UID:1165814083652");
	private CprNr cpr2 = new CprNr( "1111111119");

	public void testSha256Encode_() throws Exception {
		String hash11 = createAndAssertHash(cvrRid1, cpr1);
		String hash12 = createAndAssertHash(cvrRid1, cpr2);
		String hash21 = createAndAssertHash(cvrRid2, cpr1);
		String hash22 = createAndAssertHash(cvrRid2, cpr2);

		HashSet<String> all = new HashSet<String>(Arrays.asList(hash11, hash12, hash21, hash22));
		assertEquals("Non-unique hashes1", 4, all.size());
	}

	private String createAndAssertHash(SubjectSerialNumber cvrRid, CprNr cpr) throws NoSuchAlgorithmException {
		String hash = HashHelper.sha256Encode(cvrRid, cpr);
		assertFalse("CVR-RID in hash: " + cvrRid, 
				hash.toLowerCase().contains(cvrRid.getValue().toLowerCase()));
		assertFalse("CPR in hash: " + cpr, 
				hash.toLowerCase().contains(cpr.getValue().toLowerCase()));
		return hash;
	}
}
