package dk.sosi.sts.ocescpr.uuidrid;

import dk.sosi.sts.StsException;
import dk.sosi.sts.service.Uuid2RidService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Optional;

import static org.junit.Assert.*;

public class DbCacheUuid2RidServiceTest {

    private Uuid2RidService delegate;
    private DbUuidRidCache cache;
    private DbCacheUuid2RidService dbCacheUuid2RidService;

    @Before
    public void setUp() throws Exception {
        delegate = Mockito.mock(Uuid2RidService.class);
        cache = Mockito.mock(DbUuidRidCache.class);
        dbCacheUuid2RidService = new DbCacheUuid2RidService(delegate, cache);
    }

    @Test
    public void findRelatedRid_WhenRelationIsFoundInCache_ReturnsRidAndDoesNotUseDelegate() throws StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        String rid = "1234";
        Mockito.when(cache.findRidByUuid(uuid)).thenReturn(rid);

        Optional<String> result = dbCacheUuid2RidService.findRelatedRid(uuid);

        assertTrue(result.isPresent());
        assertEquals(rid, result.get());
        Mockito.verifyZeroInteractions(delegate);
    }

    @Test
    public void findRelatedRid_WhenRelationIsNotFoundInCacheButFromDelegate_ReturnsRidAndSavesRelationInCache() throws StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        String rid = "1234";
        Mockito.when(cache.findRidByUuid(uuid)).thenReturn(null);
        Mockito.when(delegate.findRelatedRid(uuid)).thenReturn(Optional.of(rid));

        Optional<String> result = dbCacheUuid2RidService.findRelatedRid(uuid);

        assertTrue(result.isPresent());
        assertEquals(rid, result.get());
        Mockito.verify(delegate, Mockito.times(1)).findRelatedRid(uuid);
        Mockito.verify(cache, Mockito.times(1)).insert(uuid, rid);
    }

    @Test
    public void findRelatedRid_WhenRelationIsNeitherFoundInCacheOrFromDelegate_ReturnsNullAndDoesNotSaveInCache() throws StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        Mockito.when(cache.findRidByUuid(uuid)).thenReturn(null);
        Mockito.when(delegate.findRelatedRid(uuid)).thenReturn(Optional.empty());

        Optional<String> result = dbCacheUuid2RidService.findRelatedRid(uuid);

        assertFalse(result.isPresent());
        Mockito.verify(delegate, Mockito.times(1)).findRelatedRid(uuid);
        Mockito.verify(cache, Mockito.times(1)).findRidByUuid(uuid);
        Mockito.verifyNoMoreInteractions(cache);
    }
}