package dk.sosi.sts.ocescpr.uuidrid;

import dk.sosi.sts.StsException;
import org.hsqldb.jdbc.JDBCDataSource;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.sql.Connection;
import java.sql.Statement;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

public class DbUuidRidCacheTest {

    public static final String[] DDL_CREATE = {
            "DROP TABLE IF EXISTS uuidridcache",
            "CREATE TABLE uuidridcache (uuid VARCHAR(64) NOT NULL, rid VARCHAR(32) NOT NULL, createdDate datetime default now())",
            "CREATE UNIQUE INDEX uuidridcache_uuid ON uuidridcache (uuid)"
    };
    public static final String DDL_DROP = "DROP TABLE uuidridcache";

    private JDBCDataSource dataSource;
    private DbUuidRidCache cache;

    @Before
    public void setUp() throws Exception {
        dataSource = new JDBCDataSource();
        dataSource.setDatabase("jdbc:hsqldb:mem:memoryDB");
        dataSource.setUser("sa");
        dataSource.setPassword("");
        Connection conn = dataSource.getConnection();
        Statement statement = conn.createStatement();
        for (String sql : DDL_CREATE) {
            statement.execute(sql);
        }
        conn.close();
        cache = new DbUuidRidCache(dataSource);
    }

    @After
    public void tearDown() throws Exception {
        Connection conn = dataSource.getConnection();
        conn.createStatement().execute(DDL_DROP);
        conn.close();
    }

    @Test
    public void findRidByUuid_WithUuidNotInCache_ReturnsNull() throws StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";

        String result = cache.findRidByUuid(uuid);

        assertNull(result);
    }

    @Test
    public void findRidByUuid_WithUuidRelationInsertedInCache_ReturnsRid() throws StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        String rid = "1234";
        cache.insert(uuid, rid);

        String result = cache.findRidByUuid(uuid);

        assertEquals(rid, result);
    }

    @Test
    public void insert_WithUuidAlreadyInCache_DoesNotUpdateRelation() throws StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        String rid = "1234";

        cache.insert(uuid, rid);
        cache.insert(uuid, "4312");
        String result = cache.findRidByUuid(uuid);

        assertEquals(rid, result);

    }
}