package dk.sosi.sts.ocescpr.uuidrid;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.ocescpr.uuid.uuid2cpr.Uuid2CprProxyClient;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.IOException;
import java.util.Optional;

import static org.junit.Assert.*;

public class WsUuid2RidServiceTest {

    private Uuid2CprProxyClient uuid2CprProxyClient;
    private WsUuid2RidService wsUuid2RidService;

    @Before
    public void setUp() throws Exception {
        uuid2CprProxyClient = Mockito.mock(Uuid2CprProxyClient.class);
        wsUuid2RidService = new WsUuid2RidService(uuid2CprProxyClient, new ServiceInfoCollector(0));
    }

    @Test
    public void findRelatedRid_WhenRidIsFoundWithClient_ReturnsRidAndSetsLastSuccess() throws IOException, StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        String rid = "1234";
        Mockito.when(uuid2CprProxyClient.lookupRidFromUuid(uuid)).thenReturn(Optional.of(rid));
        long startTime = System.currentTimeMillis();

        Optional<String> result = wsUuid2RidService.findRelatedRid(uuid);
        long lastSuccess = wsUuid2RidService.lastSuccess();
        long lastFailure = wsUuid2RidService.lastFailure();

        assertTrue(result.isPresent());
        assertEquals(rid, result.get());
        assertTrue(lastSuccess > startTime - 1);
        assertTrue(lastSuccess < System.currentTimeMillis() + 1);
        assertEquals(-1, lastFailure);
    }

    @Test
    public void findRelatedRid_WhenRidIsNotFoundWithClient_ReturnsEmptyAndSetsLastSuccess() throws IOException, StsException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        Mockito.when(uuid2CprProxyClient.lookupRidFromUuid(uuid)).thenReturn(Optional.empty());
        long startTime = System.currentTimeMillis();

        Optional<String> result = wsUuid2RidService.findRelatedRid(uuid);
        long lastSuccess = wsUuid2RidService.lastSuccess();
        long lastFailure = wsUuid2RidService.lastFailure();

        assertFalse(result.isPresent());
        assertTrue(lastSuccess > startTime - 1);
        assertTrue(lastSuccess < System.currentTimeMillis() + 1);
        assertEquals(-1, lastFailure);
    }

    @Test
    public void findRelatedRid_WhenLookupFails_SetsLastFailureAndThrowsException() throws IOException {
        String uuid = "UI:DK-E:G:f07daf8f-4f07-447e-a2b9-32cd15f6403c";
        Mockito.when(uuid2CprProxyClient.lookupRidFromUuid(uuid)).thenThrow(new IOException());
        long startTime = System.currentTimeMillis();

        try {
            wsUuid2RidService.findRelatedRid(uuid);
            fail("Should throw");
        } catch (StsException ignored) {
        }
        long lastSuccess = wsUuid2RidService.lastSuccess();
        long lastFailure = wsUuid2RidService.lastFailure();

        assertTrue(lastFailure > startTime - 1);
        assertTrue(lastFailure < System.currentTimeMillis() + 1);
        assertEquals(-1, lastSuccess);
    }
}