package dk.sosi.sts;

import java.time.OffsetDateTime;
import java.util.concurrent.ConcurrentLinkedQueue;

public class ServiceInfoCollector {
    private final ConcurrentLinkedQueue<OffsetDateTime>
            successfulCalls = new ConcurrentLinkedQueue<>(),
            failedCalls = new ConcurrentLinkedQueue<>();

    private final int hoursBackInTimeToRecordCalls;

    public ServiceInfoCollector(int hoursBackInTimeToRecordCalls) {
        this.hoursBackInTimeToRecordCalls = hoursBackInTimeToRecordCalls;
    }

    public void registerSuccess() {
        OffsetDateTime maxAge = OffsetDateTime.now().minusHours(hoursBackInTimeToRecordCalls);
        successfulCalls.removeIf(dateTime -> dateTime.isBefore(maxAge));
        successfulCalls.add(OffsetDateTime.now());
    }
    public void registerFailure() {
        OffsetDateTime maxAge = OffsetDateTime.now().minusHours(hoursBackInTimeToRecordCalls);
        failedCalls.removeIf(dateTime -> dateTime.isBefore(maxAge));
        failedCalls.add(OffsetDateTime.now());
    }

    public int getFailedCallsPercentage() {
        return failedCalls.size() + successfulCalls.size() == 0
                ? 0
                : failedCalls.size() * 100 / (failedCalls.size() + successfulCalls.size());
    }

    public int getHoursBackInTime() { return hoursBackInTimeToRecordCalls; }
}