/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.certificate;

import dk.sosi.sts.model.SubjectSerialNumber;

import java.security.cert.X509Certificate;

public class OcesInfo {

	public enum CertificateType { UNKNOWN, MOCES, VOCES, FOCES, POCES }
	public enum Version { @Deprecated OCES2, OCES3 }
	public enum PersistenceLevel {NONE, GLOBAL, LONG_TERM, SESSION_SPECIFIC}
	
	private final SubjectSerialNumber ssn;
	private final String cvr;
	private final CertificateType certificateType;
	private final Version version;
	private final X509Certificate certificate;
	private final PersistenceLevel persistenceLevel;

	public OcesInfo(SubjectSerialNumber ssn, String cvr, CertificateType type, Version version, final X509Certificate certificate) {
		this(ssn, cvr, type, version, certificate, PersistenceLevel.NONE);
	}

	public OcesInfo(SubjectSerialNumber ssn, String cvr, CertificateType type, Version version, final X509Certificate certificate, PersistenceLevel persistenceLevel) {
		this.ssn = ssn;
		this.cvr = cvr;
		this.certificateType = type;
		this.version = version;
		this.certificate = certificate;
		this.persistenceLevel = persistenceLevel;
	}

	public String getCvr() {
		return cvr;
	}

	public SubjectSerialNumber getSubjectSerialNumber() {
		return ssn;
	}
	
	public CertificateType getCertificateType() {
		return certificateType;
	}
	
	public Version getVersion() {
		return version;
	}

	public X509Certificate getCertificate() {
		return certificate;
	}

	public PersistenceLevel getPersistenceLevel() {
		return persistenceLevel;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("OcesInfo[\n");
		sb.append("SSN:").append(ssn).append("\n");
		sb.append("CVR:").append(cvr).append("\n");
		sb.append("TYPE:").append(certificateType).append("\n");
		sb.append("VERSION:").append(version).append("\n");
		sb.append("CERT:").append(certificate != null ? certificate.getSubjectDN().getName() : "No certificate").append("\n");
		if (persistenceLevel != PersistenceLevel.NONE) {
			sb.append("PERSISTENCE LEVEL:").append(persistenceLevel).append("\n");
		}
		sb.append("\n]");
		return sb.toString();
	}
}
