/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.certificate;

import java.security.cert.X509Certificate;
import java.util.LinkedList;
import java.util.List;

import dk.sosi.sts.certificate.OcesInfo.CertificateType;
import dk.sosi.sts.model.SubjectSerialNumber;

public abstract class OcesInfoParser {	
	private static final List<OcesInfoParser> PARSERS = new LinkedList<>();
	static {
		PARSERS.add(new Oces3InfoParser());
		PARSERS.add(new Oces2InfoParser());
	}
	private static final OcesInfo UNKNOWN_OCES_INFO = new OcesInfo(null, null, CertificateType.UNKNOWN, null, null);
	
	public static OcesInfo parseCertificate(X509Certificate certificate) {
		return PARSERS.stream()
				.filter(p -> p.isValid(certificate))
				.findFirst()
				.map(e -> e.parse(certificate))
				.orElse(getUnknownOcesInfo(certificate));
	}

	public static OcesInfo parseSubjectSerialNumber(SubjectSerialNumber subjectSerialNumber) {
		return PARSERS.stream()
				.filter(p -> p.isValid(subjectSerialNumber))
				.findFirst()
				.map(e -> e.parse(subjectSerialNumber))
				.orElse(getUnknownOcesInfo());
	}
	
	protected static OcesInfo getUnknownOcesInfo() {
		return getUnknownOcesInfo(null);
	}
	
	protected static OcesInfo getUnknownOcesInfo(X509Certificate certificate) {		
		return certificate != null ? new OcesInfo(null, null, CertificateType.UNKNOWN, null, certificate) : UNKNOWN_OCES_INFO;
	}
	
	protected abstract boolean isValid(X509Certificate cert);
	protected abstract boolean isValid(SubjectSerialNumber ssn);
	
	protected abstract OcesInfo parse(X509Certificate cert);
	protected abstract OcesInfo parse(SubjectSerialNumber ssn);	
}
