/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.model;

import java.util.List;

public interface AudienceConfiguration {

    List<String> getCertificates();

    String getJwtScope();

    /**
	 * @return the audience for this configuration. 
	 */
	String getAudience();
	
	/**
	 * This is a mandatory parameter. If this is not set, it defaults to 0, which
	 * results in invalid tokens. (the valid period is in the past)
	 * 
	 * The unit of measure is seconds.
	 * 
	 * @return the lifetime for the token.
	 */
    long getLifetime();

    /**
     * @return if the certificate reference should be included.
     */
    boolean requireCertificateAsReference();

    /**
     * @return if the cpr number should be included.
     */
    boolean requireCprNumber();

    /**
     * @return if the cvr number should be included.
     */
    boolean requireCvrNumberIdentifier();

    /**
     * @return if the organization name should be included.
     */
    boolean requireOrganizationName();

    /**
     * @return if the IT system name should be included.
     */
	boolean requireITSystemName();

    /**
     * @return if the authorization code should be included.
     */
	boolean requireAuthorizationCode();

    /**
     * @return if the user education code should be included.
     */
	boolean requireUserEducationCode();

    /**
     * This should be used to verify that id card is valid for serialization.
     * 
     * @return the maximal age for a given id card in seconds. 
     */
	Long getIdCardMaxAge();

    /**
     * Which version of the blurring instructions to use.
     */
    BlurringVersion requireBlurring();

    /**
     * Certificates that are allowed to claim a ward custody claim.
     */
    List<String> wardClaimAllowedCertificates();
}
