/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.model;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public class AudienceConfigurationBuilder<E extends AudienceConfiguration> {

	private final Class<? extends E> implClass;
	private Configuration conf;
	private String audience;

	public AudienceConfigurationBuilder(Class<? extends E> impl) {
		this.implClass = impl;
	}

	public AudienceConfigurationBuilder<E> setAudience(String audience) {
		this.audience = audience;
		return this;
	}
	
	public String getAudience() {
		return this.audience;
	}
	
	public AudienceConfigurationBuilder<E> setConfiguration(Configuration conf) {
		this.conf = conf;
		return this;
	}

	public E build() {
		try {
			Constructor<? extends E> constructor = implClass.getConstructor(audience.getClass(), conf.getClass());
			return constructor.newInstance(audience, conf);
		} catch (InstantiationException | IllegalAccessException | NoSuchMethodException | SecurityException | IllegalArgumentException | InvocationTargetException e) {
			throw new RuntimeException(e);
		}
	}
}
