/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.model;

import java.util.Optional;

import static java.util.Optional.empty;
import static java.util.Optional.of;

public class RoleDefinition {
    public static final String PREFIX = "urn:dk:healthcare:national-federation-role:";
    private final String
            roleId,
            roleName,
            externalName;

    public RoleDefinition(String code, String value, String externalName) {
        this.roleId = code;
        this.roleName = value;
        this.externalName = externalName;
    }

    public String getExternalName() { return externalName; }

    public String getRoleId() { return roleId; }

    public String getRoleName() { return roleName; }

    public String getPublicName() { return PREFIX + "code:"+ roleId + ":value:" + roleName; }

    public String getPublicNameNoCode() { return PREFIX + roleName; }

    public static boolean isNationalRole(String role) {
        if(isNationalRoleWithCodeAndValue(role)) { return true;        }
        else { return isNationalRoleWithoutCode(role); }
    }

    private static boolean isNationalRoleWithoutCode(String role) {
        if(!isRolePrefixOk(role)) { return false; }
        return !role.substring(PREFIX.length()).contains(":");
    }

    private static boolean isNationalRoleWithCodeAndValue(String role) {
        if(!isRolePrefixOk(role)) { return false; }
        String[] parts = role.substring(PREFIX.length()).split(":");
        if (parts.length != 4) { return false; }
        if (!"code".equals(parts[0])) { return false; }
        return "value".equals(parts[2]);
    }

    private static boolean isRolePrefixOk(String role) {
        if (role == null) { return false; }
        return role.startsWith(PREFIX);
    }

    public static Optional<String> extractRoleName(String v) {
        if (v.length() < PREFIX.length()) { return empty(); }
        String[] values = v.substring(PREFIX.length()).split(":");
        for (int i = 0; i < values.length; i++) {
            if (values[i].equals("value") && values.length > i + 1) {
                return of(values[i + 1]);
            }
        }
        return empty();
    }
}