/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.sla;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.api.SLALogger;
import dk.sdsd.nsp.slalog.impl.SLALoggerDummyImpl;

public class SLALogHelper {

	private static final Logger log = Logger.getLogger(SLALogHelper.class);
    private static final ThreadLocal<Map<SLALogPoint,SLALogItem>> slaLogItems = 
    		new ThreadLocal<Map<SLALogPoint,SLALogItem>>();

	private static SLALogger slaLogger = new SLALoggerDummyImpl();

	public static void setLogger(SLALogger logger) {
		slaLogger = logger;
	}

	public static SLALogItem createItem(SLALogPoint logPoint) {
		SLALogItem item = slaLogger.createLogItem(logPoint.getName(), logPoint.getId());
		setItem(logPoint, item);
		return item;
	}
	
	public static SLALogItem getItem(SLALogPoint logPoint) {
		Map<SLALogPoint, SLALogItem> map = slaLogItems.get();
		// no concurrent access to map
		if (map != null) {
			return map.get(logPoint);
		} else {
			return null;
		}
	}

	public static void clearItem(SLALogPoint logPoint) {
		setItem(logPoint, null);
	}

	public static void clearAll() {
		slaLogItems.remove();
	}
	
	public static void storeCallResultOk(SLALogPoint logPoint) {
		SLALogItem item = getItem(logPoint);
		if (item != null) {
			updateItemContext(item);
			item.setCallResultOk();
			item.store();
			clearItem(logPoint);
		} else {
			log.warn(MessageFormat.format("Storing ok result for missing log point ({0})", logPoint));			
		}
	}

	public static void storeCallResultError(SLALogPoint logPoint, String msg) {
		SLALogItem item = getItem(logPoint);
		if (item != null) {
			updateItemContext(item);
			item.setCallResultError(msg);
			item.store();
			clearItem(logPoint);
		} else {
			log.warn(MessageFormat.format("Storing error result for missing log point ({0}): ({1})", logPoint, msg));			
		}
	}
	
	protected static void updateItemContext(SLALogItem item) {
		StsContext c = getContext();
		if (item.getClientIP() == null) {
			if (c.getCallerIp() != null) {
				item.setClientIP(c.getCallerIp());
			} else if (c.getRequest() != null) {
				item.setClientIP(c.getRequest());
			}
		}
		if (item.getMessageId() == null) {
			item.setMessageId(c.getMessageId());
		}
	}

	public static StsContext getContext() {
		return StsContext.getContext();
	}
		
	protected static SLALogItem setItem(SLALogPoint logPoint, SLALogItem item) {
		Map<SLALogPoint, SLALogItem> map = slaLogItems.get();
		// no concurrent access to map
		if (map == null) {
			map = new HashMap<SLALogPoint, SLALogItem>();
			slaLogItems.set(map);
		}
		SLALogItem old = map.put(logPoint, item);
		if (old != null && item != null) {
			log.warn(MessageFormat.format("Replacing existing log point ({0}): {1}", logPoint, old));
		}
		return old;
	}

}
