/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.timing;

import org.apache.log4j.Logger;

/**
 * Records timing events using Jakarta Commons Logging. 
 * 
 * Somewhat of a misleading name. This operates now on 
 * log4j logger.
 */
public class CommonsLoggingTimeSink implements TimeSink {

	private Logger log;
	
	/**
	 * Creates an instance recording timing events to TIMING log.
	 */
	public CommonsLoggingTimeSink() {
		this(Logger.getLogger("TIMING"));
	}

	/**
	 * Creates an instance recording timing events to the given log.
	 * @param log
	 */
	public CommonsLoggingTimeSink(Logger log) {
		this.log = log;
	}

	public void enter(TimerEvent e) {
		if (log.isInfoEnabled()) {
			log.info("ENTER " + eventInfo(e));
		}
	}

	public void exit(TimerEvent e, long duration) {
		if (log.isInfoEnabled()) {
			log.info("EXIT " + eventInfo(e) + ": " + duration);
		}
	}

	public void discard(TimerEvent e, String reason) {
		log.warn("DISCARD " + eventInfo(e) + ": " + reason);
	}
	
	private String eventInfo(TimerEvent e) {
		return "(" + e.name + ")";
	}
}
