/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.util;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;

public class DbUtil {

	/**
	 * Close a {@link Connection} and optionally log any error 
	 * @param conn the {@link Connection} (<code>null</code> allowed)
	 * @param log the {@link Log} (<code>null</code> allowed)
	 */
	public static void close(Connection conn, Logger log) {
		try {
			if (conn != null) {
				conn.close();
			}
		} catch (SQLException e) {
			warn(log, e);
		}
	}

	/**
	 * Close a {@link Statement} and optionally log any error 
	 * @param stmt the {@link Statement} (<code>null</code> allowed)
	 * @param log the {@link Log} (<code>null</code> allowed)
	 */
	public static void close(Statement stmt, Logger log) {
		try {
			if (stmt != null) {
				stmt.close();
			}
		} catch (SQLException e) {
			warn(log, e);
		}
	}

	/**
	 * Close a {@link ResultSet} and optionally log any error 
	 * @param rs the {@link ResultSet} (<code>null</code> allowed)
	 * @param log the {@link Log} (<code>null</code> allowed)
	 */
	public static void close(ResultSet rs, Logger log) {
		try {
			if (rs != null) {
				rs.close();
			}
		} catch (SQLException e) {
			warn(log, e);
		}
	}

	/**
	 * Close a {@link Connection}, {@link Statement}, {@link ResultSet} and optionally 
	 * log any errors 
	 * @param conn the {@link Connection} (<code>null</code> allowed)
	 * @param stmt the {@link Statement} (<code>null</code> allowed)
	 * @param rs the {@link ResultSet} (<code>null</code> allowed)
	 * @param log the {@link Log} (<code>null</code> allowed)
	 */
	public static void close(Connection conn, Statement stmt, ResultSet rs, Logger log) {
		close(rs, log);
		close(stmt, log);
		close(conn, log);
	}

	
	private static void warn(Logger log, Object msg) {
		if (log != null) {
			log.warn(msg);			
		}
	}

}
