package dk.sosi.sts.certificate;

import static org.junit.Assert.assertEquals;

import java.security.cert.X509Certificate;

import javax.security.auth.x500.X500Principal;

import org.junit.Ignore;
import org.junit.Test;
import org.mockito.Mockito;

import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.vault.CredentialVaultTestUtil;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.certificate.OcesInfo.CertificateType;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.store.SimulatedMocesStore;
import dk.sosi.sts.store.SimulatedMocesStore.SimulatedMoces;

public class Oces3InfoParserUsingCertificateTest {

	@Test
	public void testMoces() {
		GenericCredentialVault vault = CredentialVaultTestUtil.getMoces3CredentialVault();
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();
		OcesInfo info = OcesInfoParser.parseCertificate(certificate);
		assertEquals(certificate, info.getCertificate());
		assertEquals(CertificateType.MOCES, info.getCertificateType());
		assertEquals("91026150", info.getCvr());
		assertEquals(new SubjectSerialNumber("UI:DK-E:G:dc7f4a98-d4f9-4d77-84f9-ae83c4e0bf61"), info.getSubjectSerialNumber());
	}
	
	@Test
	public void testVoces() {
		GenericCredentialVault vault = CredentialVaultTestUtil.getVoces3CredentialVault();
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();
		OcesInfo info = OcesInfoParser.parseCertificate(certificate);
		assertEquals(certificate, info.getCertificate());
		assertEquals(CertificateType.VOCES, info.getCertificateType());
		assertEquals("94354969", info.getCvr());
		assertEquals(new SubjectSerialNumber("UI:DK-O:G:c70b0207-162e-4d3d-a7f1-a19a8e07d99b"), info.getSubjectSerialNumber());
	}

	/*
	@Test
	public void testFoces() {
		// Bemærk: For OCES3 skelnes der ikke mellem FOCES og VOCES.
		GenericCredentialVault vault = CredentialVaultTestUtil.getCredentialVaultFromResource(SignatureUtil.setupCryptoProviderForJVM(), "oces3/FOCES_gyldig.p12");
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();
		OcesInfo info = OcesInfoParser.parseCertificate(certificate);
		assertEquals(certificate, info.getCertificate());
		assertEquals(CertificateType.VOCES, info.getCertificateType());
		assertEquals("94354969", info.getCvr());
		assertEquals(new SubjectSerialNumber("UI:DK-O:G:23550132-5e1f-4e43-a5f9-048acf49e0b8"), info.getSubjectSerialNumber());
	}

	 */
//	
//	@Test
//	public void testPoces() {
//		SimulatedMoces susanneAndersen = SimulatedMoces.SUSANNE_ANDERSEN;
//		X509Certificate certificate = susanneAndersen.getPoces();
//		OcesInfo info = OcesInfoParser.parseCertificate(certificate);
//		assertEquals(SimulatedMocesStore.getSusanneAndersen(), info.getCertificate());
//		assertEquals(CertificateType.POCES, info.getCertificateType());
//		assertEquals(null, info.getCvr());
//		assertEquals(new SubjectSerialNumber("PID:9208-2002-2-548474223842"), info.getSubjectSerialNumber());		
//	}
//	
	@Test
	public void testMockedWithoutSerialNumber() {
		X509Certificate certificate = Mockito.mock(X509Certificate.class);
		
		/*Known Intermediate certificate*/
		Mockito.when(certificate.getIssuerX500Principal()).thenReturn(new X500Principal("C=DK,O=Den Danske Stat,OU=Test - certifikat.local,CN=Den Danske Stat OCES udstedende-CA 1"));
		/*Unkown OCES3 format*/
		Mockito.when(certificate.getSubjectDN()).thenReturn(new X500Principal("CN=www.regionh.dk, O=CIMT, ST=Some-State, C=DK"));

		OcesInfo info = OcesInfoParser.parseCertificate(certificate);
		assertEquals(certificate, info.getCertificate());
		assertEquals(CertificateType.UNKNOWN, info.getCertificateType());
		assertEquals(null, info.getCvr());
		assertEquals(null, info.getSubjectSerialNumber());		
	}
	
	@Test
	public void testMockedIncorrectIssuer() {
		X509Certificate certificate = Mockito.mock(X509Certificate.class);
		
		/*Known Intermediate certificate*/
		Mockito.when(certificate.getIssuerX500Principal()).thenReturn(new X500Principal("C=DK,O=Den Falske Stat,OU=Test - certifikat.local,CN=Den Falske Stat OCES udstedende-CA 1"));
		/*Unkown OCES3 format*/
		Mockito.when(certificate.getSubjectDN()).thenReturn(new X500Principal("CN=www.regionh.dk, O=CIMT, ST=Some-State, C=DK"));

		OcesInfo info = OcesInfoParser.parseCertificate(certificate);
		assertEquals(certificate, info.getCertificate());
		assertEquals(CertificateType.UNKNOWN, info.getCertificateType());
		assertEquals(null, info.getCvr());
		assertEquals(null, info.getSubjectSerialNumber());		
	}
}
