/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.model;

import junit.framework.TestCase;

import java.util.Map;
import java.util.Properties;

public class ConfigurationTest extends TestCase {

  public void testSliceConfiguration() throws Exception {
    final Properties p = new Properties();
    p.setProperty("a.b.v1.vadd2.vadd3.name1", "v1name1");
    p.setProperty("a.b.v1.vadd2.vadd3.name2", "v1name2");
    p.setProperty("a.b.v1.vadd2.vadd3.name3", "v1name3");
    p.setProperty("a.b.v2.vadd2.vadd3.name1", "v2name1");
    p.setProperty("a.b.v2.vadd2.vadd3.name2", "v2name2");
    p.setProperty("a.b.v2.vadd2.vadd3.name3", "v2name3");
    p.setProperty("a.b.name3", "wrong");
    p.setProperty("fish", "cat");

    Configuration conf = new Configuration(p);
    final Map<String,Configuration> slice = conf.getPropertySlice("a.b");
    assertEquals(2, slice.size());
    final Configuration conf1 = slice.get("v1.vadd2.vadd3");
    assertNotNull(conf1);
    final Configuration conf2 = slice.get("v2.vadd2.vadd3");
    assertNotNull(conf2);

    assertEquals("v1name1",conf1.get("name1"));
    assertEquals("v1name2",conf1.get("name2"));
    assertEquals("v1name3",conf1.get("name3"));
    assertEquals("v2name1",conf2.get("name1"));
    assertEquals("v2name2",conf2.get("name2"));
    assertEquals("v2name3",conf2.get("name3"));
  }
  
  public void testGetProperties() throws Exception {
	Properties expected = new Properties();
	expected.setProperty("name1", "value1");
	expected.setProperty("name2", "value2");
	expected.setProperty("name3", "value3");
	  
	Properties actual = new Configuration(expected).getProperties();
	assertNotSame(expected, actual);
	assertEquals(expected.size(), actual.size());
	for (Object key : actual.keySet()) {
		assertTrue(expected.containsKey(key));
		assertEquals(expected.get(key), actual.get(key));
	}
  }
}
