/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import junit.framework.TestCase;

public class CprNrTest extends TestCase {
	
	public void testConstructor() {
		try {
			new CprNr("1234");
			fail("Number too short");
		} catch (Exception e) { /* expected */ }
		
		assertNotNull(new CprNr(null));

		assertNotNull(new CprNr("1111111111"));
	}

	public void testGetValue() {
		CprNr cpr = new CprNr("1111111118");
		assertEquals("1111111118", cpr.getValue());
	}
	
	public void testToString() {
		CprNr cpr = new CprNr("1111111118");
		assertFalse(cpr.toString().contains(cpr.getValue()));
		Matcher matcher = Pattern.compile("\\[\\d{6}xxxx\\]").matcher(cpr.toString());
	    assertTrue(cpr.toString() + " does not match: " + matcher.pattern().pattern(), matcher.find());
		
		cpr = new CprNr(null);
		assertNotNull(cpr.toString());
	}
	
	// pseudo test!
	public void testHashCode() {
		CprNr cpr = new CprNr("1111111118");
		cpr.hashCode();
		
		cpr = new CprNr(null);
		cpr.hashCode();
	}

	public void testEquals() {
		CprNr cpr = new CprNr("1111111118");
		assertEquals(cpr, cpr);
		
		CprNr cpr2 = new CprNr("1111111118");
		assertEquals(cpr, cpr2);

		assertFalse(cpr.equals(new Object()));
	}
}
