package dk.sosi.sts.model;

import org.junit.Test;

import static java.util.Optional.empty;
import static org.junit.Assert.*;

public class RoleDefinitionTest {
    @Test
    public void testInvalidRole() {
        boolean isRole = RoleDefinition.isNationalRole("urn:dk:healthcare:invalid-federation-role:code:1234:value:abc");

        assertFalse(isRole);
    }

    @Test
    public void testRoleWithCodeAndValue() {
        RoleDefinition roleDefinition = new RoleDefinition("1234", "abc", "external_name");

        boolean isRole = RoleDefinition.isNationalRole(roleDefinition.getPublicName());

        assertTrue(isRole);
    }

    @Test
    public void testRoleWithoutCode() {
        RoleDefinition roleDefinition = new RoleDefinition("1234", "abc", "external_name");

        boolean isRole = RoleDefinition.isNationalRole(roleDefinition.getPublicNameNoCode());

        assertTrue(isRole);
    }

    @Test
    public void givenInvalidRoleAsStringThenExtractRoleNameShouldReturnEmpty() {
        assertEquals(empty(), RoleDefinition.extractRoleName("Invalid role"));
    }

    @Test
    public void givenInvalidRoleAsStringThenExtractRoleNameShouldReturnEmpty_2() {
        assertEquals(empty(), RoleDefinition.extractRoleName(RoleDefinition.PREFIX + "value"));
    }

    @Test
    public void givenValidRoleAsStringThenExtractRoleNameShouldReturnRoleName() {
        String roleName = "abc";
        String role = "urn:dk:healthcare:national-federation-role:code:1234:value:" + roleName;
        assertEquals(roleName, RoleDefinition.extractRoleName(role).get());
    }
}
