package dk.sosi.sts.store;

import java.io.InputStream;
import java.security.Principal;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import org.apache.commons.lang3.StringUtils;

import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.certificate.OcesInfoParser;
import dk.sosi.sts.model.SubjectSerialNumber;

public class SimulatedMocesStore {
    public enum SimulatedMoces {
    	SUSANNE_ANDERSEN("1412501112","33888888","Frisørhuset v/Susanne Andersen");
    	
    	private final String givenName;
    	private final String surName;
    	private final String cpr;
    	private final String cvr;
    	private final String orgName;
    	private final X509Certificate poces;
    	
    	SimulatedMoces(String cpr, String cvr, String orgName){
    		String[] split = this.name().toLowerCase().split("_");
    		this.givenName = StringUtils.capitalize(split[0]);
    		this.surName = StringUtils.capitalize(split[1]);
    		try {
				this.poces = getPocesCertificate("poces/"+this.name().toLowerCase()+".crt");
			} catch (CertificateException e) {
				throw new RuntimeException(e);
			}
    		this.cpr = cpr;
    		this.cvr = cvr;
    		this.orgName = orgName;
    	}
    	
    	public X509Certificate getPoces() {
			return poces;
		}
    	
    	public String getCpr() {
			return cpr;
		}
    	
    	public String getGivenName() {
			return givenName;
		}
    	
    	public String getSurName() {
			return surName;
		}

    	public String getCvr() {
			return cvr;
		}
    	
    	public String getOrgName() {
			return orgName;
		}
    	
    	public OcesInfo getOcesInfo() {
    		return OcesInfoParser.parseCertificate(poces);
    	}
    	
    	public SubjectSerialNumber getSubjectSerialNumber() {
    		return getOcesInfo().getSubjectSerialNumber();
    	}
    	
    	public String getSubjectSerialNumberValue() {
    		return getSubjectSerialNumber().getValue();
    	}
    	
    	public static SimulatedMoces getByCertificate(X509Certificate cert) {
    		Principal subjectDN = cert.getSubjectDN();
    		for (SimulatedMoces simulatedMoces : SimulatedMoces.values()) {
				if(simulatedMoces.poces.getSubjectDN().equals(subjectDN)) {
					return simulatedMoces;
				}
			}
    		
			throw new RuntimeException("Simulated Moces belonging to "+subjectDN+" not found");
    	}
    }
    
    /**
     * Certificate for test citizen Susanne Andersen can be found here:
     * https://appletk.danid.dk/developers/viewstatus.jsp?userid=343037425
     * 
     * If current certificate is obsolete - download ACTIVE certificate under Private Keys.
     * Current certificate will expire "Sun Sep 12 12:16:42 CEST 2021"
     */
    public static X509Certificate getSusanneAndersen() {
    	return SimulatedMoces.SUSANNE_ANDERSEN.poces;
    }
    
    private static X509Certificate getPocesCertificate(String crt) throws CertificateException {
    	CertificateFactory factory = CertificateFactory.getInstance("X.509");
    	InputStream certificateInputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(crt);
      	X509Certificate userCertificate = (X509Certificate)factory.generateCertificate(certificateInputStream);
      	return userCertificate;
    }
}
