/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.timing;

import java.util.Arrays;

import junit.framework.TestCase;

public class TimerStackTest extends TestCase {
	
	public void testEmptyStack() {
		TimerStack stack = new TimerStack();

		assertNull(null, stack.peek());
		assertNull(null, stack.pop());
	}

	public void testPushPop() {
		TimerStack stack = new TimerStack();

		TimerEvent e1 = new TimerEvent(getClass(), "op1", System.nanoTime());
		stack.push(e1);
		TimerEvent e2 = new TimerEvent(getClass(), "op2", System.nanoTime());
		stack.push(e2);

		assertEquals(e2, stack.pop());
		assertEquals(e1, stack.pop());
		assertNull(null, stack.pop());
	}
	
	public void testPeek() {
		TimerStack stack = new TimerStack();

		stack.push(new TimerEvent(getClass(), "op1", System.nanoTime()));
		stack.push(new TimerEvent(getClass(), "op2", System.nanoTime()));

		assertEquals(stack.peek(), stack.pop());
		assertEquals(stack.peek(), stack.pop());
		assertNull(null, stack.peek());
	}
	
	public void testFilter() throws Exception {
		TimerStack stack = new TimerStack(
				Arrays.asList("TimerStackTest#filtered", "TimerStackTest#ignored"));

		TimerEvent e;
		e = stack.push(new TimerEvent(getClass(), "op1", System.nanoTime()));
		assertEquals("TimerStackTest#op1", e.name);
		
		// add filtered events
		e = stack.push(new TimerEvent(getClass(), "filtered1", System.nanoTime()));
		assertEquals(e, TimerEvent.EMPTY);
		e = stack.push(new TimerEvent(getClass(), "ignored1", System.nanoTime()));
		assertEquals(e, TimerEvent.EMPTY);
		// add nested unfiltered
		e = stack.push(new TimerEvent(getClass(), "unfiltered", System.nanoTime()));
		assertEquals(e, TimerEvent.EMPTY);
		assertEquals(TimerEvent.EMPTY, stack.peek());
		assertEquals(TimerEvent.EMPTY, stack.pop());

		// pop filtered (ignored1)
		assertEquals(TimerEvent.EMPTY, stack.peek());
		assertEquals(TimerEvent.EMPTY, stack.pop());
		// pop filtered (filtered1)
		assertEquals(TimerEvent.EMPTY, stack.peek());
		assertEquals(TimerEvent.EMPTY, stack.pop());

		// add unfiltered events
		e = stack.push(new TimerEvent(getClass(), "op2", System.nanoTime()));
		assertEquals("TimerStackTest#op2", e.name);
		e = stack.push(new TimerEvent(getClass(), "op3", System.nanoTime()));
		assertEquals("TimerStackTest#op3", e.name);

		assertEquals("TimerStackTest#op3", stack.pop().name);
		assertEquals("TimerStackTest#op2", stack.pop().name);
		assertEquals("TimerStackTest#op1", stack.pop().name);
	}
	
	public void testMaxCapacity() {
		TimerStack stack = new TimerStack(8, 8);

		for (int i = 0; i < 8; i++) {
			stack.push(new TimerEvent(getClass(), "op" + i, System.nanoTime()));
		}
		
		assertNull(stack.push(new TimerEvent(getClass(), "op", System.nanoTime())));
	}

}
