/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.util;

import java.io.IOException;

import dk.sosi.sts.StsException;
import junit.framework.TestCase;

public class StsUtilTest extends TestCase {

	public void testRethrowable_Throwable() throws Exception {
		try {
			StsUtil.rethrowable(new RuntimeException("cause"), StsException.class);
			fail("Expected rethrown exception");
		} catch (Throwable t) {
			assertEquals(RuntimeException.class, t.getClass());
			assertEquals("cause", t.getMessage());
		}
	}
	
	public void testRethrowable_Error() throws Exception {
		try {
			StsUtil.rethrowable(new AssertionError("cause"), StsException.class);
			fail("Expected rethrown exception");
		} catch (Throwable t) {
			assertEquals(AssertionError.class, t.getClass());
			assertEquals("cause", t.getMessage());
		}
	}
	
	public void testRethrowable_CheckedWrapped() throws Exception {
		try {
			StsUtil.rethrowable(new IOException("cause"), StsException.class);
			fail("Expected rethrown wrapped exception");
		} catch (Throwable t) {
			assertEquals(RuntimeException.class, t.getClass());
			assertEquals(IOException.class, t.getCause().getClass());
			assertEquals("cause", t.getCause().getMessage());
		}
	}

	public void testRethrowable_CheckedReturned() throws Exception {
		StsException e = StsUtil.rethrowable(new StsException("cause"), StsException.class);
		assertEquals("cause", e.getMessage());
	}

}
