/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.acl;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.server.StsConfig;
import dk.sosi.sts.service.AclService;

public class MockAclService implements AclService {

	private final Set<String> blackList;
	private final Set<String> whiteList;

	public MockAclService() {
		this(null, null);
	}
	
	public static final String calculateWhitelistKey(OcesInfo ocesInfo, String systemName) {
		return "CVR:" + ocesInfo.getCvr() + "-SYSTEM:" + systemName;
	}

	public MockAclService(Set<String> blackList, Set<String> whiteList) {
		this.blackList = blackList;
		this.whiteList = whiteList;
	}
	
	public MockAclService(StsConfig config) {
		String prefix = getClass().getName() + ".";
		this.blackList = createSet(config.get(prefix + "blacklist"));
		this.whiteList = createSet(config.get(prefix + "whitelist"));
	}
	
	public boolean isBlacklisted(OcesInfo ocesInfo, String systemName) {
		if (blackList == null) {
			return false;
		} else {
			return blackList.contains(ocesInfo.getSubjectSerialNumber().getValue());
		}
	}

	public boolean isWhitelisted(OcesInfo ocesInfo, String systemName) {
		if (whiteList == null) {
			return true;
		} else {
			return whiteList.contains(calculateWhitelistKey(ocesInfo, systemName));
		}
	}
	
	private Set<String> createSet(String list) {
		HashSet<String> set;

		if (list == null) {
			set = null;
		} else {
			set = new HashSet<String>(Arrays.asList(list.split("\\s+")));
		}
		
		return set;
	}
}
