package dk.sosi.sts.monitor;

import javax.sql.DataSource;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

public class CrlExpiredCheck implements Check {

    private final DataSource dataSource;

    public CrlExpiredCheck(DataSource dataSource) { this.dataSource = dataSource; }

    @Override
    public CheckResult performCheck() {
        try {
            Map<String, String> errorMessages = new LinkedHashMap<>();
            try (PreparedStatement crlsStatement = dataSource.getConnection()
                    .prepareStatement("SELECT url, nextupdate FROM crl")) {
                for (ResultSet resultSet = crlsStatement.executeQuery(); resultSet.next(); ) {
                    long nextUpdate = resultSet.getLong("nextupdate");
                    if(System.currentTimeMillis() > nextUpdate) {
                        errorMessages.put("CRL expired",
                                          resultSet.getString("url") + " expired at " + new Date(nextUpdate));
                    }
                }
                return new CheckResult(!errorMessages.isEmpty(), errorMessages);
            }
        } catch (SQLException e) { throw new RuntimeException(e); }
    }
}