/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.monitor;

import dk.sosi.sts.util.DbUtil;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Collections;

public class DBCheck implements Check {

	private final DataSource ds;
    private final String name;
    private final String sql;

	public DBCheck(final String sql, final DataSource ds, String name) {
		this.sql = sql;
		this.ds = ds;
        this.name = name;
    }

	public CheckResult performCheck() {

    if (ds != null) {
      boolean failed = false;
      Connection connection = null;
      PreparedStatement pstmt = null;
      try {
        connection = ds.getConnection();
        pstmt = connection.prepareStatement(sql);
        pstmt.execute();
        pstmt.close();
      } catch (SQLException e) {
        failed = true;
      } finally {
        DbUtil.close(pstmt, null);
        DbUtil.close(connection, null);
      }
      return new CheckResult(failed,
                             Collections.singletonMap(name, failed ? "No connection" : "OK"));
    } else {
      return new CheckResult(true,
                             Collections.singletonMap(name, "Not configured"));

    }

  }

}
