/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.monitor;

import dk.sosi.sts.service.ExternalService;

import java.util.LinkedHashMap;
import java.util.Map;

public class ExternalServiceCheck implements Check {

	private final ExternalService externalService;
    private final String serviceName;

    public ExternalServiceCheck(final ExternalService service, String serviceName) {
		this.externalService = service;
        this.serviceName = serviceName;
    }

  public CheckResult performCheck() {
    boolean failed = false;

    final Map<String, String> result = new LinkedHashMap<String, String>();
    result.put(serviceName + ", latest successful call", MonitorUtil.toDate(externalService.lastSuccess()));
    result.put(serviceName + ", latest failed call", MonitorUtil.toDate(externalService.lastFailure()));

    return new CheckResult(failed, result);
  }
}