package dk.sosi.sts.monitor;

import dk.sosi.sts.service.ExternalService;

import static java.util.Collections.emptyMap;
import static java.util.Collections.singletonMap;

public class ExternalServiceFailureCheck implements Check {

    private final ExternalService externalService;
    private final String serviceName;
    private final int percentageFailureThreshold;

    public ExternalServiceFailureCheck(ExternalService service, String serviceName, int percentageFailureThreshold) {
        this.externalService = service;
        this.serviceName = serviceName;
        this.percentageFailureThreshold = percentageFailureThreshold;
    }

    @Override
    public CheckResult performCheck() {
        int failedCallsPercentage = externalService.getFailedCallsPercentage(),
            hours = externalService.getHoursBackInTimeToCalculateFailedCallsPercentage();
        String s = hours == 1 ? " hour" : " hours";
        return failedCallsPercentage > percentageFailureThreshold
                ? new CheckResult(true, singletonMap(serviceName,
                                                     failedCallsPercentage + "% of calls has failed in the last " +
                                                             hours + s))
                : new CheckResult(false, emptyMap());
    }
}