/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.monitor;

import dk.sosi.sts.timing.Timer;
import dk.sosi.sts.util.LogUtil;
import org.apache.log4j.Logger;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.Controller;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

public class MonitorServlet implements Controller {

	protected static Logger log = Logger.getLogger(MonitorServlet.class);

	private final Check[] checks;

	public MonitorServlet(final Check[] checks) { this.checks = checks;	}

	protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
		boolean failed = false;

		Timer.start(getClass(), "doGet");
 		Map<String, String> allMessages = new LinkedHashMap<>();
		try {
			for (Check check : checks) {
				Check.CheckResult result = check.performCheck();
				failed = failed || result.failed;
				allMessages.putAll(result.result);
			}
		} catch (Throwable t) {
			LogUtil.logOnce(MonitorServlet.class.getName(), log, t);
			failed = true;
		} finally {
			Timer.stop(getClass(), "doGet");
		}

		resp.setContentType("text/plain");
		// we want to set status before writing output
		resp.setStatus(failed ? 500 : 200);
		for (Map.Entry<String, String> entry : allMessages.entrySet()) {
			resp.getWriter().print(entry.getKey() + ": " + entry.getValue() + "\n");
		}
		resp.getWriter().print("EOS\n");
	}

	@Override
	public ModelAndView handleRequest(final HttpServletRequest request, final HttpServletResponse response) throws Exception {
		doGet(request, response);
		return null;
	}
}