package dk.sosi.sts.monitor;

import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.Controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import static java.util.stream.Stream.concat;

public class StatusServlet implements Controller {

    private final List<Check> checks;

    public StatusServlet(List<Check> checks) { this.checks = checks; }

    @Override
    public ModelAndView handleRequest(HttpServletRequest httpServletRequest, HttpServletResponse resp)
            throws Exception {
        Predicate<Check> checkFailure = check -> check instanceof StatusCheck || check instanceof DBCheck;
        List<Check.CheckResult> resultsToGetMessageAndCheckFailureFrom =
                checks.stream().filter(checkFailure).map(Check::performCheck).collect(Collectors.toList());
        List<Check.CheckResult> resultsToOnlyGetMessageFrom =
                checks.stream().filter(checkFailure.negate()).map(Check::performCheck).collect(Collectors.toList());

        boolean anyFailed = resultsToGetMessageAndCheckFailureFrom.stream().map(r -> r.failed)
                        .reduce((acc, failed) -> acc || failed).orElse(false);
        resp.setContentType("text/plain;charset=utf-8");
        resp.setStatus(anyFailed ? 500 : 200);

        PrintWriter writer = resp.getWriter();
        concat(resultsToGetMessageAndCheckFailureFrom.stream(), resultsToOnlyGetMessageFrom.stream())
                .forEach(cr -> cr.result.forEach((key, value) -> writer.print(key + ": " + value + "\n")));
        writer.print("EOS\n");
        return null;
    }
}