/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.roles.authorization;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.LinkedList;
import java.util.List;

import javax.sql.DataSource;

import org.apache.log4j.Logger;

import dk.sosi.sts.StsException;
import dk.sosi.sts.model.Authorization;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.service.AuthorizationService;
import dk.sosi.sts.util.DbUtil;

public class DbAuthorizationService implements AuthorizationService {

	private static final String DEFAULT_SELECT = "SELECT aut_id, edu_id " +
			"FROM autreg WHERE cpr=?";
	private final Logger log = Logger.getLogger(this.getClass());
	private final DataSource ds;
	private String select;
	
	public DbAuthorizationService(DataSource ds, String select) {
		this.ds = ds;
		this.select = select;
	}

	public DbAuthorizationService(DataSource ds) {
		this(ds, DEFAULT_SELECT);
	}

	public List<Authorization> getAuthorizations(CprNr cpr) throws StsException {
		PreparedStatement statement = null;
		Connection connection = null;
		ResultSet rs = null;
		try {
			connection = ds.getConnection();
			statement = connection.prepareStatement(select);
			statement.setString(1, cpr.getValue());
			rs = statement.executeQuery();
			
			List<Authorization> result = new LinkedList<Authorization>();
			while (rs.next()) {
				Authorization a = new Authorization(rs.getString("aut_id"), rs.getString("edu_id"));
				result.add(a);
			}
			log.debug("Found " + result.size() + " authorizations for " + cpr);
			return result;
		} catch (Exception e) {
			throw new StsException(e);
		} finally {
			DbUtil.close(connection, statement, rs, log);
		}
	}
}
