/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.roles.authorizationdefinitions;

import java.util.List;

import dk.sosi.sts.server.cache.CachingService;
import dk.sosi.sts.service.AuthorizationDefinitionService;

public class CachingAuthorizationDefinitionsService extends CachingService<AuthorizationDefinitionService, List<String>>implements AuthorizationDefinitionService{

    public CachingAuthorizationDefinitionsService(AuthorizationDefinitionService backend) {
    	super(backend);
    }
    
    @Override
    public List<String> getAuthorizationDefinitions() {
    	return ensureCache();
    }
    
    /**
     * This method is not send to backend implementation. We need to make sure that cache is used.
     */
    @Override
    public boolean isValidRole(String role) {
    	return role != null ? getAuthorizationDefinitions().contains(role) : false;
    }
    
    @Override
    protected final List<String> getUncachedValue(AuthorizationDefinitionService backend) {
      final List<String> authorizationDefinitions = backend.getAuthorizationDefinitions();
      log.info("Found " + authorizationDefinitions.size() + " authorization definitions");
      return authorizationDefinitions;
    }
    
    @Override
    public final String getCacheId() {
    	return "AuthorizationDefinition";
    }
}