/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.roles.authorizationdefinitions;

import java.util.List;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import dk.sosi.sts.service.AuthorizationDefinitionService;

public class DbAuthorizationDefinitionsService implements AuthorizationDefinitionService {
    private final Logger log = Logger.getLogger(this.getClass());

    private DataSource dataSource;

    private String  authorizationDefinitionSql = "SELECT * FROM sts_audconf.authorizationDefinitions";

    // injected by spring
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    // injected by spring
    public void setSql(String sql) {
        this.authorizationDefinitionSql = sql;
    }
    
    @Override
    	public List<String> getAuthorizationDefinitions() {
    		JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
    		
    		RowMapper<String> rowMapper = (rs, i) -> {
    			return rs.getString("role");
    		};
    		
    		List<String> result = jdbcTemplate.query(authorizationDefinitionSql, rowMapper);
    		log.debug("Found " + result.size() + " authorizationDefinitions");

    		return result;
    	}
    
    @Override
    public boolean isValidRole(String role) {
    	if(role != null && role.length() == 4) {
    		return getAuthorizationDefinitions().contains(role);
    	}
    	
    	return false;
    }
}
