/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.roles.roledefinitions;

import dk.sosi.sts.model.RoleDefinition;
import dk.sosi.sts.service.RoleDefinitionService;
import org.apache.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.RowMapper;

import javax.sql.DataSource;
import java.util.List;
import java.util.Optional;

import static java.util.Optional.empty;
import static java.util.Optional.of;

public class DbRoleDefinitionsService implements RoleDefinitionService {
    public static final String DEFAULT_SOURCE = "NationalFederation";
    private final Logger log = Logger.getLogger(this.getClass());

    private DataSource dataSource;

    private String roleDefinitionSql = "SELECT * FROM sts_audconf.roleDefinitions WHERE source = ?";
    private Optional<String> trustedRolesSql = empty();

    // injected by spring
    public void setDataSource(DataSource dataSource) { this.dataSource = dataSource; }

    // injected by spring
    public void setSql(String sql) { this.roleDefinitionSql = sql; }

    // injected by spring
    public void setTrustedRolesSql(String trustedRolesSql) { this.trustedRolesSql = of(trustedRolesSql); }

    @Override
    public List<RoleDefinition> getRoleDefinitions() {
        JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
        PreparedStatementSetter setter = statement -> statement.setString(1, DEFAULT_SOURCE);
        RowMapper<RoleDefinition> rowMapper = (rs, i) -> {
            final String code = rs.getString("code");
            final String value = rs.getString("value");
            final String externalName = rs.getString("externalName");
            return new RoleDefinition(code, value, externalName);
        };
        List<RoleDefinition> result = jdbcTemplate.query(roleDefinitionSql, setter, rowMapper);
        log.debug("Found " + result.size() + " roleDefinitions for " + DEFAULT_SOURCE);
        return result;
    }

    @Override
    public List<String> getTrustedRoleClaimers(String endpoint, String roleName) {
        return getTrustedRoleClaimers(endpoint, of(roleName));
    }

    @Override
    public List<String> getTrustedRoleClaimers(String endpoint, Optional<String> roleName) {
        String sql = trustedRolesSql.orElseGet(
                () -> "SELECT * FROM sts_audconf.trustedCvr WHERE endpoint = '" + endpoint + "'" +
                        roleName.map(rn -> "AND (roleValue is NULL OR roleValue = '" + rn + "')").orElse(""));

        PreparedStatementSetter ps = trustedRolesSql
                .map(u -> (PreparedStatementSetter) preparedStatement -> preparedStatement.setString(1, endpoint))
                .orElse(u -> { });

        List<String> cvrs = new JdbcTemplate(dataSource).query(sql, ps, (rs, i) -> rs.getString("cvr"));
        log.debug("Found " + cvrs.size() + " trustedCvr for scope " + endpoint +
                roleName.map(r -> " and role name " + r).orElse(""));
        return cvrs;
    }
}