/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server;

import dk.sosi.seal.pki.CertificateStatus;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.pki.PKIException;
import org.apache.log4j.Logger;

import java.security.cert.X509Certificate;

public class CertificateStatusVerifier {
    private static Logger log = Logger.getLogger(CertificateStatusVerifier.class);
    private final Federation federation;

    public CertificateStatusVerifier(Federation federation) {
        this.federation = federation;
    }

    public void verifyCertificateStatus(X509Certificate oces, String faultActor) throws VerifyException {
        final CertificateStatus status;
        try {
            status = federation.getCertificateStatus(oces);
        } catch (PKIException e) {
            throw new VerifyException(StsFaultCodeValues.WST_FAILED_AUTHENTICATION,
                    StsFaultStringValues.WST_FAILED_AUTHENTICATION + ": Certificate invalid (" + e.getMessage() +")",
                    faultActor);
        }
        if (status.isValid()) {
            if (status.getTimestamp() == null) {
                // shouldn't happen: Seal validates the certificate but does not provide a CRL timestamp
                log.warn("Missing timestamp in certificate status");
            }
            return;
        } else if (status.getTimestamp() != null) {
            throw new VerifyException(StsFaultCodeValues.WST_FAILED_AUTHENTICATION,
                    StsFaultStringValues.WST_FAILED_AUTHENTICATION + ": Certificate revoked (" + status.getTimestamp() + ")",
                    faultActor);
        } else {
            throw new VerifyException(StsFaultCodeValues.WST_FAILED_AUTHENTICATION,
                    StsFaultStringValues.WST_FAILED_AUTHENTICATION + ": Certificate expired or not yet valid",
                    faultActor);
        }
    }

}
