/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server;

import dk.sosi.seal.model.ModelException;
import dk.sosi.seal.model.OIOSAMLAssertion;
import dk.sosi.seal.model.OIOWSTrustRequest;
import dk.sosi.seal.modelbuilders.ModelBuildException;
import org.apache.log4j.Logger;

/**
 * Created by Kent on 20-02-2017.
 */
public class OIOWSTrustValidation {
    private final Logger log = Logger.getLogger(getClass());

    private final String faultActor;

    public OIOWSTrustValidation(String faultActor) {
        this.faultActor = faultActor;
    }

    public void validateSignature(OIOWSTrustRequest req) throws VerifyException {
        try {
            req.validateSignature();
        } catch (ModelException me) {
            log.warn("Unable to validate envelope signature", me);
            throw new VerifyException(StsFaultCodeValues.WST_INVALID_REQUEST, "Unable to validate envelope signature", faultActor);
        } catch (ModelBuildException me) {
            log.warn("Unable to validate envelope signature", me);
            throw new VerifyException(StsFaultCodeValues.WST_INVALID_REQUEST, "Unable to validate envelope signature", faultActor);
        }
    }

    protected void validateAssuranceLevel(OIOSAMLAssertion assertion) throws VerifyException {
        if (!acceptAssuranceLevel(assertion.getAssuranceLevel())) {
            throw new VerifyException(StsFaultCodeValues.WST_ASSURANCE_LEVEL, StsFaultStringValues.WST_ASSURANCE_LEVEL, faultActor);
        }
    }

    /**
     * OIOSAML definerer AssuranceLevel som string med følgende tilladte værdier: “1”, “2”, “3”, “4” og “test”, så lad
     * os holde det som string i Seal - men i STS synes jeg vi skal vedtage at det kun er "3" og "4" der understøttes og
     * lad kald med et andet AssuranceLevel fejle.
     */
    private boolean acceptAssuranceLevel(final String assuranceLevel) {
        if (assuranceLevel == null || "test".equals(assuranceLevel)) {
            return false;
        }
        try {
            int assuranceLevelInt = Integer.parseInt(assuranceLevel);
            switch (assuranceLevelInt) {
                case 3:
                case 4:
                    return true;
                default:
                    return false;
            }
        } catch (NumberFormatException n) {
            return false;
        }
    }


}
