/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server;

import java.util.Collections;
import java.util.Map;
import java.util.Properties;
import dk.sosi.sts.model.Configuration;

public class StsConfig extends Configuration {
	
	private final Map<String, Object> resources;

  public StsConfig() {
    this(new Properties());
  }

  public StsConfig(final Properties props) {
    super(props);
    this.resources = Collections.emptyMap();
  }

  public StsConfig(Properties props, Map<String, Object> resources) {
    super(props);
    this.resources = resources;
  }

  public StsConfig(StsConfig config, Map<String, Object> resources) {
    this(config.data, resources);
  }

  /**
   * Returns an <code>Object</code> if and only if a value given by <code>key</code>
   * exists, otherwise <code>null</code> is returned.
   *
   * @param key the name of the key
   * @return the resource associated with this key
   */
  public Object getResource(String key) {
    return resources.get(key);
  }

}
