/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server;

import static dk.sosi.sts.server.StsFaultStringValues.WST_FAILED_AUTHENTICATION;

import java.text.MessageFormat;
import java.util.Collection;

import dk.sosi.sts.model.Authorization;

@SuppressWarnings("serial")
public class VerifyException extends StsFaultException {
	
	public static final String AUT_MISSING = WST_FAILED_AUTHENTICATION + ": missing authorization";
	public static final String AUT_NOT_MATCHED = WST_FAILED_AUTHENTICATION + ": authorization not found (available {0})";
	public static final String AUT_MULTIPLE_FOUND = WST_FAILED_AUTHENTICATION + ": multiple authorizations found ({0})";
	public static final String ROLE_INVALID = WST_FAILED_AUTHENTICATION + ": invalid role supplied ({0})";
	public static final String ROLE_MISSING = WST_FAILED_AUTHENTICATION + ": user does not have the requested role ({0})";
	public static final String ROLE_MULTIPLE_FOUND = WST_FAILED_AUTHENTICATION + ": multiple national roles found ({0})";

	public VerifyException(String faultCode, String faultString, String faultActor) {
		super(faultCode, faultString, faultActor);
	}
	
	public static VerifyException createMissingAuthorization() {
		final String message = AUT_MISSING;
		return failedAuth(message);
	}


	/**
	 * The faultString inclosed in the {@link VerifyException} contains a structured string
	 * of the {@link Authorization}s found for the subject.
	 */
	public static VerifyException createAuthorizationNotMatched(Collection<Authorization> auts) {
		return failedAuth(MessageFormat.format(AUT_NOT_MATCHED, auts));
	}

	/**
	 * The faultString inclosed in the {@link VerifyException} contains a structured string
	 * of the {@link Authorization}s that matched for the subject.
	 */
	public static VerifyException createAuthorizationMultipleFound(Collection<Authorization> auts) {
		return failedAuth(MessageFormat.format(AUT_MULTIPLE_FOUND, auts));
	}

	public static VerifyException failedAuth(String message) {
		return new VerifyException(StsFaultCodeValues.WST_FAILED_AUTHENTICATION,
				message,
				StsFaultActorValues.AUTHORIZATION);
	}
}
