package dk.sosi.sts.server;

import dk.sosi.seal.pki.DistinguishedName;
import dk.sosi.seal.pki.PKIException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.certificate.OcesInfoParser;
import dk.sosi.sts.certificate.OcesInfo.CertificateType;
import dk.sosi.sts.model.SubjectSerialNumber;

public class VerifyUtil {
	public static DistinguishedName verifyDN(String subjectNameID) throws VerifyException{
        try {
            return new DistinguishedName(subjectNameID);
        } catch (PKIException e) {
            throw new VerifyException(StsFaultCodeValues.WST_INVALID_REQUEST, e.getMessage(), StsFaultActorValues.ITS);
        }
	}
	
	public static SubjectSerialNumber verifySSN(String subjectNameID) throws VerifyException {
		return verifySSN(verifyDN(subjectNameID));
    }
	
	public static SubjectSerialNumber verifySSN(DistinguishedName dn) throws VerifyException {
		return new SubjectSerialNumber(dn.getSubjectSerialNumber());
    }
	
	public static SubjectSerialNumber verifyCitizenToken(String subjectNameID) throws VerifyException{
		SubjectSerialNumber subjectSerialNumber = verifySSN(subjectNameID);        
        OcesInfo ocesInfo = OcesInfoParser.parseSubjectSerialNumber(subjectSerialNumber);
        if (ocesInfo.getCertificateType() != CertificateType.POCES) {
            throw new VerifyException(StsFaultCodeValues.WST_INVALID_REQUEST, "Token not issued to a citizen " + subjectSerialNumber, StsFaultActorValues.ITS);
        }
        return ocesInfo.getSubjectSerialNumber();
	}
}
