/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.aop;

import org.aopalliance.intercept.MethodInterceptor;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;

import java.util.Arrays;

/**
 *
 */
public class StsAopProxySetup implements InitializingBean {

	private final Logger log = Logger.getLogger(StsAopProxySetup.class);

	private Object[] targets;
	private MethodInterceptor interceptor;

	public Object[] getTargets() {
		return targets;
	}

	public void setTargets(final Object[] targets) {
		this.targets = targets;
	}

	public MethodInterceptor getInterceptor() {
		return interceptor;
	}

	public void setInterceptor(final MethodInterceptor interceptor) {
		this.interceptor = interceptor;
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		for (Object target : targets) {
			if (!(target instanceof StsProxy)) {
				throw new IllegalArgumentException("Bean of class " + target.getClass().getName() + " is not a StsProxy.");
			}

			add(target.getClass().getSuperclass(), interceptor);
		}
	}

	private void add(final Class<?> clazz, final MethodInterceptor interceptor) {
		final MethodInterceptor[] methodInterceptors = StsProxyFactory.globalInterceptors.get(clazz);
		if (methodInterceptors == null) {
			StsProxyFactory.globalInterceptors.put(clazz, new MethodInterceptor[]{interceptor});
		} else {
			final MethodInterceptor[] interceptors = Arrays
					.copyOf(methodInterceptors, methodInterceptors.length + 1);
			interceptors[methodInterceptors.length] = interceptor;
			StsProxyFactory.globalInterceptors.put(clazz, interceptors);
		}
	}
}
