package dk.sosi.sts.server.audit;

import dk.sosi.seal.xml.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.security.cert.CertificateException;
import java.util.Properties;

public class EnvelopeCompressor {
    private X509CertificateCompressor certificateCompressor;

    public EnvelopeCompressor(X509CertificateCompressor certificateCompressor) {
        this.certificateCompressor = certificateCompressor;
    }

    public String compress(String envelope) {
        if(envelope == null) {
            throw new IllegalArgumentException("Provided envelope was null!");
        }

        return compress(parse(envelope));
    }

    public String compress(Document envelope) {
        if(envelope == null) {
            throw new IllegalArgumentException("Provided envelope was null!");
        }

        // Look for nodes that we think contain a certificate, and for those, compress the value.
        NodeList certNodes = getCompressibleNodes(envelope);
        for(int i = 0; i < certNodes.getLength(); i++) {
            Node n = certNodes.item(i);
            compressNode(n);
        }

        return serialize(envelope);
    }

    private Document parse(String envelope) {
        try {
            boolean validate = false;
            return XmlUtil.readXml(new Properties(), envelope, validate);
        }
        catch(Exception e) {
            throw new IllegalArgumentException(String.format("Malformed envelope: %s", envelope), e);
        }
    }

    private String serialize(Document doc) {
        boolean pretty = false;
        boolean includeXMLHeader = true;
        return XmlUtil.node2String(doc, pretty, includeXMLHeader);
    }

    private NodeList getCompressibleNodes(Document envelope) {
        // For now, look exclusively for X509Certificate nodes.
        return envelope.getElementsByTagNameNS("http://www.w3.org/2000/09/xmldsig#", "X509Certificate");
    }

    private void compressNode(Node n) {
        String certString = n.getTextContent();
        try {
            String compressed = certificateCompressor.compress(certString);
            n.setTextContent(compressed);
        }
        catch(CertificateException ce) {
            throw new IllegalArgumentException(String.format("Error during envelope compression. Could not compress certificate string %s", certString), ce);
        }
    }
}
