package dk.sosi.sts.server.audit;

import java.io.ByteArrayInputStream;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

public class X509CertificateCompressor {
    private CertificateFactory certificateFactory;

    public X509CertificateCompressor() {

    }

    public String compress(String encodedCertificate) throws CertificateException {
        if(encodedCertificate == null) {
            throw new IllegalArgumentException("Cannot compress null!");
        }
        if(certificateFactory == null) {
            init();
        }

        X509Certificate cert = null;
        try {
            String enclosed = ensureEnclosingLines(encodedCertificate);
            // This cast is guaranteed to succeed, per the documentation of generateCertificate.
            cert = (X509Certificate)certificateFactory.generateCertificate(new ByteArrayInputStream(enclosed.getBytes()));
        }
        catch(CertificateException ce) {
            throw new IllegalArgumentException(String.format("Not a valid base64 encoded certificate: %s", encodedCertificate), ce);
        }

        return cert.getSubjectX500Principal().toString();
    }

    private void init() throws CertificateException {
        certificateFactory = CertificateFactory.getInstance("X.509");
    }


    private String ensureEnclosingLines(String cert) {
        if(cert.startsWith("-----BEGIN CERTIFICATE-----\n") && cert.endsWith("\n-----END CERTIFICATE-----")) {
            return cert;
        }
        return "-----BEGIN CERTIFICATE-----\n" + cert + "\n-----END CERTIFICATE-----";
    }
}
