/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.log4j.Logger;

import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.server.StsConfig;
import dk.sosi.sts.service.AclService;
import dk.sosi.sts.util.DbUtil;

public class StsAclExtendedService implements AclService {
	
	private Logger log = Logger.getLogger(StsAclExtendedService.class);
	private final DataSource dataSource;
	
	public StsAclExtendedService(DataSource ds) {
		this.dataSource = ds;
	}

	public boolean isBlacklisted(OcesInfo ocesInfo, String systemName) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		boolean isBlacklisted;
		try {
			conn = dataSource.getConnection();
			pstmt = conn.prepareStatement("SELECT * FROM blacklist WHERE ssn = ?");
			pstmt.setString(1, ocesInfo.getSubjectSerialNumber().getValue());
			
			ResultSet rs = pstmt.executeQuery();
			isBlacklisted = rs.next();
			rs.close();
		} catch (SQLException e) {
			throw new RuntimeException("Error checking blacklist", e);
		} finally {
			DbUtil.close(pstmt, log);
			DbUtil.close(conn, log);
		}
		
		return isBlacklisted;
	}

	public boolean isWhitelisted(OcesInfo ocesInfo, String systemName) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		boolean isWhitelisted;
		try {
			conn = dataSource.getConnection();
			pstmt = conn.prepareStatement("SELECT * FROM whitelist WHERE ssn = ?");
			pstmt.setString(1, "CVR:" + ocesInfo.getCvr() + "-SYSTEM:" + systemName);
			ResultSet rs = pstmt.executeQuery();
			isWhitelisted = rs.next();
			rs.close();
		} catch (SQLException e) {
			throw new RuntimeException("Error checking whitelist", e);
		} finally {
			DbUtil.close(pstmt, log);
			DbUtil.close(conn, log);
		}
		
		return isWhitelisted;
	}
}
