/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service;

import java.util.List;

import javax.sql.DataSource;

import org.apache.log4j.Logger;

import dk.sosi.sts.StsException;
import dk.sosi.sts.roles.authorization.DbAuthorizationService;
import dk.sosi.sts.model.Authorization;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.service.AuthorizationService;

public class StsAuthorizationService implements AuthorizationService {
	private final Logger log = Logger.getLogger(this.getClass());
	private final DbAuthorizationService service;
	private final DataSource ds;

	public StsAuthorizationService(DataSource ds, String sql) {
		this.ds = ds;
		if (ds == null)
			throw new RuntimeException("Missing datasource for StsAuthorizationService");
		service = new DbAuthorizationService(ds, checkSql(sql));

	}

	public StsAuthorizationService(DataSource ds) {
		this.ds = ds;
		if (ds == null)
			throw new RuntimeException("Missing datasource for StsAuthorizationService");
		service = new DbAuthorizationService(ds);
	}

	public List<Authorization> getAuthorizations(CprNr cpr) throws StsException {
		return service.getAuthorizations(cpr);
	}

	private String checkSql(String select) {
		if (select == null || select.trim().isEmpty()) {
			throw new IllegalArgumentException(select);
		}
		log.info("Using custom select for '" + StsAuthorizationService.class.getSimpleName() + "':" + select);
		return select;
	}
}
