/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.ocescpr.cvrrid.service.impl.MockOcesCvrRidService;

/**
 * To set up:
 * <bean id="cvrRidService" class="dk.sosi.sts.server.service.StsMockOcesCvrRidService">
 * <constructor-arg type="java.lang.String" value="[CVR:19343634-RID:1165813849809 1111111118] [CVR:19343634-UID:1165814083652 1111111119] [CVR:19343634-UID:1165813731180 1111111120] [CVR:19343634-UID:1165813676175 1111111121] [CVR:25767535-RID:1118061043356 1111111118] [CVR:26912865-RID:82139564 1602763141] [CVR:26912865-RID:76426064 1111111118] [CVR:26912865-RID:66294347 1602760037]"/>
 * </bean>
 */
public class StsMockOcesCvrRidService  extends MockOcesCvrRidService {

	public StsMockOcesCvrRidService(String config) {
		super(createMap(config));
	}

	/**
	 * Creates a map of <code>CvrRid</code> and <code>CprNr</code> associations
	 * from a sequence of cvrRid/cpr pairs on the following form:
	 * 
	 * <pre>
	 *   [CVRRID1 CPR1] [CVRRID2 CPR2] ....
	 * </pre>
	 * 
	 * @param input a string with a sequence of cvrRid-cpr pairs
	 * @return a map containing the 
	 */
	public static Map<SubjectSerialNumber, CprNr>createMap(String input) {
		if (input == null) {
			return Collections.<SubjectSerialNumber, CprNr>emptyMap();
		}
		HashMap<SubjectSerialNumber, CprNr> map = new HashMap<SubjectSerialNumber, CprNr>();
		
	    String patternStr = "\\s*\\[(\\S+)\\s+(\\S+)\\]";
	    Pattern pattern = Pattern.compile(patternStr);
	    Matcher matcher = pattern.matcher(input);
	    
	    while (matcher.find()) {
	    	map.put(new SubjectSerialNumber(matcher.group(1)), new CprNr(matcher.group(2)));
	    }
	    
	    return map;
	}
}
