/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.test;

import dk.sosi.sts.ocescpr.cvrrid.service.impl.WsOcesCvrRidService;
import dk.sosi.sts.timing.Timer;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.Controller;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
 *
 */
public class CheckEndpointsServlet extends HttpServlet implements Controller {

	private Map<String, WsOcesCvrRidService> wsEndpoints;

	public CheckEndpointsServlet(Map<String, WsOcesCvrRidService> wsEndpoints) {
		this.wsEndpoints = wsEndpoints;
	}

	@Override
	protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
		final Checker rChecker = new Checker() {
			@Override
			protected void output(final String s) {
				try {
					resp.getWriter().println(s);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
			}
		};
		final String action = req.getParameter("action");

		resp.setContentType("text/plain");
		resp.setStatus(200);

		if (action == null) {
			displayMessage(resp);
			return;
		}
		Timer.start(getClass(), "doGet");
	    try {
	    	if ("check".equals(action)) {
	    		// 3. cvr-rid access
	    		rChecker.checkCvrRidAccess(wsEndpoints);
	    	}
	    } finally {
			Timer.stop(getClass(), "doGet");
	    }
	}

	private void displayMessage(final HttpServletResponse resp) throws IOException {
		resp.getWriter().append("No 'action' specified.");
	}

	@Override
	public ModelAndView handleRequest(final HttpServletRequest request, final HttpServletResponse response) throws Exception {
		service(request, response);
		return null;
	}
}
