/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.timer;

import dk.sosi.sts.util.StsUtil;
import org.springframework.beans.factory.InitializingBean;

import java.util.Timer;
import java.util.TimerTask;

/**
 *
 */
public class AddStsTimer implements InitializingBean {

	private TimerTask timerTask;
	private Timer timer;
	private long delay;
	private long period;
	private boolean oneTime;

	public void setTimerTask(final TimerTask timerTask) {
		this.timerTask = timerTask;
	}

	public void setTimer(final Timer timer) {
		this.timer = timer;
	}

	public void setDelay(final long delay) {
		this.delay = delay;
	}

	public void setPeriod(final long period) {
		this.period = period;
	}

	public void setOneTime(boolean oneTime) {
		this.oneTime = oneTime;
	}
	@Override
	public void afterPropertiesSet() throws Exception {
		checkState();

		if (oneTime)
			timer.schedule(timerTask, delay);
		else
			timer.scheduleAtFixedRate(timerTask, delay,period);
	}

	private void checkState() {
		StsUtil.checkNotNull(timerTask, "timerTask may not be null");
		StsUtil.checkNotNull(timer, "timer may not be null");
		if (!oneTime && period < 1) {
			throw new IllegalStateException("Period should be larger than 0 for non-one-timers.");
		}
	}
}
