package dk.sosi.sts.server.timer;

import java.util.TimerTask;

import dk.sosi.sts.monitor.Check;
import dk.sosi.sts.monitor.Check.CheckResult;
import dk.sosi.sts.server.service.StsOcesCrlService;

public class CheckInvalidCertificateTask extends TimerTask {
	
	private final Check monitor;
	private final StsOcesCrlService service;
	private final Long sleep;
	
	public CheckInvalidCertificateTask(final Check monitor,  final StsOcesCrlService service, Long sleep) {
		this.monitor = monitor;
		this.service = service;
		this.sleep = sleep;
	}
	
	@Override
	public void run() {	
		// We try 50 times otherwise wait for next start
		int outOfControlBrake = 50;
		while(isFailed(monitor.performCheck()) && outOfControlBrake > 0) {
			service.refresh();
			try {
				Thread.sleep(sleep);
			} catch (InterruptedException e) {
				//We do nothing. Trying again
			}
			outOfControlBrake--;
		}
	}
	
	private boolean isFailed(CheckResult performCheck) {
		return performCheck.failed && !isSame(performCheck.result.get("Sts certificate valid"), "OK");
		
	}
	
	private boolean isSame(String a, String b) {
		return a != null ? a.equals(b) : b == null;
	}

}
