/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.server.timer;

import java.util.Collections;
import java.util.List;
import java.util.TimerTask;

import org.apache.log4j.Logger;

import dk.sosi.sts.server.cache.DefinitionServiceCache;

public class DefinitionServiceCacheReloader extends TimerTask {
	private final Logger log = Logger.getLogger(this.getClass());
    private List<DefinitionServiceCache> services;

    // Invoked by spring
    public void setService(DefinitionServiceCache service){
    	setServices(Collections.singletonList(service));	
    }
    
    // invoked by spring
    public void setServices(List<DefinitionServiceCache> services) {
        this.services = services;
    }

    @Override
    public void run() {
        for (DefinitionServiceCache definitionServiceCache : services) {
        	try {
        		definitionServiceCache.refresh();
        	} catch(RuntimeException ex) {
        		log.warn("Error refreshing cache "+definitionServiceCache.getCacheId(), ex);
        	}
		}
    }
}
