/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.timer;

import dk.sosi.seal.pki.CRLCache;
import dk.sosi.sts.server.service.StsOcesCrlService;
import org.apache.log4j.Logger;

import java.util.Date;
import java.util.Map;
import java.util.TimerTask;

/**
*
*/ // post start asynchronous sequential prefetch of configured crls
public class PrePopulateCrl extends TimerTask {
	private Logger log = Logger.getLogger(PrePopulateCrl.class);

	private final CRLCache cache;
	private final Map<String, String> prefetchProperties;
	private final StsOcesCrlService ocesCrlService;

	PrePopulateCrl(final StsOcesCrlService ocesCrlService,
				   final CRLCache cache,
				   final Map<String, String> prefetchProperties) {
		this.cache = cache;
		this.prefetchProperties = prefetchProperties;
		this.ocesCrlService = ocesCrlService;
	}

	@Override
	public void run() {

		for (Map.Entry<String, String> conf : prefetchProperties.entrySet()) {
			final String url = conf.getValue();
			if (cache.get(url) == null) {
				log.debug("Pre-populating CRL cache (" + conf.getKey() + ") from URL " + url);
				try {
					final CRLCache.CRLInfo load = ocesCrlService.loadFromURL(url, null);
					if (load != null) {
						cache.update(url, load);
						log.debug("Updated: lastModified = " + new Date(load.getLastModified()));
					}
				} catch (Throwable e) {
					log.error("Error in pre-populate:", e);
				}
			}
		}
	}
}
