/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.user.data;

import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.UserData;
import dk.sosi.sts.service.UserDataService;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.RowMapper;

import javax.sql.DataSource;

import java.util.List;

public class DbUserDataService implements UserDataService {
    private DataSource dataSource;
    private String userDataSql = "SELECT * FROM sts_audconf.userData WHERE cpr = ?";

	
    // injected by spring
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    // injected by spring
    public void setSql(String sql) {
        this.userDataSql = sql;
    }

	@Override
	public List<UserData> getUserInfo(CprNr cprNr) {
        JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
        PreparedStatementSetter setter = statement -> statement.setString(1, cprNr.getValue());
        RowMapper<UserData> rowMapper = (rs, i) -> {
            final String cpr = rs.getString("cpr");
            final String given = rs.getString("givenName");
            final String sur = rs.getString("surName");
            return new UserData(new CprNr(cpr), given, sur);
        };
        return jdbcTemplate.query(userDataSql, setter, rowMapper);
	}

}
