/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.pki;

import java.net.URI;
import java.security.cert.X509Certificate;
import java.util.List;

/**
 * These are methods hijacked from Seal's internal API. They might change!
 *
 */
public class SealInternalApi {
	public static boolean isOces2Certificate(X509Certificate cert) {
		return OCESUtil.isProbableOCES2Certificate(cert);
	}

	public static boolean isIntermediateCertificate(X509Certificate cert) {
		return OCESUtil.isProbableIntermediateOrRootCertificate(cert) && !isOCES2RootCertificate(cert);
	}

	public static URI getIntermediateCertificateURL(X509Certificate cert) {
		return OCESUtil.retrieveIntermediateCertificateURI(cert);
	}

	public static boolean isEndCertificate(X509Certificate cert) {
		return OCESUtil.isProbableOCES2Certificate(cert) && !OCESUtil.isProbableIntermediateOrRootCertificate(cert);
	}

    public static boolean isOCES2RootCertificate(X509Certificate cert) {
        return cert == getOces2RootPPCertificate() || cert == getOces2RootPRCertificate();
    }


  public static boolean isIssuerOf(X509Certificate child, X509Certificate parent) {
		return OCESUtil.isIssuerOf(child, parent);
	}


	public static X509Certificate getOces2RootPRCertificate() {
		return OCESCertificationAuthority.OCES_2_ROOT_CERTIFICATE;
	}

	public static X509Certificate getOces2RootPPCertificate() {
		return OCESTestCertificationAuthority.OCES_2_TEST_PP_ROOT_CERTIFICATE;
	}
	
	public static List<X509Certificate> getOces3RootPPCertificates() {
		return OCESTestCertificationAuthority.OCES_3_TEST_ROOT_CERTIFICATES;
	}
}
