/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.monitor;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.mockito.Mockito;

import junit.framework.TestCase;

/**
 *
 */
public class DBCheckTest extends TestCase {

	public void testPerform() throws Exception {
		final DataSource mockDs = Mockito.mock(DataSource.class);
		final String sql = "select 1";
		final DBCheck dbCheck = new DBCheck(sql, mockDs, "MOCK-DB");

		final Connection mockConn = Mockito.mock(Connection.class);
		final PreparedStatement mockPstmt = Mockito.mock(PreparedStatement.class);
		mockPstmt.execute();
		mockPstmt.close();
		mockConn.close();
		Mockito.when(mockConn.prepareStatement(sql)).thenReturn(mockPstmt);
		Mockito.when(mockDs.getConnection()).thenReturn(mockConn);

		final Check.CheckResult checkResult = dbCheck.performCheck();

		assertEquals(1, checkResult.result.size());

		assertEquals("OK", checkResult.result.values().iterator().next());

		assertFalse(checkResult.failed);
	}

	public void testPerformWithError() throws Exception {
		final DataSource mockDs = Mockito.mock(DataSource.class);
		final String sql = "select 1";
		final DBCheck dbCheck = new DBCheck(sql, mockDs, "MOCK-DB");

		final Connection mockConn = Mockito.mock(Connection.class);
		final PreparedStatement mockPstmt = Mockito.mock(PreparedStatement.class);
		Mockito.when(mockPstmt.execute()).thenThrow(new SQLException("Fabricated"));
		Mockito.when(mockConn.prepareStatement(sql)).thenReturn(mockPstmt);
		Mockito.when(mockDs.getConnection()).thenReturn(mockConn);

		final Check.CheckResult checkResult = dbCheck.performCheck();

		assertEquals(1, checkResult.result.size());

		assertEquals("No connection", checkResult.result.values().iterator().next());
		assertTrue(checkResult.failed);
	}

	public void testPerformNoDataSource() throws Exception {
		final String sql = "select 1";
		final DBCheck dbCheck = new DBCheck(sql, null, "MOCK-DB");


		final Check.CheckResult checkResult = dbCheck.performCheck();

		assertEquals(1, checkResult.result.size());

		assertEquals("Not configured", checkResult.result.values().iterator().next());

		assertTrue(checkResult.failed);
	}
}
