/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.roles.nationalroles;

import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.model.RoleDefinition;
import dk.sosi.sts.model.SubjectSerialNumber;
import dk.sosi.sts.server.test.TestIdentity;
import dk.sosi.sts.server.test.TestIdentityProvider;
import dk.sosi.sts.service.NationalRoleService;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class MockNationalRolesService implements NationalRoleService {
    private MultiValueMap<SubjectSerialNumber, String> roles = new LinkedMultiValueMap<>();

    public void put(SubjectSerialNumber subjectSerialNumber, RoleDefinition roleDefinition) {
        roles.add(subjectSerialNumber, roleDefinition.getExternalName());
    }

    @Override
    public List<String> getRoles(OcesInfo ocesInfo) {
        List<RoleDefinition> identityRoles = TestIdentityProvider.IDENTITIES.stream()
                .filter(identity -> identity.getSubjectSerialNumber().equals(ocesInfo.getSubjectSerialNumber().getValue()))
                .map(TestIdentity::getNationalRoles)
                .findAny()
                .orElse(Collections.emptyList());
        List<String> identityStringRoles = identityRoles.stream().map(RoleDefinition::getExternalName).collect(Collectors.toList());
        return Optional.ofNullable(roles.get(ocesInfo.getSubjectSerialNumber())).orElse(identityStringRoles);
    }
}
