/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server;

import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.certificate.OcesInfo.Version;
import dk.sosi.sts.server.test.SealPPCredentialVaults;
import dk.sosi.sts.store.SimulatedMocesStore;

public class CertificateStore {

	public static X509Certificate getRevokedMoces(Version version) throws Exception {
        return SealPPCredentialVaults.getRevokedMocesVault(version).getSystemCredentialPair().getCertificate();
    }

	public static X509Certificate getValidMoces(Version version) throws Exception {
        return SealPPCredentialVaults.getValidMocesWithCpr(version).getSystemCredentialPair().getCertificate();
    }

	public static X509Certificate getExpiredMoces(Version version) throws Exception {
        return SealPPCredentialVaults.getExpiredMocesVault(version).getSystemCredentialPair().getCertificate();
    }

	public static X509Certificate getValidVoces(Version version) throws Exception {
        return getValidVocesVault(version).getSystemCredentialPair().getCertificate();
    }

    public static GenericCredentialVault getValidVocesVault(Version version) {
        return SealPPCredentialVaults.getValidVocesVault(version);
    }

    public static X509Certificate getValidFoces(Version version) throws Exception {
        return getValidFocesVault(version).getSystemCredentialPair().getCertificate();
    }

    public static GenericCredentialVault getValidFocesVault(Version version) {
        return SealPPCredentialVaults.getValidFocesVault(version);
    }
    
    /**
     * Specifikt POCES2 certifikat
     * @return
     * @throws CertificateException
     */
    public static X509Certificate getSusanneAndersen() throws CertificateException {
    	return SimulatedMocesStore.getSusanneAndersen();
    }
}
