/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server;

import dk.sosi.seal.model.OIOSAMLAssertion;
import dk.sosi.seal.test.TestParameterBuilder;
import dk.sosi.sts.certificate.OcesInfo.Version;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;
import org.mockito.Mockito;

import static org.junit.Assert.fail;

@RunWith(Parameterized.class)
public class NboAssuranceLevelTest {
	@Parameter(0)
	public String suiteName;

	@Parameter(1)
	public Version version;

	/* Setting up test parameters */
	@Parameters(name = "{0}")
	public static Iterable<Object[]> data() {
		TestParameterBuilder builder = new TestParameterBuilder();

		builder.newTest("OCES3").add(Version.OCES3);

		return builder.build();
	}

	private NboRequestHandler makeHander() {
		final NboRequestHandler handler = new NboRequestHandler();

		final NboConfig nboConfig = Mockito.mock(NboConfig.class);
		Mockito.when(nboConfig.getTrustedVault()).thenReturn(CertificateStore.getValidVocesVault(version));
		handler.setNboConfig(nboConfig);

		return handler;
	}

	private void verify(String value) throws VerifyException {
		final NboRequestHandler handler = makeHander();

		final OIOSAMLAssertion mockToken = Mockito.mock(OIOSAMLAssertion.class);

		Mockito.when(mockToken.getAssuranceLevel()).thenReturn(value);

		handler.validateAssertion(mockToken);
	}

	@Test
	public void test3() throws VerifyException {
		verify("3");
	}

	@Test
	public void test4() throws VerifyException {
		verify("4");
	}

	@Test
	public void test1() throws VerifyException {
		try {
			verify("1");
			fail("Should throw exception");
		} catch (VerifyException e) {
		}
	}

	@Test
	public void test2() throws VerifyException {
		try {
			verify("2");
			fail("Should throw exception");
		} catch (VerifyException e) {
		}
	}

	@Test
	public void testTest() throws VerifyException {
		try {
			verify("test");
			fail("Should throw exception");
		} catch (VerifyException e) {
		}
	}

	@Test
	public void testOther() throws VerifyException {
		try {
			verify("nothing good");
			fail("Should throw exception");
		} catch (VerifyException e) {
		}
	}
}
