/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.server.facade;

import dk.sosi.seal.pki.CertificateInfo;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.certificate.OcesInfo.Version;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.server.CertificateStore;
import dk.sosi.sts.server.test.StsCredentialVaults;

/**
 * Created by Kent on 20-03-2017.
 */
public class DefaultSamlTokenTestModel implements SamlTokenTestModel {
	public final static CprNr EXISTING_CPR = new CprNr("0501792275");
	private final CertificateInfo DEFAULT_CERT_INFO;
	
	protected static final String CRYPT_AUDIENCE = "http://audience/crypt";
	protected static final String CLEAR_AUDIENCE = "http://audience/clear";
	protected static final String CLEAR_AUDIENCE_NO_WARD = "http://audience/noWardWhitelisting";
	protected static final String BLURRING_AUDIENCE = "http://audience/blurring";
	protected static final String SBO3_AUDIENCE = "http://audience/sbo3";
	protected static final String SBO3_AUDIENCE_NO_BEST = "http://audience/sbo3_noBST";
	
	protected final Version version;
	
	public DefaultSamlTokenTestModel(Version version) {
		this.version = version;

		DEFAULT_CERT_INFO = CertificateInfo.fromString(
				"SubjectDN={SERIALNUMBER=UI:DK-P:G:23550132-5e1f-4e43-a5f9-048acf49e0b8, O=Testorganisation nr. " +
						"94354969, C=DK},IssuerDN={CN=Den Danske Stat OCES udstedende-CA 1, O=Den Danske Stat, C=DK}," +
						"CertSerial={314811519240226669371527400489320833750098446007}");
	}
	
	@Override
	public Version getVersion() {
		return version;
	}

	@Override
	public CredentialVault getExternalTrustedIdp() {
		return StsCredentialVaults.getExternalTrustedIdp(version);
	}

	public CredentialVault getRandomMocesWithCvrTrustedNationalRole() {
		return StsCredentialVaults.getRandomMocesWithCvrTrustedNationalRole(version);
	}

	@Override
	public CredentialVault getAllTrustedCertificates() {
		return StsCredentialVaults.getAllTrustedCertificates(version);
	}

	@Override
	public GenericCredentialVault getValidWhitelistedVault() {
		return CertificateStore.getValidFocesVault(version);
	}

	public GenericCredentialVault getValidWhitelistedVocesVault() {
		return CertificateStore.getValidVocesVault(version);
	}

	@Override
	public CprNr getPatientCpr() {
		return EXISTING_CPR;
	}

	@Override
	public CertificateInfo getPatientCertificateInfo() {
		return DEFAULT_CERT_INFO;
	}

	@Override
	public String getClearTextAudience() {
		return CLEAR_AUDIENCE;
	}

	@Override
	public String getClearTextAudienceNoWardWhitelisting() {
		return CLEAR_AUDIENCE_NO_WARD;
	}

	@Override
	public String getCryptAudience() {
		return CRYPT_AUDIENCE;
	}
	
	@Override
	public String getSBOAudience() {
		return getSBOAudience(false);
	}
	
	@Override
	public String getSBOAudience(boolean includeBst) {
		return includeBst ? SBO3_AUDIENCE : SBO3_AUDIENCE_NO_BEST;
	}

	@Override
	public String getBlurringAudience() {
		return BLURRING_AUDIENCE;
	}

	public static String getSBOAudience(Version version, boolean includeBst) {
		return includeBst ? SBO3_AUDIENCE : SBO3_AUDIENCE_NO_BEST;
	}
}
