package dk.sosi.sts.server.facade;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Date;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.model.AbstractIdentityTokenResponse;
import dk.sosi.seal.model.IdentityToken;
import dk.sosi.seal.model.constants.DSTags;
import dk.sosi.seal.model.constants.SAMLTags;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.xml.XmlUtil;

/**
 * Created by Kent on 21-03-2017.
 */
public class IdentityTokenResponseValidation {
	public static void assertHolderOfKey(IdentityToken identityToken, X509Certificate certificate) throws CertificateEncodingException {
		final Document bootstrapTokenDom = identityToken.getDOM();
		final Element subject = OIOWSTrustResponseValidation.getFirstChild(bootstrapTokenDom.getDocumentElement(), SAMLTags.subject);
		final Element subjectConfirmation = OIOWSTrustResponseValidation.getFirstChild(subject, SAMLTags.subjectConfirmation);
		assertEquals("urn:oasis:names:tc:SAML:2.0:cm:holder-of-key", subjectConfirmation.getAttributes().getNamedItem("Method").getNodeValue());
		
		final Element subjectConfirmationData = OIOWSTrustResponseValidation.getFirstChild(subjectConfirmation, SAMLTags.subjectConfirmationData);
		final Element keyInfo = OIOWSTrustResponseValidation.getFirstChild(subjectConfirmationData, DSTags.keyInfo);
		final Element keyData = OIOWSTrustResponseValidation.getFirstChild(keyInfo, DSTags.x509Data);
		final Element x509Cert = OIOWSTrustResponseValidation.getFirstChild(keyData, DSTags.x509Certificate);
		
		final String signingCertBase64 = XmlUtil.toBase64(certificate.getEncoded());
		assertEquals(signingCertBase64, XmlUtil.getTextNodeValue(x509Cert));
	}

	public static void assertValidInTime(IdentityToken identityToken) {
		final Date now = new Date();
		assertTrue("not before: " + identityToken.getNotBefore(), identityToken.getNotBefore().before(now));
		assertTrue("not on or after: " + identityToken.getNotOnOrAfter(), identityToken.getNotOnOrAfter().after(now));
	}

	public static void assertFault(AbstractIdentityTokenResponse response, String requiredActor, String requiredErrorStringPart) {
		OIOWSTrustResponseValidation.assertFault(response, requiredActor, requiredErrorStringPart);
	}

	public static void assertFault(AbstractIdentityTokenResponse response, String requiredActor, String requiredFaultCode, String... requiredErrorStringParts) {
		OIOWSTrustResponseValidation.assertFault(response, requiredActor, requiredFaultCode, requiredErrorStringParts);
	}

	public static void assertSuccess(AbstractIdentityTokenResponse response, Federation federation) {
		OIOWSTrustResponseValidation.assertSuccess(response, federation);
	}
}
