/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.facade;

import dk.sosi.seal.model.constants.HealthcareSAMLAttributes;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.PrivilegeGroup;
import dk.sosi.sts.service.ProcurationService;

import static dk.sosi.sts.fuldmagt.stub.ProcurationTestModel.*;

public class ProcurationMockService implements ProcurationService {

    @Override
    public PrivilegeGroup getProcurationPrivileges(CprNr providedBy, CprNr representative) {
        PrivilegeGroup privilegeGroup = new PrivilegeGroup(HealthcareSAMLAttributes.THROUGH_PROCURATION_BY_CPR +
                                                                   ":" + providedBy.getValue());
        if (CPR_3.equals(representative.getValue())) {
            if (CPR_1.equals(providedBy.getValue())) { privilegeGroup.addPrivilege(FMK_READ); }
            if (CPR_2.equals(providedBy.getValue())) { privilegeGroup.addPrivilege(FMK_WRITE); }
        }
        if(representative.equals(new CprNr("1234567890")) || providedBy.equals(new CprNr("0987654321"))) {
                privilegeGroup.addPrivilege(FMK_READ);
        }
        return privilegeGroup;
    }

    @Override
    public long lastSuccess() {
        return 0;
    }

    @Override
    public long lastFailure() {
        return 0;
    }

    @Override
    public int getFailedCallsPercentage() { return 0; }

    @Override
    public int getHoursBackInTimeToCalculateFailedCallsPercentage() { return 0; }
}
