package dk.sosi.sts.server.facade;

import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import dk.sosi.sts.certificate.OcesInfo.Version;
import dk.sosi.sts.certificate.OcesInfoParser;
import dk.sosi.sts.model.RoleDefinition;
import dk.sosi.sts.server.test.StsCredentialVaults;
import dk.sosi.sts.server.test.TestIdentityProvider;
import dk.sosi.sts.service.RoleDefinitionService;

import static java.util.Optional.of;

public class RoleDefinitionServiceStub implements RoleDefinitionService {

    public static final RoleDefinition TESTROLLE2 = new RoleDefinition("99992", "TestRolle2", "nspTestRolle2");
    public static final RoleDefinition TESTROLLE1 = new RoleDefinition("99991", "TestRolle1", "nspTestRolle1");
    public static final RoleDefinition PLEJE = new RoleDefinition("41003", "PlejeAssR3", "nspPlejeAssR3");
    public static final RoleDefinition LAEGE_SEKRETAER = new RoleDefinition("41002", "SundAssistR2", "nspSundAssistR2");
    public static final RoleDefinition PORTOER = new RoleDefinition("41001", "SundAssistR1", "nspSundAssistR1");
    public static final RoleDefinition UKENDT = new RoleDefinition("99999", "Ukendt", "nspUkendt");

    private final Version version;
    
    public RoleDefinitionServiceStub(Version version){
    	this.version = version;
    }

    @Override
    public List<RoleDefinition> getRoleDefinitions() {
        return Arrays.asList(LAEGE_SEKRETAER, PORTOER, PLEJE, TESTROLLE1, TESTROLLE2);
    }

    @Override
    public List<String> getTrustedRoleClaimers(String endpoint, String roleName) {
        return getTrustedRoleClaimers(endpoint, of(roleName));
    }

    @Override
    public List<String> getTrustedRoleClaimers(String endpoint, Optional<String> roleName) {
        final X509Certificate randomSsiCertificate = StsCredentialVaults.getRandomMocesWithCvrTrustedNationalRole(version).getSystemCredentialPair().getCertificate();
        final String ssiCvr = OcesInfoParser.parseCertificate(randomSsiCertificate).getCvr();
        if ("STS".equals(endpoint)) {
            String trustedOces3Cvr = TestIdentityProvider.getIdentityWithTrustedCvrAndThreeAuthorizations(Version.OCES3).getCvr();
            return Collections.unmodifiableList(Arrays.asList(ssiCvr, trustedOces3Cvr));
        } else {
            return Collections.emptyList();
        }

    }
}