/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.facade;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.Optional;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import dk.sosi.sts.server.AbstractStsRequestHandler;
import dk.sosi.testtools.testrequest.MessageGeneratorException;

import static java.util.Optional.*;

public class TestClientHelper implements TestClient {

    public final AbstractStsRequestHandler<?, ?, ?> stsService;
	
	public TestClientHelper(AbstractStsRequestHandler impl) {
		stsService = impl;
	}

    public static Optional<String> getMessageId(Document doc) {
        final Node soapHeader = doc.getDocumentElement().getChildNodes().item(0);
        final NodeList headers = soapHeader.getChildNodes();
        for (int i = 0; i < headers.getLength(); i++) {
            final Node header = headers.item(i);
            if (header.getLocalName().equals("MessageID")) {
                return of(header.getTextContent());
            }
        }
        return empty();
    }


	@Override
  public Response execute(final String soapEnvelope) {
    try {
		final ByteArrayOutputStream resp = new ByteArrayOutputStream();
        boolean fault = stsService.handleRequest(soapEnvelope, resp);
		org.xml.sax.InputSource inputSource;
      try {
		  String asString = new String(resp.toByteArray(), "UTF-8");
		  //String sanitized = asString.replaceAll("\\>\\s+\\<", "><");  // could break signature
		  inputSource = new InputSource(new StringReader(asString));
	  } catch (Exception e) {
        throw new RuntimeException(e);
      }
      DocumentBuilderFactory factory = DocumentBuilderFactory
              .newInstance();
      factory.setValidating(false);
      factory.setNamespaceAware(true);
      final Document document;
      try {
        DocumentBuilder builder = factory.newDocumentBuilder();
        document = builder.parse(inputSource);

      } catch (SAXException sxe) {
        Exception x = ofNullable(sxe.getException()).orElse(sxe);
        throw new MessageGeneratorException(x.getMessage(), x);
      } catch (ParserConfigurationException pce) {
        throw new MessageGeneratorException(pce.getMessage(), pce);
      }
      return new Response(document, fault ? 500 : 200);
    } catch (IOException axisFault) {
      System.err.println("Error:");
      axisFault.printStackTrace();
      throw new RuntimeException(axisFault);
    } catch (Exception e) {
      System.err.println("Error:");
      e.printStackTrace();
      throw new RuntimeException(e);
    }
  }
}
