/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.facade.bootstrap;

import dk.sosi.seal.model.*;
import dk.sosi.seal.model.dombuilders.OIOBootstrapToIdentityTokenRequestDOMBuilder;
import dk.sosi.sts.server.Bootstrap2IdwsRequestHandler;
import dk.sosi.sts.server.NboConfig;
import dk.sosi.sts.server.facade.TestClient;
import dk.sosi.sts.server.facade.TestClientHelper;
import dk.sosi.sts.server.facade.sbo.SboTokenServiceEmbeddedTest;
import dk.sosi.sts.server.test.StsCredentialVaults;
import org.junit.*;
import org.w3c.dom.Document;

import static dk.sosi.sts.server.facade.IdentityTokenResponseValidation.assertSuccess;
import static org.junit.Assert.*;

public class BootstrapTokenProcurationLocalTest extends BootstrapTokenProcurationTest {
    protected TestClientHelper embeddedTestClient;
    private NboConfig nboConfig;
    private Bootstrap2IdwsRequestHandler requestHandler;

    public BootstrapTokenProcurationLocalTest() {
        super(true);
    }

    @Before
    public void setUp() throws Exception {
        EmbeddedBootstrapTokenToIdentityTokenBuilder builder = new EmbeddedBootstrapTokenToIdentityTokenBuilder(testModel, version);
        embeddedTestClient = builder.build();
        nboConfig = builder.getNboConfig();
        requestHandler = builder.getRequestHandler();
    }

    @Override
    protected TestClient getTestClient() {
        return embeddedTestClient;
    }

    @Test
    public void testProcurationServiceNotInvokedWithNoClaims() {
        requestHandler.setProcurationService(null);
        OIOBootstrapToIdentityTokenRequestDOMBuilder requestBuilder = createRequestDomBuilder(assertionBuilder.build(), testModel.getValidWhitelistedVault(),
                testModel.getClearTextAudience(), testModel.getPatientCpr().getValue(), false, encryptionKey);
        // no claims
        Document requestDocument = buildRequestDocument(requestBuilder);
               
        final Document responseDoc = getResponseDocument(requestDocument);

        OIOBootstrapToIdentityTokenResponse response = getResponseModel(responseDoc);
        assertFalse(response.getFaultString(), response.isFault());
        assertSuccess(response, federation);

        assertNotNull(response.getIdentityToken());
        assertNull(response.getEncryptedIdentityTokenElement());

        final IdentityToken identityToken = response.getIdentityToken();

        Assert.assertEquals(testModel.getPatientCpr().getValue(), identityToken.getCpr());
        assertNull(identityToken.getPrivileges());
        assertEquals(testModel.getClearTextAudience(), identityToken.getAudienceRestriction());
    }

    @Test
    @Ignore // Fails until JWT2OIOSAML output token has been aligned with BST2IDWS input tokens
	public void testCallingWithSboBootstrapToken() throws Exception {
        Assume.assumeTrue(suiteName.equals(OIOBSTSAMLAssertion.Type.OIO2BST_LEGACY.name())); // Sbo service only produces legacy tokens
		SboTokenServiceEmbeddedTest sboService = new SboTokenServiceEmbeddedTest();
		sboService.version = version;
		sboService.setupTestModel();
		sboService.setUp();
		
		final String certsubdn = "SERIALNUMBER=PID:9208-2002-2-514358910503 + CN=Lars Larsen, O=Ingen organisatorisk tilknytning, C=DK";
		final String cpr = "0501792275";
		
		OIOBootstrapToken bst = sboService.callValidRequestWithCprClaimIncludeBst(certsubdn, cpr, "ssi_foces_2024");
		bst.validateSignatureAndTrust(StsCredentialVaults.getStsCredentialVault(version));

		nboConfig.setTrustedVault(StsCredentialVaults.getStsCredentialVault(version));
		
		
    	OIOBootstrapTokenBuilder builder = new OIOBootstrapTokenBuilder(StsCredentialVaults.getStsCredentialVault(version), bst.getIssuer());
    	builder.setAudienceRestriction(bst.getAudienceRestriction());
    	builder.setCpr(bst.getCpr());
    	builder.setHolderOfKeyCertificate(bst.getSigningCertificate());
    	builder.setSubjectName(bst.getSubjectNameID());
    	OIOBootstrapToken bootstrapToken = builder.build();
    	
    	
		final String audience = testModel.getClearTextAudience();
        OIOBootstrapToIdentityTokenRequestDOMBuilder requestBuilder = createRequestDomBuilder(bootstrapToken, testModel.getValidWhitelistedVault(), audience, bst.getCpr());

        Document requestDocument = buildRequestDocument(requestBuilder);
        
        final Document responseDoc = getResponseDocument(requestDocument);
 
        OIOBootstrapToIdentityTokenResponse response = getResponseModel(responseDoc);
        assertFalse(response.getFaultString(), response.isFault());
        assertSuccess(response, federation);

        assertNotNull(response.getIdentityToken());
        assertNull(response.getEncryptedIdentityTokenElement());

        final IdentityToken identityToken = response.getIdentityToken();

        Assert.assertEquals(bst.getCpr(), identityToken.getCpr());
        assertNull(identityToken.getPrivileges());
        assertEquals(audience, identityToken.getAudienceRestriction());
	}
}