/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.facade.bootstrap;

import dk.sosi.seal.model.OIOBootstrapToIdentityTokenResponse;
import dk.sosi.seal.model.dombuilders.OIOBootstrapToIdentityTokenRequestDOMBuilder;
import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.server.Bootstrap2IdwsRequestHandler;
import dk.sosi.sts.server.facade.TestClient;
import dk.sosi.sts.server.facade.TestClientHelper;
import dk.sosi.sts.server.service.procuration.ProcurationWebservice;
import org.junit.Assume;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.w3c.dom.Document;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import static dk.sosi.seal.model.OnBehalfOfClaim.OnBehalfOfClaimType.Procuration;
import static dk.sosi.sts.fuldmagt.stub.ProcurationTestModel.CPR_1;
import static dk.sosi.sts.server.facade.IdentityTokenResponseValidation.assertSuccess;

public class BootstrapTokenProcurationRemoteIT extends BootstrapTokenProcurationTest {
    protected TestClientHelper embeddedTestClient;

    public BootstrapTokenProcurationRemoteIT() {
        super(false);
    }

    @Before
    public void assumeProcurationEnabled() {
        Assume.assumeFalse(Boolean.getBoolean("procurationtests.skip"));
    }

    @Before
    public void setUp() throws Exception {
        EmbeddedBootstrapTokenToIdentityTokenBuilder embeddedBuilder = new EmbeddedBootstrapTokenToIdentityTokenBuilder(testModel, version);
        embeddedBuilder.build();
        Bootstrap2IdwsRequestHandler impl = embeddedBuilder.getRequestHandler();

        ProcurationWebservice procurationWebservice = new ProcurationWebservice(new ServiceInfoCollector(0));
        procurationWebservice.setEntityId("https://saml.sts.nspop.dk");
        procurationWebservice.setEndpoint(new URL("https://delegationquery.fuldmagt.test-nemlog-in.dk/QueryWebService.svc"));
        procurationWebservice.init();

        impl.setProcurationService(procurationWebservice);
        embeddedTestClient = new TestClientHelper(impl);

        System.setProperty("javax.net.ssl.keyStore", "src/main/config/nsp-test-rid.jks");
        System.setProperty("javax.net.ssl.keyStorePassword", "Test1234");
    }

    @Override
    protected TestClient getTestClient() {
        return embeddedTestClient;
    }

    @Test
    @Ignore
    public void testNotTooSlow() throws ExecutionException, InterruptedException {
        OIOBootstrapToIdentityTokenRequestDOMBuilder requestBuilder = createRequestDomBuilder(assertionBuilder.build(), testModel.getValidWhitelistedVault(),
                testModel.getClearTextAudience(), testModel.getPatientCpr().getValue(), false, encryptionKey);
        requestBuilder.setOnBehalfOfClaim(CPR_1, Procuration);
        Document requestDocument = buildRequestDocument(requestBuilder);

        getResponseDocument(requestDocument);

        final ExecutorService executorService = Executors.newFixedThreadPool(10);

        Runnable runnable = () -> {
            final Document responseDoc = getResponseDocument(requestDocument);
            OIOBootstrapToIdentityTokenResponse response = getResponseModel(responseDoc);
            assertSuccess(response, federation);
        };

        List<Future> futures = new ArrayList<>();

        for (int i = 0; i < 100; i++) {
            final Future<?> future = executorService.submit(runnable);
            futures.add(future);
        }
        for (Future future : futures) {
            future.get();
        }
    }
}