/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.server.facade.bootstrap;

import static dk.sosi.sts.server.facade.IdentityTokenResponseValidation.assertFault;


import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.w3c.dom.Document;

import dk.sosi.seal.model.BootstrapTokenBuilder;
import dk.sosi.seal.model.OIOBootstrapToIdentityTokenResponse;
import dk.sosi.seal.model.OIOBootstrapToken;
import dk.sosi.sts.server.NboConfig;
import dk.sosi.sts.server.StsFaultActorValues;
import dk.sosi.sts.server.StsFaultCodeValues;
import dk.sosi.sts.server.facade.TestClient;
import dk.sosi.sts.server.facade.TestClientHelper;
import dk.sosi.sts.server.test.SealPPCredentialVaults;

/**
 * Created by Kent on 20-03-2017.
 */
public class BootstrapTokenToIdentityTokenLegacySpecificationEmbeddedTest extends BootstrapTokenToIdentityTokenLegacySpecificationTest {

	protected TestClientHelper testClientHelper;
	private NboConfig nboConfig;

	@Before
	public void setUp() throws Exception {
		EmbeddedBootstrapTokenToIdentityTokenBuilder builder = new EmbeddedBootstrapTokenToIdentityTokenBuilder(testModel, version);
		testClientHelper = builder.build();
		nboConfig = builder.getNboConfig();
	}

	/**
	 * This is Testcase #T6 in the specification document
	 */
	@Test
	public void testSigningCertificateRevoked() throws Exception {
		final BootstrapTokenBuilder assertionBuilder = new BootstrapTokenBuilder(testModel.getExternalTrustedIdp(), testModel.getPatientCertificateInfo());
		OIOBootstrapToken bootstrapToken = assertionBuilder.build();
		Document requestDocument = buildRequestDocument(bootstrapToken, SealPPCredentialVaults.getRevokedFocesVault(version), testModel.getClearTextAudience(), testModel.getPatientCpr().getValue());
        final Document responseDoc = getResponseDocument(requestDocument);
        OIOBootstrapToIdentityTokenResponse response = getResponseModel(responseDoc);

        
        
		assertFault(response, StsFaultActorValues.ITS, StsFaultCodeValues.WST_FAILED_AUTHENTICATION, "Certificate revoked");
	}

	/**
	 * This is Testcase #T14 in the specification document
	 */
	@Test
	public void testBootstrapTokenSignerExpired() throws Exception {
		nboConfig.setTrustedVault(SealPPCredentialVaults.getExpiredFocesVault(version));
		final BootstrapTokenBuilder assertionBuilder = new BootstrapTokenBuilder(SealPPCredentialVaults.getExpiredFocesVault(version), testModel.getPatientCertificateInfo());
		OIOBootstrapToken bootstrapToken = assertionBuilder.build();
		Document requestDocument = buildRequestDocument(bootstrapToken, testModel.getValidWhitelistedVault(), testModel.getClearTextAudience(), testModel.getPatientCpr().getValue());
        final Document responseDoc = getResponseDocument(requestDocument);
        OIOBootstrapToIdentityTokenResponse response = getResponseModel(responseDoc);

		assertFault(response, StsFaultActorValues.ITS, StsFaultCodeValues.WST_REQUEST_FAILED, "Bootstrap token signer expired");
	}

	/**
	 * This is Testcase #T15 in the specification document
	 */
	@Test
	public void testBootstrapTokenSignerRevoked() throws Exception {
		nboConfig.setTrustedVault(SealPPCredentialVaults.getRevokedFocesVault(version));
		final BootstrapTokenBuilder assertionBuilder = new BootstrapTokenBuilder(SealPPCredentialVaults.getRevokedFocesVault(version), testModel.getPatientCertificateInfo());
		OIOBootstrapToken bootstrapToken = assertionBuilder.build();
		Document requestDocument = buildRequestDocument(bootstrapToken, testModel.getValidWhitelistedVault(), testModel.getClearTextAudience(), testModel.getPatientCpr().getValue());
        final Document responseDoc = getResponseDocument(requestDocument);
        OIOBootstrapToIdentityTokenResponse response = getResponseModel(responseDoc);

		assertFault(response, StsFaultActorValues.ITS, StsFaultCodeValues.WST_REQUEST_FAILED, "Bootstrap token signer revoked");
	}

	@Override
	protected TestClient getTestClient() {
		return testClientHelper;
	}
}
