package dk.sosi.sts.server.facade.bootstrap;

import dk.sosi.seal.OIOSAMLFactory;
import dk.sosi.seal.model.*;
import dk.sosi.seal.pki.CertificateInfo;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.test.TestParameterBuilder;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.server.facade.DefaultSamlTokenTestModel;
import dk.sosi.sts.server.facade.SamlTokenTestModel;
import dk.sosi.sts.server.test.TestUtil;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import java.security.PublicKey;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Date;

@RunWith(Parameterized.class)
public abstract class BootstrapTokenToIdentityTokenParameterizedTest extends BootstrapTokenToIdentityTokenTest {
    private static final OIOSAMLFactory OIOSAML_FACTORY = new OIOSAMLFactory();
    protected Federation federation;

    @Parameterized.Parameter(0)
    public String suiteName;

    @Parameterized.Parameter(1)
    public OcesInfo.Version version;

    @Parameterized.Parameter(2)
    public Class<OIOBSTSAMLAssertion> assertionClass;

    protected SamlTokenTestModel testModel;
    protected PublicKey encryptionKey;
    protected AbstractOIOBSTSAMLAssertionBuilder<?, ?> assertionBuilder;

    /* Setting up test parameters */
    @Parameterized.Parameters(name = "{0}")
    public static Iterable<Object[]> data() {
        TestParameterBuilder builder = new TestParameterBuilder();
        builder.newTest(OIOBSTSAMLAssertion.Type.OIO2BST_CITIZEN.name()).add(OcesInfo.Version.OCES3, OIO2BSTCitizenSAMLAssertion.class);
        builder.newTest(OIOBSTSAMLAssertion.Type.OIO3BST_CITIZEN.name()).add(OcesInfo.Version.OCES3, OIO3BSTCitizenSAMLAssertion.class);
        // TODO SDS-6660
        // builder.newTest(OIOBSTSAMLAssertion.Type.OIO2BST_LEGACY.name()).add(OcesInfo.Version.OCES3, OIOBootstrapToken.class);
        return builder.build();
    }

    public BootstrapTokenToIdentityTokenParameterizedTest(boolean isEmbeddedTest) {
        federation = TestUtil.createTestFederation(isEmbeddedTest);
    }

    @Before
    public void setup() {
        this.testModel = new DefaultSamlTokenTestModel(version);
        encryptionKey = testModel.getValidWhitelistedVault().getSystemCredentialPair().getCertificate().getPublicKey();
        switch (OIOBSTSAMLAssertion.Type.valueOf(suiteName)) {
            case OIO2BST_CITIZEN :
                assertionBuilder = createOIO2BSTCitizenSAMLAssertionBuilder(testModel.getValidWhitelistedVault(), testModel.getExternalTrustedIdp(), testModel.getClearTextAudience());
                break;
            case OIO3BST_CITIZEN :
                assertionBuilder = createOIO3BSTCitizenSAMLAssertionBuilder(testModel.getValidWhitelistedVault(), testModel.getExternalTrustedIdp(), testModel.getClearTextAudience());
                break;
            case OIO2BST_LEGACY :
                assertionBuilder = createOIO2BSTLegacySAMLAssertionBuilder(testModel.getValidWhitelistedVault(), testModel.getExternalTrustedIdp(), testModel.getPatientCertificateInfo(), testModel.getClearTextAudience());
                break;
            default :
                throw new RuntimeException("Unrecognized token type");
        }
    }

    private static OIO2BSTCitizenSAMLAssertionBuilder createOIO2BSTCitizenSAMLAssertionBuilder(CredentialVault requestSigningVault, CredentialVault assertionSigningVault, String audience) {
        OIO2BSTCitizenSAMLAssertionBuilder builder = OIOSAML_FACTORY.createOIO2BSTCitizenSAMLAssertionBuilder();
        builder.setAssuranceLevel(AssuranceLevel.NIST.NIVEAU_3);
        builder.setCpr("0501792275");
        builder.setIssuer("https://oio2bst-issuer.dk");
        builder.setNameId("dk:gov:saml:attribute:CprNumberIdentifier:0501792275");
        builder.setHolderOfKeyCertificate(requestSigningVault.getSystemCredentialPair().getCertificate());
        builder.setAudience(audience);
        builder.setNotOnOrAfter(Date.from(Instant.now().plus(2, ChronoUnit.HOURS)));
        builder.setSigningVault(assertionSigningVault);
        return builder;
    }

    private static OIO3BSTCitizenSAMLAssertionBuilder createOIO3BSTCitizenSAMLAssertionBuilder(CredentialVault requestSigningVault, CredentialVault assertionSigningVault, String audience) {
        OIO3BSTCitizenSAMLAssertionBuilder builder = OIOSAML_FACTORY.createOIO3BSTCitizenSAMLAssertionBuilder();
        builder.setAssuranceLevel(AssuranceLevel.NSIS.Substantial);
        builder.setCpr("0501792275");
        builder.setIssuer("https://oio3bst-issuer.dk");
        builder.setNameId("dk:gov:saml:attribute:CprNumberIdentifier:0501792275");
        builder.setHolderOfKeyCertificate(requestSigningVault.getSystemCredentialPair().getCertificate());
        builder.setAudience(audience);
        builder.setNotOnOrAfter(Date.from(Instant.now().plus(2, ChronoUnit.HOURS)));
        builder.setSigningVault(assertionSigningVault);
        return builder;
    }

    private static BootstrapTokenBuilder createOIO2BSTLegacySAMLAssertionBuilder(CredentialVault requestSigningVault, CredentialVault assertionSigningVault, CertificateInfo certInfo, String audience) {
        BootstrapTokenBuilder builder = new BootstrapTokenBuilder(assertionSigningVault, certInfo);
        builder.setAssuranceLevel(AssuranceLevel.NIST.NIVEAU_3);
        builder.setCpr("0501792275");
        builder.setIssuer("https://oio2bst-legacy-issuer.dk");
        builder.setHolderOfKeyCertificate(requestSigningVault.getSystemCredentialPair().getCertificate());
        builder.setAudience(audience);
        builder.setNotOnOrAfter(Date.from(Instant.now().plus(2, ChronoUnit.HOURS)));
        builder.setSigningVault(assertionSigningVault);
        return builder;
    }
}
