/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.facade.bootstrap;


import dk.sosi.seal.OIOSAMLFactory;
import dk.sosi.seal.model.LibertySignatureValidator;
import dk.sosi.seal.model.OIOBSTSAMLAssertion;
import dk.sosi.seal.model.OIOBootstrapToIdentityTokenResponse;
import dk.sosi.seal.model.OnBehalfOfClaim;
import dk.sosi.seal.model.SignatureConfiguration.KeyInfoType;
import dk.sosi.seal.model.dombuilders.OIOBootstrapToIdentityTokenRequestDOMBuilder;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.EmptyCredentialVault;
import dk.sosi.seal.xml.XmlUtil;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.server.facade.TestClient;
import dk.sosi.sts.server.test.TestUtil;
import org.w3c.dom.Document;

import java.io.IOException;
import java.security.PublicKey;

import static dk.sosi.seal.model.OnBehalfOfClaim.OnBehalfOfClaimType.Procuration;

/**
 * User: ksr
 * Date: 22-10-13
 * Time: 14:58
 */
public abstract class BootstrapTokenToIdentityTokenTest {

	protected final Federation federation = TestUtil.createTestFederation(true);
	protected final Document buildRequestDocument(OIOBSTSAMLAssertion bootstrapToken, CredentialVault signingVault, String appliesTo, String cprClaim) throws IOException {
		return buildRequestDocument(bootstrapToken, signingVault, appliesTo, cprClaim, false);
	}
	
	protected final Document buildRequestDocument(OIOBSTSAMLAssertion bootstrapToken, CredentialVault signingVault, String appliesTo, String cprClaim, boolean useSecurityTokenReference) throws IOException {
		OIOBootstrapToIdentityTokenRequestDOMBuilder requestBuilder = createRequestDomBuilder(bootstrapToken, signingVault, appliesTo, cprClaim, useSecurityTokenReference, null);
		return buildRequestDocument(requestBuilder);
	}

	protected OIOBootstrapToIdentityTokenRequestDOMBuilder createRequestDomBuilder(OIOBSTSAMLAssertion bootstrapToken, CredentialVault signingVault, String appliesTo, String cprClaim) {
		return createRequestDomBuilder(bootstrapToken, signingVault, appliesTo, cprClaim, false, null);
	}

	protected OIOBootstrapToIdentityTokenRequestDOMBuilder createRequestDomBuilder(OIOBSTSAMLAssertion bootstrapToken, CredentialVault signingVault, String appliesTo, String cprClaim, CprNr onBehalfOfCpr, OnBehalfOfClaim.OnBehalfOfClaimType claimType, boolean useSecurityTokenReference, PublicKey encryptionKey) {
		OIOBootstrapToIdentityTokenRequestDOMBuilder builder = createRequestDomBuilder(bootstrapToken,
																				 signingVault,
																				 appliesTo,
																				 cprClaim,
																				 useSecurityTokenReference,
																				 encryptionKey);
		builder.setOnBehalfOfClaim(onBehalfOfCpr.getValue(), claimType);
		return builder;
	}

	protected OIOBootstrapToIdentityTokenRequestDOMBuilder createRequestDomBuilder(OIOBSTSAMLAssertion bootstrapToken, CredentialVault signingVault, String appliesTo, String cprClaim, boolean useSecurityTokenReference, PublicKey encryptionKey) {
		OIOBootstrapToIdentityTokenRequestDOMBuilder requestBuilder = new OIOSAMLFactory().createOIOBootstrapToIdentityTokenRequestDOMBuilder();
		requestBuilder.setCPRNumberClaim(cprClaim);
		requestBuilder.setOIOBootstrapToken(bootstrapToken);

		if(useSecurityTokenReference) {
			requestBuilder.setSigningVault(signingVault, KeyInfoType.WSS_REFERENCE);
		} else {
			requestBuilder.setSigningVault(signingVault);
		}

		requestBuilder.setEncryptionKey(encryptionKey);
		requestBuilder.setAudience(appliesTo);
		return requestBuilder;
	}

	protected Document buildRequestDocument(OIOBootstrapToIdentityTokenRequestDOMBuilder requestBuilder) {
		final Document requestDocument = requestBuilder.build();
		LibertySignatureValidator validator = new LibertySignatureValidator(new EmptyCredentialVault(), requestDocument.getDocumentElement());
		validator.validateSignature();
		return requestDocument;
	}

	public static Document getResponseDocument(TestClient client, Document requestDocument) {
		return client.execute(XmlUtil.node2String(requestDocument, false, false)).document;
	}

	protected final Document getResponseDocument(Document requestDocument) {
		return getTestClient().execute(XmlUtil.node2String(requestDocument, false, false)).document;
	}

	protected final OIOBootstrapToIdentityTokenResponse getResponseModel(Document responseDoc) {
		return new OIOSAMLFactory().createOIOBootstrapToIdentityTokenResponseModelBuilder().build(responseDoc);
	}

	protected abstract TestClient getTestClient();
}