package dk.sosi.sts.server.facade.bst2sosi;

import dk.sosi.sts.roles.DBTestUtil;
import dk.sosi.sts.server.facade.HttpTestClient;
import org.apache.commons.lang3.RandomStringUtils;
import org.junit.Assume;
import org.junit.Before;
import org.junit.Test;
import org.springframework.jdbc.core.JdbcTemplate;

import static org.junit.Assert.assertEquals;

public class DatabaseIT {

    @Before
    public void setUp() {
        Assume.assumeTrue(HttpTestClient.testEnvironmentHost.contains("localhost"));
    }

    @Test
    public void testTrustedIdpConfiguration() {
        testInsertionOfAttributeValueShouldNotBeTruncated("trustedIdpConfiguration", "issuer");
    }

    @Test
    public void testTrustedIdpCitizenConfiguration() {
        testInsertionOfAttributeValueShouldNotBeTruncated("trustedIdpCitizenConfiguration", "issuer");
    }

    @Test
    public void testAudienceConfiguration() {
        testInsertionOfAttributeValueShouldNotBeTruncated("audienceConfiguration", "audience");
    }


    static void testInsertionOfAttributeValueShouldNotBeTruncated(String table, String dummyField) {
        final String value = RandomStringUtils.randomAlphanumeric(128);
        JdbcTemplate stsAudconf = new JdbcTemplate(DBTestUtil.getTestDataSource("sts_audconf"));
        stsAudconf.execute("insert into " + table +  " (attribute, " + dummyField + ") " +
                "values ('" + value + "', 'dummy')");

        String readValue = stsAudconf
                .queryForObject("select attribute from " + table + " where attribute='" + value + "'",
                        String.class);

        assertEquals(value, readValue);
    }
}
