/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.server.facade.sbo;

import java.io.File;
import java.util.Collections;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.sts.certificate.OcesInfo.Version;
import dk.sosi.sts.server.SboRequestHandler;
import dk.sosi.sts.server.StsStatus;
import dk.sosi.sts.server.facade.DefaultSamlTokenTestModel;
import dk.sosi.sts.server.facade.MockServicesImpl;
import dk.sosi.sts.server.facade.SamlTokenEmbeddedTestHelper;
import dk.sosi.sts.server.facade.TestClient;
import dk.sosi.sts.server.facade.TestClientHelper;
import dk.sosi.sts.server.service.jwt.OcesKeyCloakJwtStrategy;
import dk.sosi.sts.server.test.StsCredentialVaults;
import dk.sosi.sts.service.ItsConfigService;

public class EmbeddedSboServiceBuilder {

    private File keyStoreFile = StsCredentialVaults.getJwtTrustStoreFile();
    
    private final Version version;
    
    public EmbeddedSboServiceBuilder(Version version) {
    	this.version = version;
    }
    
    public TestClient build() throws Exception {
        final MockServicesImpl mockServices = new MockServicesImpl(version);
        SOSIFactory serverSosiFactory = mockServices.buildSosiFactory(true);

        //JWT2IdwsRequestHandler impl = new JWT2IdwsRequestHandler();
        SboRequestHandler impl = new SboRequestHandler();

        DefaultSamlTokenTestModel testModel = new DefaultSamlTokenTestModel(version);
		ItsConfigService itsConfig = SamlTokenEmbeddedTestHelper.buildItsConfigService(testModel);
       
        impl.setItsConfigService(itsConfig);

        impl.setCrlService(mockServices);
        impl.setSosiFactory(serverSosiFactory);
        impl.setStsStatus(new StsStatus());
        impl.setOcesPidService(SamlTokenEmbeddedTestHelper.initMockOcesPidService());
        //impl.setProcurationService(null);

        impl.setFuzzyTime(300000);
        impl.setAllowedDriftInSeconds(0L);
        
        impl.setTrustStorePassword("Test1234");
        impl.setTrustStoreFile(keyStoreFile);

        final OcesKeyCloakJwtStrategy keyCloakJwtStrategy = new OcesKeyCloakJwtStrategy();
        keyCloakJwtStrategy.setRequiredStsScope("sosi-sts");
        impl.setIssuerStrategies(Collections.singletonMap("http://sts-tester", keyCloakJwtStrategy));
        impl.afterPropertiesSet();

        return new TestClientHelper(impl);
    }
}
