/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.hsqldb.jdbc.JDBCDataSource;

import junit.framework.TestCase;

public class StsAuthorizationServiceTest extends TestCase {

	private static final String PREFIX = StsAuthorizationService.class.getName();
	private Properties props = new Properties();
	private Map<String, Object> resources;
	private JDBCDataSource dataSource;

	@Override
	protected void setUp() throws Exception {
		props.put(PREFIX + ".datasource", "sts.datasource");
		dataSource = new JDBCDataSource();
		dataSource.setDatabase("jdbc:hsqldb:mem:memoryDB");
		resources = new HashMap<String, Object>();
		resources.put("sts.datasource", dataSource);
	}

	public void testMissingDatasource() {
		resources.remove("sts.datasource");

		try {
			new StsAuthorizationService(null);
			fail("Should fail when configured datasource is missing");
		} catch (RuntimeException e)  {
			/* expected */
			assertTrue(e.getMessage().contains("datasource"));
		}
	}

	public void testDatasourceIllegalSelect() {

		try {
			new StsAuthorizationService(dataSource, "");
			fail("Empty select should fail");
		} catch (IllegalArgumentException e) { /* expected */ }
	}

	public void testDatasource() {
		new StsAuthorizationService(dataSource);
	}

}
