/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service;

import java.security.cert.X509Certificate;
import java.util.Map;

import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.SubjectSerialNumber;
import junit.framework.TestCase;

public class StsMockOcesCvrRidServiceTest extends TestCase {

	public void testCreateMapFromNull() {
		Map<SubjectSerialNumber, CprNr> map = StsMockOcesCvrRidService.createMap(null);
		assertEquals(0, map.size());
	}

	public void testCreateMapFromEmpty() {
		Map<SubjectSerialNumber, CprNr> map = StsMockOcesCvrRidService.createMap("");
		assertEquals(0, map.size());
	}

	public void testCreateMapWithInvalid() {
		Map<SubjectSerialNumber, CprNr> map;
		map = StsMockOcesCvrRidService.createMap("CvrRid1 1234567890");
		assertEquals(0, map.size());
		
		map = StsMockOcesCvrRidService.createMap("CvrRid1 1234567890 [XXX] [CvrRid3 1234567892]");
		assertEquals(1, map.size());
	}

	public void testCreateMap() {
		Map<SubjectSerialNumber, CprNr> map;
		map = StsMockOcesCvrRidService.createMap("[CvrRid1 1234567890]");
		assertEquals(1, map.size());

		map = StsMockOcesCvrRidService.createMap("[CvrRid1 1234567890] [CvrRid2 1234567891] [CvrRid3 1234567892]");
		assertEquals(3, map.size());
		
		map = StsMockOcesCvrRidService.createMap("[CvrRid1 1234567890] [CvrRid2 1234567891] [CvrRid2 1234567892]");
		assertEquals(2, map.size());
	}
	
	public void testCreateService() throws StsException {
		StsMockOcesCvrRidService service = new StsMockOcesCvrRidService("[CvrRid1 1234567890] [CvrRid2 1234567891]");

    assertEquals("1234567890", service.findRelatedCpr(WrappedOcesInfo.wrap("CvrRid1")).getValue());
		assertTrue(service.isRelated(WrappedOcesInfo.wrap("CvrRid2"), new CprNr("1234567891")));

		assertNull(service.findRelatedCpr(WrappedOcesInfo.wrap("CvrRid3")));
		assertFalse(service.isRelated(WrappedOcesInfo.wrap("CvrRid1"), new CprNr("1234567891")));
	}

  private static class WrappedOcesInfo extends OcesInfo {

    public static OcesInfo wrap(String ssn) {
      return new WrappedOcesInfo(new SubjectSerialNumber(ssn), "loko", CertificateType.UNKNOWN, null, null);
    }
    
    public WrappedOcesInfo(SubjectSerialNumber ssn, String cvr, CertificateType type, Version version, final X509Certificate certificate) {
      super(ssn, cvr, type, version, certificate);
    }

    @Override
    public String getCvr() {
      throw new UnsupportedOperationException("getCvr not support on WrappedOcesInfo");
    }

    @Override
    public SubjectSerialNumber getSubjectSerialNumber() {
      return super.getSubjectSerialNumber();
    }

    @Override
    public X509Certificate getCertificate() {
      throw new UnsupportedOperationException("getCertificate not support on WrappedOcesInfo");
    }
  }
}
