/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service.procuration;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.PrivilegeGroup;
import dk.sosi.sts.service.ProcurationService;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class AggregatedProcurationServiceTest {

    @Test
    public void getProcurationPrivileges_returnsPrivilegesFromBothServices() throws StsException {
        // Arrange
        CprNr cpr1 = new CprNr("0101019999");
        CprNr cpr2 = new CprNr("0202028888");

        PrivilegeGroup pg1 = new PrivilegeGroup("scope1");
        pg1.addPrivilege("priv1");
        pg1.addPrivilege("priv2");
        ProcurationService procurationService1 = mock(ProcurationService.class);
        when(procurationService1.getProcurationPrivileges(cpr1, cpr2)).thenReturn(pg1);

        PrivilegeGroup pg2 = new PrivilegeGroup("scope2");
        pg2.addPrivilege("priv3");
        pg2.addPrivilege("priv4");
        ProcurationService procurationService2 = mock(ProcurationService.class);
        when(procurationService2.getProcurationPrivileges(cpr1, cpr2)).thenReturn(pg2);

        AggregatedProcurationService aggregatedProcurationService = new AggregatedProcurationService(
                mock(ServiceInfoCollector.class), procurationService1, procurationService2);

        // Act
        PrivilegeGroup result = aggregatedProcurationService.getProcurationPrivileges(cpr1, cpr2);

        // Assert
        assertNotNull(result);
        assertEquals("urn:dk:healthcare:saml:actThroughProcurationBy:cprNumberIdentifier:0101019999", result.getScope());
        List<String> privileges = result.getPrivileges();
        assertEquals(4, privileges.size());
        assertTrue(privileges.contains("priv1"));
        assertTrue(privileges.contains("priv2"));
        assertTrue(privileges.contains("priv3"));
        assertTrue(privileges.contains("priv4"));
    }


}