/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.service.procuration;

import dk.sosi.sts.ServiceInfoCollector;
import dk.sosi.sts.StsException;
import dk.sosi.sts.fuldmagt.stub.ProcurationTestModel;
import dk.sosi.sts.model.CprNr;
import dk.sosi.sts.model.PrivilegeGroup;
import org.junit.Assume;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import static org.junit.Assert.*;

public class ProcurationWebServiceIT {
    private ProcurationWebservice service;

    @BeforeClass
    public static void initSSLContext() {
        System.setProperty("javax.net.ssl.keyStore", "src/main/config/nsp-test-rid.jks");
        System.setProperty("javax.net.ssl.keyStorePassword", "Test1234");
        Assume.assumeFalse(Boolean.getBoolean("procurationtests.skip"));
    }


    @Before
    public void initService() throws MalformedURLException {
        ProcurationWebservice service = new ProcurationWebservice(new ServiceInfoCollector(0));

        service.setEntityId("https://saml.sts.nspop.dk");
        //service.setEndpoint(new URL("http://localhost:8081/fuldmagt-stub/QueryWebService"));
        service.setEndpoint(new URL("https://delegationquery.fuldmagt.test-nemlog-in.dk/QueryWebService.svc"));
        service.init();
        this.service = service;
    }

    @Test
    public void testReadOnly() throws StsException {
        PrivilegeGroup procurationPrivileges = service.getProcurationPrivileges(new CprNr(ProcurationTestModel.CPR_1), new CprNr(ProcurationTestModel.CPR_3));
        List<String> privileges = procurationPrivileges.getPrivileges();
        assertFalse(privileges.isEmpty());
        assertTrue(procurationPrivileges.getConstraints().isEmpty());
        assertTrue(privileges.contains(ProcurationTestModel.FMK_READ));
        assertFalse(privileges.contains(ProcurationTestModel.FMK_WRITE));
    }

    @Test
    public void testNoProcurations() throws StsException {
        PrivilegeGroup procurationPrivileges = service.getProcurationPrivileges(new CprNr(ProcurationTestModel.CPR_3), new CprNr(ProcurationTestModel.CPR_1));
        assertEquals(0, procurationPrivileges.getPrivileges().size());
        assertEquals(0, procurationPrivileges.getConstraints().size());
    }

    @Test
    public void testWrite() throws StsException {
        PrivilegeGroup procurationPrivileges = service.getProcurationPrivileges(new CprNr(ProcurationTestModel.CPR_2), new CprNr(ProcurationTestModel.CPR_3));
        List<String> privileges = procurationPrivileges.getPrivileges();
        assertFalse(privileges.isEmpty());
        assertTrue(procurationPrivileges.getConstraints().isEmpty());
        assertTrue(privileges.contains(ProcurationTestModel.FMK_WRITE));
        assertFalse(privileges.contains(ProcurationTestModel.FMK_READ));
    }
}