/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.test;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.mockito.Mockito;

import dk.sosi.sts.StsException;
import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.ocescpr.cvrrid.service.impl.WsOcesCvrRidService;
import junit.framework.TestCase;

/**
 *
 */
public class CheckerTest extends TestCase {

	private final List<String> writer = new LinkedList<String>();

	private final Checker checker = new Checker() {
		@Override
		protected void output(final String s) {
			writer.add(s);
		}
	};

	@Override
	protected void setUp() throws Exception {
		writer.clear();
	}

	public void testCheckCvrRidAccess() throws Exception {
		final Map<String, WsOcesCvrRidService> mappings = new HashMap<String, WsOcesCvrRidService>();

		final WsOcesCvrRidService service = Mockito.mock(WsOcesCvrRidService.class);
		mappings.put("TESTING", service);

		checker.checkCvrRidAccess(mappings);

		assertEquals("    Result: null", writer.get(writer.size() - 1));
	}

	public void testCheckCvrRidAccess_Failed() throws Exception {
		final Map<String, WsOcesCvrRidService> mappings = new HashMap<String, WsOcesCvrRidService>();

		final WsOcesCvrRidService service = Mockito.mock(WsOcesCvrRidService.class);
		Mockito.when(service.findRelatedCpr(Mockito.<OcesInfo>any())).thenThrow(new StsException("problem"));
		mappings.put("TESTING", service);

		checker.checkCvrRidAccess(mappings);
		assertEquals("    Result: failed with problem", writer.get(writer.size() - 1));

	}
}
