package dk.sosi.sts.server.test;

import java.util.HashMap;
import java.util.Map;

import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.certificate.OcesInfo.Version;
import dk.sosi.sts.server.test.SealPPCredentialVaults.SealPPCredentialVaultProvider.Key;

/**
 * User: ksr
 * Date: 21-10-14
 * Time: 22:53
 */
public class SealPPCredentialVaults {
    interface SealPPCredentialVaultProvider {
    	enum Key{
    		VALID_MOCES_NO_CPR,
    		VALID_MOCES_WITH_CPR,
            VALID_MOCES_GLOBAL_UUID,
            VALID_MOCES_CERTIFICATE_SPECIFIC_UUID,
    		EXPIRED_MOCES_VAULT,
    		REVOKED_MOCES_VAULT,
    		VALID_VOCES_VAULT,
    		EXPIRED_VOCES_VAULT,
    		REVOKED_VOCES_VAULT,
    		VALID_FOCES_VAULT,
    		EXPIRED_FOCES_VAULT,
    		REVOKED_FOCES_VAULT
    	};
    	GenericCredentialVault getCredentialVault(Key key);
    }
    
	private static final Map<Version, SealPPCredentialVaultProvider> SEAL_PP_CREDENTIAL_VAULT_PROVIDER = new HashMap<Version, SealPPCredentialVaultProvider>();

	static {
		SEAL_PP_CREDENTIAL_VAULT_PROVIDER.put(Version.OCES3, new SealPPOces3CredentialVaultProvider());
		SEAL_PP_CREDENTIAL_VAULT_PROVIDER.put(Version.OCES2, new SealPPOces2CredentialVaultProvider());
	}

	private static SealPPCredentialVaultProvider getProvider(Version version) {
		return SEAL_PP_CREDENTIAL_VAULT_PROVIDER.get(version);
	}

	private static GenericCredentialVault getSealPPCredentialVaultFromProvider(SealPPCredentialVaultProvider provider, Key key) {
		return provider.getCredentialVault(key);
	}

	private static GenericCredentialVault getSealPPCredentialVaultByVersion(Version version, Key key) {
		SealPPCredentialVaultProvider provider = getProvider(version);
		return getSealPPCredentialVaultFromProvider(provider, key);
	}


	public static GenericCredentialVault getValidMocesNoCpr(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.VALID_MOCES_NO_CPR);
    }

    public static GenericCredentialVault getValidMocesWithCpr(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.VALID_MOCES_WITH_CPR);
    }

    public static GenericCredentialVault getValidMocesGlobalUuid() {
        SealPPCredentialVaultProvider provider = getProvider(Version.OCES3);
        return getSealPPCredentialVaultFromProvider(provider, Key.VALID_MOCES_GLOBAL_UUID);
    }

    public static GenericCredentialVault getValidMocesCertificateSpecificUuid() {
        SealPPCredentialVaultProvider provider = getProvider(Version.OCES3);
        return getSealPPCredentialVaultFromProvider(provider, Key.VALID_MOCES_CERTIFICATE_SPECIFIC_UUID);
    }

    public static  GenericCredentialVault getExpiredMocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.EXPIRED_MOCES_VAULT);
    }

    public static  GenericCredentialVault getRevokedMocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.REVOKED_MOCES_VAULT);
    }

    public static GenericCredentialVault getValidVocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.VALID_VOCES_VAULT);
    }

    public static  GenericCredentialVault getExpiredVocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.EXPIRED_VOCES_VAULT);
    }

    public static  GenericCredentialVault getRevokedVocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.REVOKED_VOCES_VAULT);
    }

    public static GenericCredentialVault getValidFocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.VALID_FOCES_VAULT);
    }

    public static  GenericCredentialVault getExpiredFocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.EXPIRED_FOCES_VAULT);
    }

    public static  GenericCredentialVault getRevokedFocesVault(Version version) {
		return getSealPPCredentialVaultByVersion(version, Key.REVOKED_FOCES_VAULT);
    }
}
