package dk.sosi.sts.server.test;

import org.junit.internal.AssumptionViolatedException;

import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.server.test.SealPPCredentialVaults.SealPPCredentialVaultProvider;

public class SealPPOces3CredentialVaultProvider implements SealPPCredentialVaultProvider {
	private static final String password = "Test1234";

	@Override
	public GenericCredentialVault getCredentialVault(Key key) {
		switch (key) {
			case VALID_MOCES_NO_CPR :
				return TestUtil.getVaultFromResource("oces3/MOCES_gyldig.p12", password);
			case VALID_MOCES_WITH_CPR :
				// Thormund Nissen, cpr 0306894781, cvr 91026150, 1 authorization code, 0 national roles. Cvr trusted in test environments.
				return TestUtil.getVaultFromResource("oces3/MOCES_cpr_gyldig.p12", password);
            case VALID_MOCES_GLOBAL_UUID:
                return TestUtil.getVaultFromResource("oces3/MOCES_STSTEST_Ole_Olsen_global_uuid.p12", password);
            case VALID_MOCES_CERTIFICATE_SPECIFIC_UUID:
                return TestUtil.getVaultFromResource("oces3/MOCES_STSTEST_Ole_Olsen_specific_uuid.p12", password);
			case EXPIRED_MOCES_VAULT :
				throw new AssumptionViolatedException(key + " er ikke implementeret i " + getClass().getName());
			case REVOKED_MOCES_VAULT :
				// Samme fil som er i Seal.Java 2.6.44 og frem, så når vi skifter til den kan det ændres tilbage til "oces3/MOCES_spaerret.p12"
				return TestUtil.getVaultFromResource("oces3/STS_MOCES_spaerret.p12", password); 
			case VALID_VOCES_VAULT :
				return TestUtil.getVaultFromResource("oces3/VOCES_gyldig.p12", password);
			case EXPIRED_VOCES_VAULT :
				throw new AssumptionViolatedException(key + " er ikke implementeret i " + getClass().getName());
			case REVOKED_VOCES_VAULT :
				return TestUtil.getVaultFromResource("oces3/VOCES_spaerret.p12", password);
			case VALID_FOCES_VAULT :
				return TestUtil.getVaultFromResource("oces3/FOCES_gyldig.p12", password);
			case EXPIRED_FOCES_VAULT :
				throw new AssumptionViolatedException(key + " er ikke implementeret i " + getClass().getName());
			case REVOKED_FOCES_VAULT :
				return TestUtil.getVaultFromResource("oces3/FOCES_spaerret.p12", password);
			default :
				throw new RuntimeException(key + " er ikke implementeret i " + getClass().getName());
		}
	}
}
