/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.sts.server.test;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.certificate.OcesInfo.Version;
import dk.sosi.sts.server.test.StsCredentialVaults.StsCredentialVaultProvider.Key;

/**
 * User: ksr Date: 22-10-14 Time: 12:23
 */
public class StsCredentialVaults {
	 interface StsCredentialVaultProvider {
		enum Key {
			STS_CREDENTIAL_VAULT,
			EXTERNAL_TRUSTED_INVALID_IDP,
			EXTERNAL_TRUSTED_IDP,
			EXTERNAL_TRUSTED_IDP_CPR_NOT_TRUSTED,
			RANDOM_MOCES_WITH_CPR_TRUSTED_NATIONAL_ROLE,
			ALL_TRUSTED_CERTIFICATES,
			JWT_TRUST_STORE_CREDENTIAL_VAULT,
			UNAUTHORIZED_WITH_NATIONAL_ROLE,
			UNAUTHORIZED_WITH_NO_NATIONAL_ROLE,
			MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES,
			MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES_2_SAME_EDU_CODES,
			MITID_AUTHORIZATIONS_3_WITH_NATIONAL_ROLE,
			MITID_AUTHORIZATIONS_3_WITH_NO_NATIONAL_ROLE_TRUSTED_CVR,
			MITID_AUTHORIZED_WITH_NO_NATIONAL_ROLE,
			MITID_UNAUTHORIZED_WITH_3_NATIONAL_ROLES,
			NON_GLOBAL_MOCES;
		}

		GenericCredentialVault getCredentialVault(Key key);
	}

	private static final Map<Version, StsCredentialVaultProvider> STS_CREDENTIAL_VAILT_PROVIDERS = new HashMap<Version, StsCredentialVaults.StsCredentialVaultProvider>();

	static {
		STS_CREDENTIAL_VAILT_PROVIDERS.put(Version.OCES3, new StsOces3CredentialVaultProvider());
		STS_CREDENTIAL_VAILT_PROVIDERS.put(Version.OCES2, new StsOces2CredentialVaultProvider());
	}

	private static StsCredentialVaultProvider getProvider(Version version) {
		return STS_CREDENTIAL_VAILT_PROVIDERS.get(version);
	}

	private static GenericCredentialVault getStsCredentialVaultFromProvider(StsCredentialVaultProvider provider, Key key) {
		return provider.getCredentialVault(key);
	}

	private static GenericCredentialVault getStsCredentialVaultByVersion(Version version, Key key) {
		StsCredentialVaultProvider provider = getProvider(version);
		if(provider == null) {
			throw new RuntimeException("Ingen provider fundet til version:"+version+" og key:"+key);
		}
		return getStsCredentialVaultFromProvider(provider, key);
	}
	
	public static File getJwtTrustStoreFile() {
		return new File("src/main/config/test-jwt-idp-trust.jks");
	}

//	public static GenericCredentialVault getStsCredentialVault() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.STS_CREDENTIAL_VAULT);
//	}
//
//	public static CredentialVault getExternalTrustedInvalidIdp() throws IOException {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.EXTERNAL_TRUSTED_INVALID_IDP);
//	}
//
//	public static CredentialVault getExternalTrustedIdp() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.EXTERNAL_TRUSTED_IDP);
//	}
//
//	public static CredentialVault getExternalTrustedIdpCprNotTrusted() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.EXTERNAL_TRUSTED_IDP_CPR_NOT_TRUSTED);
//	}
//
//	public static CredentialVault getRandomMocesWithCvrTrustedNationalRole() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.RANDOM_MOCES_WITH_CPR_TRUSTED_NATIONAL_ROLE);
//	}
//
//	public static CredentialVault getAllTrustedCertificates() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.ALL_TRUSTED_CERTIFICATES);
//	}
//
//	public static GenericCredentialVault getJwtTrustStoreCredentialVault() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.JWT_TRUST_STORE_CREDENTIAL_VAULT);
//	}
//
//	public static GenericCredentialVault getUnauthorizedWithNationalRole() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.UNAUTHORIZED_WITH_NATIONAL_ROLE);
//	}
//
//	public static GenericCredentialVault getUnauthorizedWithNoNationalRole() {
//		return getStsCredentialVaultByVersion(Version.OCES2, Key.UNAUTHORIZED_WITH_NO_NATIONAL_ROLE);
//	}
	
	public static GenericCredentialVault getStsCredentialVault(Version version) {
		return getStsCredentialVaultByVersion(version, Key.STS_CREDENTIAL_VAULT);
	}

	public static CredentialVault getExternalTrustedInvalidIdp(Version version) throws IOException {
		return getStsCredentialVaultByVersion(version, Key.EXTERNAL_TRUSTED_INVALID_IDP);
	}

	public static CredentialVault getExternalTrustedIdp(Version version) {
		return getStsCredentialVaultByVersion(version, Key.EXTERNAL_TRUSTED_IDP);
	}

	public static CredentialVault getExternalTrustedIdpCprNotTrusted(Version version) {
		return getStsCredentialVaultByVersion(version, Key.EXTERNAL_TRUSTED_IDP_CPR_NOT_TRUSTED);
	}

	public static CredentialVault getRandomMocesWithCvrTrustedNationalRole(Version version) {
		return getStsCredentialVaultByVersion(version, Key.RANDOM_MOCES_WITH_CPR_TRUSTED_NATIONAL_ROLE);
	}

	public static CredentialVault getAllTrustedCertificates(Version version) {
		return getStsCredentialVaultByVersion(version, Key.ALL_TRUSTED_CERTIFICATES);
	}

	public static GenericCredentialVault getJwtTrustStoreCredentialVault(Version version) {
		return getStsCredentialVaultByVersion(version, Key.JWT_TRUST_STORE_CREDENTIAL_VAULT);
	}

	public static GenericCredentialVault getUnauthorizedWithNationalRole(Version version) {
		return getStsCredentialVaultByVersion(version, Key.UNAUTHORIZED_WITH_NATIONAL_ROLE);
	}

	public static GenericCredentialVault getUnauthorizedWithNoNationalRole(Version version) {
		return getStsCredentialVaultByVersion(version, Key.UNAUTHORIZED_WITH_NO_NATIONAL_ROLE);
	}

	public static GenericCredentialVault getMitIDAuthorizations3With3NationalRoles(Version version) {
		return getStsCredentialVaultByVersion(version, Key.MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES);
	}

	public static GenericCredentialVault getMitIDAuthorizations3With3NationalRoles2SameEduCodes(Version version) {
		return getStsCredentialVaultByVersion(version, Key.MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES_2_SAME_EDU_CODES);
	}

	public static GenericCredentialVault getMitIDAuthorizations3WithNationalRole(Version version) {
		return getStsCredentialVaultByVersion(version, Key.MITID_AUTHORIZATIONS_3_WITH_NATIONAL_ROLE);
	}

	public static GenericCredentialVault getMitIDAuthorizations3WithNoNationalRoleTrustedCvr(Version version) {
		return getStsCredentialVaultByVersion(version, Key.MITID_AUTHORIZATIONS_3_WITH_NO_NATIONAL_ROLE_TRUSTED_CVR);
	}

	public static GenericCredentialVault getMitIDAuthorizedWithNoNationalRole(Version version) {
		return getStsCredentialVaultByVersion(version, Key.MITID_AUTHORIZED_WITH_NO_NATIONAL_ROLE);
	}

	public static GenericCredentialVault getMitIDUnauthorizedWith3NationalRoles(Version version) {
		return getStsCredentialVaultByVersion(version, Key.MITID_UNAUTHORIZED_WITH_3_NATIONAL_ROLES);
	}

	public static GenericCredentialVault getNonGlobalMoces(Version version) {
		return getStsCredentialVaultByVersion(version, Key.NON_GLOBAL_MOCES);
	}
}
