package dk.sosi.sts.server.test;

import java.io.File;
import java.util.Properties;

import dk.sosi.seal.vault.FileBasedCredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.server.test.StsCredentialVaults.StsCredentialVaultProvider;

public class StsOces2CredentialVaultProvider implements StsCredentialVaultProvider {

	private static final String password = "Test1234";

	@Override
	public GenericCredentialVault getCredentialVault(Key key) {
		switch (key) {
			case STS_CREDENTIAL_VAULT :
				return TestUtil.getVaultFromResource("keystores/NSP_Test_Service_Consumer_sds.p12", "Test1234");
			case EXTERNAL_TRUSTED_INVALID_IDP :
				return TestUtil.getVaultFromResource("keystores/test-trusted-IdP.jks", "Test1234");
			case EXTERNAL_TRUSTED_IDP :
				// The same as Statens_Serum_Institut_FOCES.jks - expires september 2018
				return TestUtil.getVaultFromResource("keystores/unitteststs-nbo.keystore", "Test1234");
			case EXTERNAL_TRUSTED_IDP_CPR_NOT_TRUSTED :
				// The same as Statens_Serum_Institut_VOCES.jks - expires september 2018
				return TestUtil.getVaultFromResource("keystores/Statens_Serum_Institut_VOCES.jks", "Test1234");
			case RANDOM_MOCES_WITH_CPR_TRUSTED_NATIONAL_ROLE :
				return TestUtil.getVaultFromResource("keystores/Sonja_Bach_Laege.jks", "Test1234");
			case ALL_TRUSTED_CERTIFICATES :
				// Trust NemLog-in test and the 2 "idp's" above
				final File allTrustedCertificatesFile = new File("src/main/config/test-new-nemLogin-idp.keystore");
				if (!allTrustedCertificatesFile.exists()) {
					throw new IllegalArgumentException(allTrustedCertificatesFile.getAbsolutePath() + " not found");
				}
				return new FileBasedCredentialVault(new Properties(), allTrustedCertificatesFile, "Test1234");
			case JWT_TRUST_STORE_CREDENTIAL_VAULT :
				final File jwtTrustStoreCredentialVaultFile = StsCredentialVaults.getJwtTrustStoreFile();
				if (!jwtTrustStoreCredentialVaultFile.exists()) {
					throw new IllegalArgumentException(jwtTrustStoreCredentialVaultFile.getAbsolutePath() + " not found");
				}
				return new FileBasedCredentialVault(new Properties(), jwtTrustStoreCredentialVaultFile, "Test1234");
			case UNAUTHORIZED_WITH_NATIONAL_ROLE :
				return TestUtil.getVaultFromResource("keystores/LAKESIDE  - (ikke sundhedsfaglig)Grethe Pedersen.p12", "Test1234");
			case UNAUTHORIZED_WITH_NO_NATIONAL_ROLE :
				return TestUtil.getVaultFromResource("keystores/LAKESIDE TEST - Trine Pedersen.p12", "Test1234");
			case MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES:
				return TestUtil.getVaultFromResource("keystores/LAKESIDE AS - NSTSStreAtre Hansen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES_2_SAME_EDU_CODES:
				return TestUtil.getVaultFromResource("keystores/LAKESIDE AS - NSTSStreArteTOENSUDD Nielsen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_NATIONAL_ROLE:
				return TestUtil.getVaultFromResource("keystores/LAKESIDE AS - NSTSSenAtre Jensen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_NO_NATIONAL_ROLE_TRUSTED_CVR:
				return TestUtil.getVaultFromResource("keystores/moces-test.p12", password);
			case MITID_AUTHORIZED_WITH_NO_NATIONAL_ROLE:
				return TestUtil.getVaultFromResource("keystores/LAKESIDE AS - NSTSSnullAen Jensen.p12", password);
			case MITID_UNAUTHORIZED_WITH_3_NATIONAL_ROLES:
				return TestUtil.getVaultFromResource("keystores/LAKESIDE AS - NSTSStreAnull Mortensen.p12", password);
			default :
				throw new RuntimeException(key + " er ikke implementeret i " + getClass().getName());
		}
	}
}
