package dk.sosi.sts.server.test;

import java.io.File;
import java.util.Properties;

import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.vault.CredentialVaultTestUtil;
import dk.sosi.seal.vault.FileBasedCredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.server.test.StsCredentialVaults.StsCredentialVaultProvider;

public class StsOces3CredentialVaultProvider implements StsCredentialVaultProvider {

	private static final String password = "Test1234";

	@Override
	public GenericCredentialVault getCredentialVault(Key key) {
		switch (key) {
			case STS_CREDENTIAL_VAULT :
				return TestUtil.getVaultFromResource("keystores/NSP_Test_Service_Consumer_sds.p12", "Test1234");
			case EXTERNAL_TRUSTED_INVALID_IDP :
		    	return CredentialVaultTestUtil.getVoces3CredentialVault();
			case EXTERNAL_TRUSTED_IDP :
				return CredentialVaultTestUtil.getCredentialVaultFromResource(SignatureUtil.setupCryptoProviderForJVM(), "oces3/FOCES_gyldig.p12");
			case EXTERNAL_TRUSTED_IDP_CPR_NOT_TRUSTED :
				return CredentialVaultTestUtil.getCredentialVaultFromResource(SignatureUtil.setupCryptoProviderForJVM(), "oces3/VOCES_gyldig.p12");
			case RANDOM_MOCES_WITH_CPR_TRUSTED_NATIONAL_ROLE :
		    	return CredentialVaultTestUtil.getMoces3CredentialVault();
			case ALL_TRUSTED_CERTIFICATES :
				// Trust NemLog-in test and the 2 "idp's" above
				final File allTrustedCertificatesFile = new File("src/main/config/test-new-nemLogin-idp.keystore");
				if (!allTrustedCertificatesFile.exists()) {
					throw new IllegalArgumentException(allTrustedCertificatesFile.getAbsolutePath() + " not found");
				}
				return new FileBasedCredentialVault(new Properties(), allTrustedCertificatesFile, "Test1234");
			case JWT_TRUST_STORE_CREDENTIAL_VAULT :
				throw new RuntimeException(key + " er ikke implementeret i " + getClass().getName());
			case UNAUTHORIZED_WITH_NATIONAL_ROLE :
				return TestUtil.getVaultFromResource("oces3/MOCES_STSTEST_EnRolle_IngenAut_Clausen.p12", password);
			case UNAUTHORIZED_WITH_NO_NATIONAL_ROLE :
				return TestUtil.getVaultFromResource("oces3/MOCES_STSTEST_IngenRoller_IngenAut_Hansen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES:
				return TestUtil.getVaultFromResource("oces3/NSTSStreAtre_Hansen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_3_NATIONAL_ROLES_2_SAME_EDU_CODES:
				return TestUtil.getVaultFromResource("oces3/NSTSStreArteTOENSUDD_Nielsen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_NATIONAL_ROLE:
				return TestUtil.getVaultFromResource("oces3/NSTSSenAtre_Jensen.p12", password);
			case MITID_AUTHORIZATIONS_3_WITH_NO_NATIONAL_ROLE_TRUSTED_CVR:
				return TestUtil.getVaultFromResource("oces3/NSTSSnullAtre_Larsen.p12", password);
			case MITID_AUTHORIZED_WITH_NO_NATIONAL_ROLE:
				return TestUtil.getVaultFromResource("oces3/MOCES_STSTEST_IngenRoller_EnAut_Jensen.p12", password);
			case MITID_UNAUTHORIZED_WITH_3_NATIONAL_ROLES:
				return TestUtil.getVaultFromResource("oces3/MOCES_STSTEST_TreRoller_IngenAut_Mortensen.p12", password);
			case NON_GLOBAL_MOCES:
				return TestUtil.getVaultFromResource("oces3/NonGlobal_MOCES.p12", password);
			default :
				throw new RuntimeException(key + " er ikke implementeret i " + getClass().getName());
		}
	}
}
