package dk.sosi.sts.server.test;

import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.sts.certificate.OcesInfoParser;
import dk.sosi.sts.model.Authorization;
import dk.sosi.sts.model.RoleDefinition;

import java.util.Collections;
import java.util.List;

public class TestIdentity {
    private GenericCredentialVault credentialVault;
    private final String subjectSerialNumber;
    private final List<Authorization> authorizations;
    private final List<RoleDefinition> nationalRoles;
    private final String cpr;
    private final String cvr;
    private final String givenName;
    private final String surName;

    public TestIdentity(GenericCredentialVault credentialVault, List<Authorization> authorizations, List<RoleDefinition> nationalRoles, String cpr, String cvr, String givenName, String surName) {
        this(ssnFromVault(credentialVault), authorizations, nationalRoles, cpr, cvr, givenName, surName);
        this.credentialVault = credentialVault;
    }

    public TestIdentity(String subjectSerialNumber, List<Authorization> authorizations, List<RoleDefinition> nationalRoles, String cpr, String cvr, String givenName, String surName) {
        this.subjectSerialNumber = subjectSerialNumber;
        this.authorizations = authorizations != null ? authorizations : Collections.emptyList();
        this.nationalRoles = nationalRoles != null ? nationalRoles : Collections.emptyList();
        this.cpr = cpr;
        this.cvr = cvr;
        this.givenName = givenName;
        this.surName = surName;
    }

    public GenericCredentialVault getCredentialVault() {
        return credentialVault;
    }

    public String getSubjectSerialNumber() {
        return subjectSerialNumber;
    }

    public List<Authorization> getAuthorizations() {
        return authorizations;
    }

    public List<RoleDefinition> getNationalRoles() {
        return nationalRoles;
    }

    public String getCpr() {
        return cpr;
    }

    public String getCvr() {
        return cvr;
    }

    public String getGivenName() {
        return givenName;
    }

    public String getSurName() {
        return surName;
    }

    private static String ssnFromVault(GenericCredentialVault vault) {
        return OcesInfoParser.parseCertificate(vault.getSystemCredentialPair().getCertificate()).getSubjectSerialNumber().getValue();
    }
}
