package dk.sosi.sts.server.test;

import dk.sosi.sts.certificate.OcesInfo;
import dk.sosi.sts.model.Authorization;
import dk.sosi.sts.server.facade.RoleDefinitionServiceStub;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static dk.sosi.sts.certificate.OcesInfo.Version.OCES3;

public class TestIdentityProvider {

    // OCES3
    private static final TestIdentity NONGLOBAL_MOCES3 = new TestIdentity(StsCredentialVaults.getNonGlobalMoces(OCES3), null, null,
            "1502816079", "99900499", "Thormund", "Nissen");
    private static final TestIdentity ONE_AUTH_TRUSTED_CVR_MOCES3 = new TestIdentity(SealPPCredentialVaults.getValidMocesWithCpr(OCES3),
            Collections.singletonList(new Authorization("KT2Z4", "7170")), null,
            "0306894781", "91026150", "Thormund", "Nissen");
    private static final TestIdentity THREE_AUTH_TRUSTED_CVR_MOCES3 = new TestIdentity(StsCredentialVaults.getMitIDAuthorizations3WithNoNationalRoleTrustedCvr(OCES3),
            Arrays.asList(new Authorization("2VSRX", "7170"), new Authorization("SQFFX", "5175"), new Authorization("0CNCG", "5155")), null,
            "1206629661", "91026150", "NSTSSnullAtre", "Larsen");
    private static final TestIdentity ONE_AUTH_MOCES3 = new TestIdentity(StsCredentialVaults.getMitIDAuthorizedWithNoNationalRole(OCES3),
            Collections.singletonList(new Authorization("006NX", "7170")),
            null,
            "0909744321", "94354969", "NSTSSnullAen", "Jensen");
    private static final TestIdentity THREE_AUTH_ONE_ROLE_MOCES3 = new TestIdentity(StsCredentialVaults.getMitIDAuthorizations3WithNationalRole(OCES3),
            Arrays.asList(new Authorization("8PG2H", "7170"), new Authorization("HPDWZ", "5151"), new Authorization("JM7VX", "5158")),
            Collections.singletonList(RoleDefinitionServiceStub.PLEJE),
            "0606559739", "25450442", "NSTSSenAtre", "Jensen");
    private static final TestIdentity NO_AUTH_NO_ROLE_MOCES3 = new TestIdentity(StsCredentialVaults.getUnauthorizedWithNoNationalRole(OCES3),null,null,
            "2409017870", "94354969", "Hanne", "Hansen");
    private static final TestIdentity THREE_AUTH_SAME_UDD_CODE_THREE_ROLES_MOCES3 = new TestIdentity(StsCredentialVaults.getMitIDAuthorizations3With3NationalRoles2SameEduCodes(OCES3),
            Arrays.asList(new Authorization("X87JM", "7170"), new Authorization("22DLD", "7170"), new Authorization("03DZN", "5166")),
            Arrays.asList(RoleDefinitionServiceStub.PLEJE, RoleDefinitionServiceStub.PORTOER, RoleDefinitionServiceStub.LAEGE_SEKRETAER),
            "0106779791", "25450442", "NSTSStreArteTOENSUDD", "Nielsen");
    private static final TestIdentity THREE_AUTH_THREE_ROLES_MOCES3 = new TestIdentity(StsCredentialVaults.getMitIDAuthorizations3With3NationalRoles(OCES3),
            Arrays.asList(new Authorization("MCH48", "7170"), new Authorization("BRBJL", "5159"), new Authorization("K7TJC", "5175")),
            Arrays.asList(RoleDefinitionServiceStub.PLEJE, RoleDefinitionServiceStub.PORTOER, RoleDefinitionServiceStub.LAEGE_SEKRETAER),
            "0206994029", "25450442", "NSTSStreAtre", "Hansen");
    private static final TestIdentity ONE_ROLE_MOCES3 = new TestIdentity(StsCredentialVaults.getUnauthorizedWithNationalRole(OCES3),
            null,
            Collections.singletonList(RoleDefinitionServiceStub.PLEJE),
            "1111901113", "94354969", "Claus", "Clausen");
    private static final TestIdentity THREE_ROLE_MOCES3 = new TestIdentity(StsCredentialVaults.getMitIDUnauthorizedWith3NationalRoles(OCES3),
            null,
            Arrays.asList(RoleDefinitionServiceStub.PLEJE, RoleDefinitionServiceStub.PORTOER, RoleDefinitionServiceStub.LAEGE_SEKRETAER),
            "1111901115", "94354969", "Morten", "Mortensen");
    private static final TestIdentity NO_AUTH_ONE_ROLE_CVRRID_MOCES3 = new TestIdentity("UI:DK-E:G:53d4ff1d-836e-4f59-bda2-2cbfe6ce448c", null,
            null, // SSN maps to rid 39214740 and ssn 'CVR:25450442-RID:39214740' has role RoleDefinitionServiceStub.PORTOER
            "1010101235", "25450442", "Thusnelda Truls", "Mathiasen");


    public static final List<TestIdentity> MOCES3_IDENTITIES = Arrays.asList(NONGLOBAL_MOCES3, ONE_AUTH_TRUSTED_CVR_MOCES3, THREE_AUTH_TRUSTED_CVR_MOCES3, ONE_AUTH_MOCES3,
            THREE_AUTH_ONE_ROLE_MOCES3, NO_AUTH_NO_ROLE_MOCES3, THREE_AUTH_SAME_UDD_CODE_THREE_ROLES_MOCES3, THREE_AUTH_THREE_ROLES_MOCES3, ONE_ROLE_MOCES3, THREE_ROLE_MOCES3, NO_AUTH_ONE_ROLE_CVRRID_MOCES3);
    public static final List<TestIdentity> IDENTITIES = MOCES3_IDENTITIES.stream().collect(Collectors.toList());
    public static TestIdentity getNonGlobalMoces3Identity() {
        return NONGLOBAL_MOCES3;
    }

    public static TestIdentity getIdentityWithTrustedCvrAndOneAuthorization(OcesInfo.Version version) {
        return ONE_AUTH_TRUSTED_CVR_MOCES3;
    }

    public static TestIdentity getIdentityWithTrustedCvrAndThreeAuthorizations(OcesInfo.Version version) {
        return THREE_AUTH_TRUSTED_CVR_MOCES3;
    }

    public static TestIdentity getIdentityWithOneAuthorization(OcesInfo.Version version) {
        return ONE_AUTH_MOCES3;
    }

    public static TestIdentity getIdentityWithThreeAuthorizationsAndOneNationalRole(OcesInfo.Version version) {
        return THREE_AUTH_ONE_ROLE_MOCES3;
    }

    public static TestIdentity getIdentityWithNoAuthorizationsOrNationalRoles(OcesInfo.Version version) {
        return NO_AUTH_NO_ROLE_MOCES3;
    }

    public static TestIdentity getIdentityWithThreeAuthorizationsWithTwoMatchingUddCodesAndThreeNationalRoles(OcesInfo.Version version) {
        return THREE_AUTH_SAME_UDD_CODE_THREE_ROLES_MOCES3;
    }

    public static TestIdentity getIdentityWithThreeAuthorizationsAndThreeNationalRoles(OcesInfo.Version version) {
        return THREE_AUTH_THREE_ROLES_MOCES3;
    }

    public static TestIdentity getIdentityWithOneNationalRole(OcesInfo.Version version) {
        return ONE_ROLE_MOCES3;
    }

    public static TestIdentity getIdentityWithThreeNationalRoles(OcesInfo.Version version) {
        return THREE_ROLE_MOCES3;
    }

    public static TestIdentity getMoces3IdentityWithNoAuthorizationAndOneNationalRoleForMappedSsn() {
        return NO_AUTH_ONE_ROLE_CVRRID_MOCES3;
    }
}
