/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.sts.server.timer;

import java.util.Timer;
import java.util.TimerTask;

import org.mockito.Mockito;

import junit.framework.TestCase;

/**
 *
 */
public class AddStsTimerTest extends TestCase {

	public void testSetupOneTime() throws Exception {

		final AddStsTimer addStsTimer = new AddStsTimer();

		final Timer mock = Mockito.mock(Timer.class);
		addStsTimer.setTimer(mock);

		final TimerTask timerTask = new TimerTask() {
			@Override
			public void run() {

			}
		};
		addStsTimer.setTimerTask(timerTask);
		addStsTimer.setDelay(42);
		addStsTimer.setOneTime(true);
		addStsTimer.afterPropertiesSet();

		Mockito.verify(mock).schedule(timerTask, 42);

	}

	public void testSetupTimer() throws Exception {

		final AddStsTimer addStsTimer = new AddStsTimer();

		final Timer mock = Mockito.mock(Timer.class);
		addStsTimer.setTimer(mock);

		final TimerTask timerTask = new TimerTask() {
			@Override
			public void run() {

			}
		};
		addStsTimer.setTimerTask(timerTask);
		addStsTimer.setDelay(42);
		addStsTimer.setPeriod(42);
		addStsTimer.afterPropertiesSet();

		Mockito.verify(mock).scheduleAtFixedRate(timerTask, 42, 42);
	}
}
